/*
 * Authenticated User Community
 * Copyright (c) 2000 David C. Moore
 *
 * You may distribute and/or use for any purpose modified or unmodified
 * copies of this software if you preserve the copyright notice above.
 *
 * auth.c
 *
 * Routines governing access control, authentication, and user/group
 * maintenance in AUC
 *
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <auc.h>
#include <stdlib.h>
#include <pwd.h>
#include <string.h>
#include <errno.h>

#ifdef HAVE_LIBPAM
#include <security/pam_appl.h>
#endif

#include <sys/types.h>

#define _XOPEN_SOURCE
#include <unistd.h>

#include <crypt.h>

#include <auc/auc.h>
#include <auc/cgic.h>
#include <auc/files.h>
#include <auc/error.h>
#include <auc/util.h>
#include <auc/pages.h>
#include <auc/database.h>
#include <auc/html.h>


int usedbpasswd(char *);
int checkdbpasswd(char *, char *);
int store_groups(aucAuthCachePtr cache, int maxgrps);


struct chpass
{
	char *old, *new;
};

char *aucDirectoryGrps = "staff,student,visitor";

char *permtypes[] = {
	"root",
	"sysop",
	"normal",
	"guest",
	NULL
};

#ifdef HAVE_LIBPAM

struct pam_response *resp_data;

int
auc_conv(int num_msg, const struct pam_message **msg, struct pam_response **resp, void *appdata_ptr)
{
	resp_data = (struct pam_response *) calloc(num_msg, sizeof(struct pam_response));
	*resp = resp_data;
	resp_data->resp = appdata_ptr;
	resp_data->resp_retcode = 0;
	return (PAM_SUCCESS);
}

int
auc_conv_chpass(int num_msg, const struct pam_message **msg,
				struct pam_response **resp, void *appdata_ptr)
{
	resp_data = (struct pam_response *) calloc(num_msg, sizeof(struct pam_response));
	*resp = resp_data;
	if (!strncmp(msg[0]->msg, "(current)", 9))
		resp_data->resp = (char *) strdup(((struct chpass *) appdata_ptr)->old);
	else {
		resp_data->resp = (char *) strdup(((struct chpass *) appdata_ptr)->new);
		(resp_data + 1)->resp = (char *) strdup(((struct chpass *) appdata_ptr)->new);
	}
	resp_data->resp_retcode = 0;
	return (PAM_SUCCESS);
}

#endif /* HAVE_LIBPAM */

int
getpermissions(aucAuthCachePtr cache, char *uname)
{
	MYSQL_RES *res;
	MYSQL_ROW row;
	char buffer[1024];
	int result = 0;

	if (!strcmp(uname, aucDBUser)) {
		snprintf(buffer, sizeof(buffer), "INSERT INTO userinfo (uname, fname, \
         lname, mname, id, grp, class, permissions) VALUES (\"%s\", \"%s\", \
         \"%s\", \"%s\", \"%s\", \"%s\", %s, \"%s\")", aucDBUser, "System", "Admin", "", "", "system", "NULL", "sysop");
		mysql_query(db, buffer);
	}

	snprintf(buffer, sizeof(buffer),
			 "SELECT permissions, grp, class, id FROM userinfo WHERE uname = \"%s\"", uname);
	if (mysql_query(db, buffer))
		aucError(mysql_error(db));
	else {
		res = mysql_store_result(db);
		cache->class = 0;
		cache->id = 0;
		if (mysql_num_rows(res)) {
			row = mysql_fetch_row(res);
			snprintf(cache->permissions, sizeof(cache->permissions), "%s", row[0]);
			snprintf(cache->role, sizeof(cache->role), "%s", row[1]);
			if (row[2])
				cache->class = strtoul(row[2], NULL, 10);
			if (row[3])
				cache->id = strtoul(row[3], NULL, 10);
			result = 1;
		}
		mysql_free_result(res);
	}
	return result;
}

aucAuthCachePtr
aucAuthNewCache(void)
{
	aucAuthCachePtr buf;

	buf = (aucAuthCachePtr) malloc(sizeof(aucAuthCache));
	buf->uid = buf->gid = -1;
	buf->login = buf->last_request = 0;
	return (buf);
}

void
aucAuthFreeCache(aucAuthCachePtr cache)
{
	if (cache) {
		free(cache);
	}
}

void
aucAuthWriteCache(aucAuthCachePtr cache, char *uname)
{
	char buf[1024];
	FILE *outfile;

	aucSuRoot();
	
	snprintf(buf, sizeof(buf), "%s/cache", aucVarRoot);
	aucCheckDir(buf, 0755, 0, 0);
	snprintf(buf, sizeof(buf), "%s/cache/%s", aucVarRoot, uname);
	if ((outfile = fopen(buf, "w"))) {
		if (fwrite(cache, sizeof(aucAuthCache), 1, outfile)) {
			fclose(outfile);
			aucSuUser();
			return;
		} else
			fclose(outfile);
	}
	aucError("%s: %s", _("ERROR: could not write cache file"), strerror(errno));
	aucSuUser();
}

int
aucAuthReadCache(aucAuthCachePtr cache, char *uname)
{
	char buf[1024];
	FILE *infile;

	snprintf(buf, sizeof(buf), "%s/cache/%s", aucVarRoot, uname);
	if ((infile = fopen(buf, "r"))) {
		if (fread(cache, sizeof(aucAuthCache), 1, infile)) {
			fclose(infile);
			return (1);
		}
		aucError(_("LOGIN ERROR: user cache file appears to be corrupted"));
		fclose(infile);
		return (0);
	}
	return (0);
}

void
aucAuthClearCache(char *uname)
{
	char buf[1024];

	snprintf(buf, sizeof(buf), "%s/cache/%s", aucVarRoot, uname);
	unlink(buf);
}

int
passwordchange(aucAuthCachePtr cache)
{
	char oldpass[17], newpass[17], newpass2[17], buf[1024];
	struct chpass appdata;
#ifdef HAVE_LIBPAM
	pam_handle_t *pamh = NULL;
	struct pam_conv conv;

	conv.conv = auc_conv_chpass;
#endif

	strcpy(oldpass, "");
	strcpy(newpass, "");
	strcpy(newpass2, "");
	cgiFormStringNoNewlines("oldpass", oldpass, sizeof(oldpass));
	cgiFormStringNoNewlines("newpass", newpass, sizeof(newpass));
	cgiFormStringNoNewlines("newpass2", newpass2, sizeof(newpass2));

	if ((strlen(oldpass) + strlen(newpass) + strlen(newpass2)) == 0)
		return 0;

	if (strcmp(oldpass, cache->passwd))
		return -1;

	if (strcmp(newpass, newpass2))
		return -2;

	if (!strcmp(oldpass, newpass))
		return -3;

	strcpy(cache->passwd, newpass);
	aucAuthScramblePasswd(buf, sizeof(buf), cache->passwd);
	cgiSendCookie("key", buf, aucPagesRoot, 0);

	if (cache->authtype == AUTH_TYPE_DBPASSWD) {
		snprintf(buf, sizeof(buf), "REPLACE INTO passwd (uname, passwd) \
         VALUES (\"%s\", PASSWORD(\"%s\"))", cache->uname, cache->passwd);
		if (mysql_query(db, buf)) {
			aucError(mysql_error(db));
			return -4;
		}
		snprintf(buf, sizeof(buf), "User %s changed password (database)", cache->uname);
		aucError(buf);
	} else if (cache->authtype == AUTH_TYPE_PAM) {
#ifdef HAVE_LIBPAM
		appdata.old = (char *) oldpass;
		appdata.new = (char *) cache->passwd;
		conv.appdata_ptr = (void *) &appdata;
		if (pam_start("auc", cache->uname, &conv, &pamh) != PAM_SUCCESS)
			return -4;
		if (pam_chauthtok(pamh, PAM_SILENT) != PAM_SUCCESS)
			return -4;
		if (pam_end(pamh, PAM_SUCCESS) != PAM_SUCCESS)
			pamh = NULL;

		snprintf(buf, sizeof(buf), "User %s changed password (PAM)", cache->uname);
		aucError(buf);

#else /* HAVE_LIBPAM */
		return -4;
#endif /* HAVE_LIBPAM */
	}
	return 1;
}

char *
gettemphomedir(char *uname, uid_t uid, gid_t gid)
{
	char buf[1024];

	snprintf(buf, sizeof(buf), "%s/home", aucVarRoot);
	aucCheckDir(buf, 0755, 0, 0);
	snprintf(buf, sizeof(buf), "%s/home/%s", aucVarRoot, uname);
	aucCheckDir(buf, 0700, uid, gid);
	return ((char *) strdup(buf));
}

int
aucAuthCheck(aucAuthCachePtr cache)
{
	MYSQL *tempdb = NULL;
	struct passwd *pw_ent;
	int result, authtype, fresh = 0;
	char password[17], uname[65];
	char buffer[128];
#ifdef HAVE_LIBPAM
	pam_handle_t *pamh = NULL;
	struct pam_conv conv;

	conv.conv = auc_conv;
#endif

	if (cgiFormSuccess == cgiFormStringNoNewlines("front", buffer, sizeof(buffer)))
		fresh = 1;

	if (cgiFormSuccess != cgiFormStringNoNewlines("uname", uname, sizeof(uname)))
		return (AUTH_NONE);

	aucSanitizeUsername(uname);

	strcpy(password, "");
	strcpy(buffer, "");
	cgiFormStringNoNewlines("key", buffer, sizeof(buffer));
	if (!fresh)
		aucAuthUnscramblePasswd(password, sizeof(password), buffer);
	else
		snprintf(password, sizeof(password), buffer);


	tempdb = mysql_init(tempdb);
//	aucError("%s %s %s %s", aucDBHost, uname, password, aucDBName);
	if (mysql_real_connect(tempdb, aucDBHost, uname, password, aucDBName, 0, NULL, 0)) {
		authtype = AUTH_TYPE_DATABASE;
		
		if (!strlen(password)) {
			aucError("LOGIN WARNING: The MySQL authentication database contains users with empty passwords.  These users should be removed to maintain AUC and MySQL security.");
			mysql_close(tempdb);
			return AUTH_INVALID;
		}
	} else if (usedbpasswd(uname))
		authtype = AUTH_TYPE_DBPASSWD;
	else
		authtype = AUTH_TYPE_PAM;
	mysql_close(tempdb);


	if (!fresh && aucAuthReadCache(cache, uname)) {
		if (strcmp(cache->passwd, password))
			return (AUTH_INVALID);
		if ((aucTime - cache->last_request) > aucMaxIdleSecs) {
			aucAuthClearCache(uname);
			return (AUTH_EXPIRED);
		}
		aucChPasswdStatus = passwordchange(cache);
		cache->last_request = aucTime;
	} else if (fresh) {

		if (authtype == AUTH_TYPE_DBPASSWD) {
			if (!checkdbpasswd(uname, password)) {
				aucError("LOGIN FAILED: %s from %s: DB password incorrect",
						 uname, strlen(cgiRemoteHost) ? cgiRemoteHost : cgiRemoteAddr);
				return (AUTH_INVALID);
			}

		} else if (authtype == AUTH_TYPE_PAM) {
			char *newuname;

#ifdef HAVE_LIBPAM
			conv.appdata_ptr = (char *) strdup(password);
			result = pam_start("auc", uname, &conv, &pamh);
			if (result == PAM_SUCCESS) {
				result = pam_set_item(pamh, PAM_RHOST, cgiRemoteHost);
				result = pam_authenticate(pamh, PAM_SILENT);
			}
			if (result == PAM_SUCCESS) {
				result = pam_get_item(pamh, PAM_USER, (const void **) &newuname);
				if (result == PAM_SUCCESS) {
					strcpy(uname, newuname);
					strcpy(cache->uname, newuname);
				}
				result = pam_acct_mgmt(pamh, PAM_SILENT);
			}
			if (result == PAM_SUCCESS) {
				result = pam_setcred(pamh, PAM_ESTABLISH_CRED);
			}
			if (pam_end(pamh, result) != PAM_SUCCESS)
				pamh = NULL;

			if (result != PAM_SUCCESS) {
				snprintf(buffer, sizeof(buffer), _("LOGIN FAILED: %s from %s: PAM: %s"),
						 uname, strlen(cgiRemoteHost) ? cgiRemoteHost : cgiRemoteAddr,
						 pam_strerror(pamh, result));
				aucError(buffer);
				return (AUTH_INVALID);
			}
#else /* HAVE_LIBPAM */
			return (AUTH_INVALID);
#endif /* HAVE_LIBPAM */
		}

		snprintf(buffer, sizeof(buffer), _("LOGIN: %s authenticated from %s"),
				 uname, strlen(cgiRemoteHost) ? cgiRemoteHost : cgiRemoteAddr);
		aucError(buffer);

		if (!getpermissions(cache, uname)) {
			snprintf(buffer, sizeof(buffer),
					 _("LOGIN ERROR: userlist entry not found for %s"), uname);
			aucError(buffer);
			return (AUTH_INVALID);
		}

	} else
		return (AUTH_INVALID);

	if (fresh) {
		strcpy(cache->uname, uname);
		strcpy(cache->passwd, password);
		strcpy(cache->homedir, "");
		cache->local = 0;
		cache->authtype = authtype;

		if (aucAllowLocal && (pw_ent = getpwnam(cache->uname))) {
			cache->uid = pw_ent->pw_uid;
			cache->gid = pw_ent->pw_gid;
			if (authtype != AUTH_TYPE_DATABASE) {
				cache->local = 1;
				snprintf(cache->homedir, sizeof(cache->homedir), "%s", pw_ent->pw_dir);
			}
		} else {
			cache->uid = getuid();
			cache->gid = getgid();
		}
		store_groups(cache, MAXGRPS);
		aucAuthGetFullName(cache->fullname, sizeof(cache->fullname), cache->uname, 0);
		cache->login = aucTime;
		cache->last_request = aucTime;
		if (!cache->local)
			snprintf(cache->homedir, sizeof(cache->homedir), "%s",
					 gettemphomedir(uname, cache->uid, cache->gid));
		aucDBQuery(db, "UPDATE userinfo SET lastlogin=NOW() WHERE uname=\"%s\"", cache->uname);
	}

	aucChownUploadedFiles(cache->uid, cache->gid);

	return (AUTH_VALID);
}


void
aucAuthLoginPage(char *message)
{
	char buffer[1024];

	fprintf(cgiOut, "<form name=\"auth\" method=post action=\"%s/logincheck\">\n", aucPagesRoot);
	fprintf(cgiOut, "<p><b>%s</b></p>\n", message);

	aucGetFullPath("login-panel.html", aucDataDir, buffer, sizeof(buffer));
	aucHtmlEchoFile(cgiOut, buffer, 0);

	fprintf(cgiOut, "<input type=hidden name=front value=yes>\n");
	if (cgiFormSuccess == cgiFormStringNoNewlines("url", buffer, sizeof(buffer)))
		fprintf(cgiOut, "<input type=hidden name=url value=\"%s\">\n", buffer);
	else if ((strlen(cgiPathInfo) > 1) && strlen(cgiQueryString))
		fprintf(cgiOut, "<input type=hidden name=url value=\".%s?%s\">\n", cgiPathInfo,
				cgiQueryString);
	else if (strlen(cgiPathInfo) > 1)
		fprintf(cgiOut, "<input type=hidden name=url value=\".%s\">\n", cgiPathInfo);
	else
		fprintf(cgiOut, "<input type=hidden name=url value=\"home\">\n");

	fprintf(cgiOut, "</form>\n");

	aucGetFullPath("login-mess.html", aucDataDir, buffer, sizeof(buffer));
	aucHtmlEchoFile(cgiOut, buffer, 0);

}

int
usedbpasswd(char *uname)
{
	char buf[1024];
	int result = 0;

	snprintf(buf, sizeof(buf), "uname = \"%s\"", uname);
	result = aucDBNumRows("passwd", buf);
	return result;
}

int
checkdbpasswd(char *uname, char *password)
{
	MYSQL_RES *res;
	MYSQL_ROW row;
	char buf[1024];
	int result = 0;

	snprintf(buf, sizeof(buf), "NOT STRCMP(uname,\"%s\") AND NOT STRCMP(passwd,PASSWORD(\"%s\"))", uname, password);
	result = aucDBNumRows("passwd", buf);
	if (!result) {
		if (!aucDBQuery(db, "SELECT passwd from passwd WHERE NOT STRCMP(uname,\"%s\")", uname)) {
			res = mysql_store_result(db);
			if ((row = mysql_fetch_row(res))) {
				if (!strcmp(row[0], crypt(password, row[0])))
					result = 1;
			}
			mysql_free_result(res);
		}
	}
	return result;
}

int
aucAuthPermsof(char *authlevel)
{
	char **a;

	if (!authlevel || !strcmp(aucUserCache->permissions, authlevel))
		return (1);
	for (a = permtypes; strcmp(*a, authlevel) && *a; a++) {
		if (!strcmp(*a, aucUserCache->permissions))
			return (1);
	}
	return (0);
}

void
aucAuthScramblePasswd(char *scrambled, size_t len, char *unscrambled)
{
	char *a, *b;

	if (len >= (strlen(unscrambled) * 2 + 1))
		for (a = scrambled, b = unscrambled; *b; a += 2, b++)
			sprintf(a, "%2X", *b);
	else
		strcpy(scrambled, "");
}

void
aucAuthUnscramblePasswd(char *unscrambled, size_t len, char *scrambled)
{
	char *a, *b;
	int t;

	if (len >= (strlen(scrambled) / 2 + 1)) {
		for (a = unscrambled, b = scrambled; *b; a++, b += 2) {
			sscanf(b, "%2x", &t);
			*a = (char) t;
		}
		*a = '\0';
	} else
		strcpy(unscrambled, "");
}

void
check_group(FILE * output, char *group)
{
	char buf[512];

	snprintf(buf, sizeof(buf), "grp = \"%s\"", group);

	if (!aucDBNumRows("groups", buf)) {
		snprintf(buf, sizeof(buf), "INSERT INTO groups VALUES (\"%s\")", group);
		if (mysql_query(db, buf))
			aucError(mysql_error(db));
	}
}

int
aucPageSetPasswd(FILE * output, char *extrapath)
{
	char buf[1024], *fields[2];

	fields[0] = fields[1] = NULL;

	if (!strcmp(cgiRequestMethod, "POST")) {
		aucHtmlReadForm("username password", fields, STR_ESCAPE);
		snprintf(buf, sizeof(buf), "REPLACE INTO passwd (uname, passwd) \
         VALUES (\"%s\", PASSWORD(\"%s\"))", fields[0], fields[1]);
		mysql_query(db, buf);
		cgiHeaderLocation("account-manager");
		return 0;
	}

	aucHtmlReadForm("username", fields, 0);

	cgiStaticContentType("text/html");

	if (!strlen(fields[0]))
		return -1;

	aucHtmlSlimHead("Reset Password");

	fprintf(output, "<p>%s</p>", _("Type a new password for the user and \
press \"Store Password\" to save it in the database.  This will change the \
password stored in the SQL database, not a traditional unix password."));

	aucDrawTableForm(output, "post", "set-passwd");
	aucDrawHidden(output, "username", fields[0]);
	fprintf(output, "<tr><td>%s:</td><td><b>%s</b></td></tr>\n", _("Username"), fields[0]);
	aucDrawTextbox(output, "password", 16, 16, NULL, _("New Password"));
	aucDrawSubmit(output, _("Store Password"));
	aucDrawTableFormEnd(output);

	aucHtmlSlimFoot();
	return 0;
}

int
aucPageAccountManager(FILE * output, char *extrapath)
{
	char buffer[1024], sort[1024], *aucClasses[64], **a;
	MYSQL_RES *res;
	MYSQL_ROW row;
	int x, count;

	aucSortPattern sorts[] = {
		{"lname", "ORDER BY lname ASC, fname ASC", SORT_ALPHA, 0, NULL, NULL},
		{"userinfo.uname", "ORDER BY userinfo.uname ASC", SORT_ALPHA, 0, NULL, NULL},
		{"id", "ORDER BY id ASC", SORT_PAGES, 100, NULL, NULL},
		{"grp", "ORDER BY grp ASC, class ASC, lname ASC, fname ASC",
		 SORT_CUSTOM, 0, aucClasses, aucClasses},
		{"permissions", "ORDER BY permissions ASC, lname ASC, fname ASC",
		 SORT_CUSTOM, 0, permtypes, permtypes},
		{NULL}
	};

	if (mysql_query(db, "SELECT DISTINCT class from userinfo \
         WHERE grp=\"student\" ORDER BY class ASC"))
		aucError(mysql_error(db));
	else {
		res = mysql_store_result(db);
		a = aucClasses;
		for (x = 0; aucRoles[x]; x++)
			if (strcmp(aucRoles[x], "student"))
				*a++ = aucRoles[x];
		for (; (row = mysql_fetch_row(res)); a++)
			if (row[0] && strlen(row[0]))
				*a = (char *) strdup(row[0]);
			else
				*a = "other";
		*a = NULL;
		mysql_free_result(res);
	}

	aucHtmlSetSortCookies(output, "account-manager");

	cgiHeaderContentType("text/html");

	aucHtmlSlimHead("%s", _("User Accounts Manager"));

	fprintf(output, "<table border=0 cellpadding=0 cellspacing=0 width=100%%>\n");
	fprintf(output, "<tr valign=top><td>\n\n");

	if (mysql_query(db, "SELECT COUNT(*) from userinfo"))
		aucError(mysql_error(db));
	else {
		res = mysql_store_result(db);
		row = mysql_fetch_row(res);
		aucHtmlProcessSort(output, sort, sizeof(sort), "account-manager",
					   strtoul(row[0], NULL, 10), sorts, "grp");
		mysql_free_result(res);
	}

	fprintf(output, "</td><td align=right>\n\n");

	fprintf(output, "<p>%s</p>\n", _("For authorized users only"));

	fprintf(output, "<b><a href=\"account-edit\">%s</a></b>\n", _("Create New Account"));
	fprintf(output, "<br><b><a href=\"account-bulk-add\">%s</a></b>\n", _("Add Accounts in Bulk"));

	fprintf(output, "</td></tr></table>\n\n");

	snprintf(buffer, sizeof(buffer), "SELECT userinfo.uname, fname, lname, id, grp, class, \
		permissions, lastlogin, passwd.uname FROM userinfo LEFT OUTER JOIN \
      passwd on userinfo.uname=passwd.uname %s", sort);
	if (mysql_query(db, buffer))
		aucError(mysql_error(db));
	else {
		res = mysql_store_result(db);

		fprintf(output, "<p><table cellpadding=2 cellspacing=0 border=0 width=100%%>\n");
		fprintf(output, "<tr bgcolor=%s>\n", aucTrimColor);
		fprintf(output, "<td><font size=-1 color=#FFFFFF><b>%s</b></font></td>\n", _("Username"));
		fprintf(output, "<td><font size=-1 color=#FFFFFF><b>%s</b></font></td>\n", _("First"));
		fprintf(output, "<td><font size=-1 color=#FFFFFF><b>%s</b></font></td>\n", _("Last"));
		fprintf(output, "<td><font size=-1 color=#FFFFFF><b>%s</b></font></td>\n", _("ID"));
		fprintf(output, "<td><font size=-1 color=#FFFFFF><b>%s</b></font></td>\n", _("Role"));
		fprintf(output, "<td><font size=-1 color=#FFFFFF><b>%s</b></font></td>\n", _("GradYr"));
		fprintf(output, "<td><font size=-1 color=#FFFFFF><b>%s</b></font></td>\n",
				_("Permissions"));
		fprintf(output, "<td><font size=-1 color=#FFFFFF><b>%s</b></font></td>\n", _("LastLogin"));
		fprintf(output, "<td><font size=-1 color=#FFFFFF><b>%s</b></font></td>\n", _("Local?"));
		fprintf(output, "<td><font size=-1 color=#FFFFFF><b>%s</b></font></td>\n", _("Options"));
		fprintf(output, "</tr>\n");

		count = 0;
		while ((row = mysql_fetch_row(res))) {
			fprintf(output, "<tr%s>\n", (count % 2) ? " bgcolor=#EEEEEE" : "");
			for (x = 0; x < 10; x++)
				if (x == 0)
					fprintf(output, "<td><small><a href=\"users/%s\">%s</a></small></td>\n",
							row[x], row[x]);
				else if (x == 8)
					fprintf(output, "<td><small>%s</small></td>\n",
							getpwnam(row[0]) ? _("Yes") : _("No"));
				else if (x == 9) {
					fprintf(output, "<td><small><a href=\"account-edit?username=%s\">%s</a>, \
                                <a href=\"account-delete?username=%s\">%s</a>, ", row[0], _("Edit"), row[0], _("Del."));
					if (row[8])
						fprintf(output, "<a href=\"set-passwd?username=%s\">%s</a>, ",
								row[0], _("Passwd"));
					fprintf(output, "<a href=\"dirinfo-edit?username=%s\">%s</a></small></td>\n",
							row[0], _("DirInfo"));
				} else
					fprintf(output, "<td><small>%s&nbsp;</small></td>\n", row[x] ? row[x] : "");
			fprintf(output, "</tr>\n");
			count++;
		}

		mysql_free_result(res);

		fprintf(output, "</table></p>\n");
	}

	aucHtmlSlimFoot();
	return 0;
}

int
aucPageBulkAddAccount(FILE * output, char *extrapath)
{
	char buffer[256];
	
	cgiStaticContentType("text/html");
	aucHtmlSlimHead("Bulk Addition of User Accounts");

	aucGetFullPath("account-bulk-add.html", aucDataDir, buffer, sizeof(buffer));
	aucHtmlEchoFile(output, buffer, 0);
	
	aucHtmlSlimFoot();
	return 0;
}

int
aucPageAddAccount(FILE * output, char *extrapath)
{
	MYSQL_RES *res = NULL;
	MYSQL_ROW row = NULL;
	int x, num, edit = 0, nogood = 0, pam = 1;
	char buffer[1024], uname[65], *fields[32];
	char *authtypes[] = { "pam", "db", NULL };
	char *authtext[3];


#ifdef HAVE_LIBPAM
	authtext[0] = (char *) strdup(_("Use PAM to authenticate user"));
	authtext[1] = (char *) strdup(_("Store password in built-in database"));
	authtext[2] = NULL;
#else
	authtext[0] = (char *) strdup(_("Store password in built-in database"));
	authtext[1] = NULL;
	authtypes[0] = "db";
	authtypes[1] = NULL;
	pam = 0;
#endif


	strcpy(uname, "");
	for (x = 0; x < 32; x++)
		fields[x] = NULL;

	if (!strcmp(cgiRequestMethod, "GET")) {
		if (cgiFormSuccess == cgiFormString("username", uname, sizeof(uname))) {
			snprintf(buffer, sizeof(buffer), "SELECT * FROM userinfo LEFT OUTER \
            JOIN passwd on userinfo.uname=passwd.uname WHERE userinfo.uname=\"%s\"", uname);
			if (mysql_query(db, buffer))
				aucError(mysql_error(db));
			else {
				edit = 1;
				res = mysql_store_result(db);
				row = mysql_fetch_row(res);
				for (x = 0, num = mysql_num_fields(res); x < num; x++)
					fields[x] = row[x];
			}
		}
		if (fields[9])
			pam = 0;
	} else {
		aucHtmlReadForm("username fname lname mname id grp class permissions \
         authentication passwd", fields, STR_ESCAPE);
		if (strcmp(fields[8], "pam"))
			pam = 0;
		aucSanitizeUsername(fields[0]);
		snprintf(buffer, sizeof(buffer), "REPLACE INTO userinfo \
         (uname, fname, lname, mname, id, grp, class, permissions) \
         VALUES (\"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", %s, \"%s\")", fields[0], fields[1], fields[2], fields[3], fields[4], fields[5], strlen(fields[6]) ? fields[6] : "NULL", fields[7]);
		if (strlen(fields[0]) && strlen(fields[1]) && strlen(fields[2])) {
			if (mysql_query(db, buffer))
				aucError(mysql_error(db));
			if (pam)
				snprintf(buffer, sizeof(buffer), "DELETE FROM passwd WHERE \
               uname=\"%s\"", fields[0]);
			else
				snprintf(buffer, sizeof(buffer), "INSERT INTO passwd \
               (uname, passwd) VALUES (\"%s\", PASSWORD(\"%s\"))", fields[0], "");
			mysql_query(db, buffer);
			cgiHeaderLocation("account-manager");
			return 0;
		} else
			nogood = 1;
	}

	cgiHeaderContentType("text/html");

	if (edit)
		aucHtmlSlimHead("%s", _("Edit User Entry"));
	else
		aucHtmlSlimHead("%s", _("Add New User Entry"));

	if (nogood)
		fprintf(output, "<p><b>%s:</b> %s</p>\n\n", _("Error"),
				_("You must specify a Username, First Name, and Last Name."));

	aucDrawTableForm(output, "POST", "account-edit");
	if (edit) {
		aucDrawHidden(output, "username", fields[0]);
		fprintf(output, "<tr><td align=right>%s:</td><td><b>%s</b></td></tr>\n\n",
				_("Username"), fields[0]);
	} else
		aucDrawTextbox(output, "username", 16, aucMaxUsernameLen, fields[0], _("Username"));
	aucDrawTextbox(output, "fname", 20, 32, fields[1], _("First Name"));
	aucDrawTextbox(output, "lname", 20, 32, fields[2], _("Last Name"));
	aucDrawTextbox(output, "mname", 20, 32, fields[3], _("Middle Name/Initial"));
	aucDrawTextbox(output, "id", 16, 16, fields[4], _("Student ID"));
	aucDrawDropdown(output, "grp", aucRoles, aucRoles,
					(fields[5]) ? fields[5] : "student", _("Role"));
	aucDrawTextbox(output, "class", 4, 4, fields[6], _("Graduation Year"));
	aucDrawDropdown(output, "permissions", &permtypes[1], &permtypes[1],
					(fields[7]) ? fields[7] : "normal", _("Access Level"));
	aucDrawRadio(output, "authentication", authtypes, authtext,
				 (pam == 1) ? "pam" : "db", _("Authentication"));
	if (edit)
		aucDrawSubmit(output, _("Save Entry"));
	else
		aucDrawSubmit(output, _("Add Entry"));
	aucDrawTableFormEnd(output);
	aucHtmlSlimFoot();

	if (edit)
		mysql_free_result(res);

	if (strcmp(cgiRequestMethod, "GET"))
		for (x = 0; fields[x]; x++)
			free(fields[x]);


	return 0;
}

int
aucPageDeleteAccount(FILE * output, char *extrapath)
{
	char uname[65], fullname[64], buf[512], force[8];
	int nogood = 0;

	if (cgiFormSuccess != cgiFormString("username", uname, sizeof(uname)))
		nogood = 1;

	if (nogood) {
		cgiHeaderLocation("account-manager");
		return -1;
	}

	if (cgiFormSuccess != cgiFormString("force", force, sizeof(force))) {
		aucAuthGetFullName(fullname, sizeof(fullname), uname, 0);
		cgiHeaderContentType("text/html");
		aucHtmlSlimHead("%s", _("Confirmation"));
		fprintf(output, "<p>%s <b>%s</b> (%s)?</p>\n",
				_("Are you sure you want to delete user"), fullname, uname);
		fprintf(output,
				"<ul><p><b><a href=\"account-delete?force=yes&username=%s\">%s</a></b></p>\n",
				uname, ("Yes"));
		fprintf(output, "<p><b><a href=\"account-manager\">%s</a></b></p></ul>\n", _("No"));
		aucHtmlSlimFoot();
		return 0;
	}

	snprintf(buf, sizeof(buf), "DELETE FROM userinfo WHERE uname=\"%s\"", uname);
	if (mysql_query(db, buf))
		aucError(mysql_error(db));
	snprintf(buf, sizeof(buf), "DELETE FROM passwd WHERE uname=\"%s\"", uname);
	if (mysql_query(db, buf))
		aucError(mysql_error(db));
	snprintf(buf, sizeof(buf), "DELETE FROM directory WHERE uname=\"%s\"", uname);
	if (mysql_query(db, buf))
		aucError(mysql_error(db));
	snprintf(buf, sizeof(buf), "DELETE FROM peoplelinks WHERE uname=\"%s\"", uname);
	if (mysql_query(db, buf))
		aucError(mysql_error(db));
	snprintf(buf, sizeof(buf), "DELETE FROM scheds WHERE uname=\"%s\"", uname);
	if (mysql_query(db, buf))
		aucError(mysql_error(db));
	snprintf(buf, sizeof(buf), "DELETE FROM prefs WHERE uname=\"%s\"", uname);
	if (mysql_query(db, buf))
		aucError(mysql_error(db));

	cgiHeaderLocation("account-manager");
	return 0;
}


int
aucPageGroupManager(FILE * output, char *extrapath)
{
	char buffer[1024];
	MYSQL_RES *res, *usrs;
	MYSQL_ROW row, usr;

	cgiHeaderContentType("text/html");

	aucHtmlSlimHead("%s", _("Group Manager"));
	fprintf(output, "<p>%s</p>\n", _("For authorized users only"));

	fprintf(output, "<blockquote><a href=\"group-edit\">%s</a></blockquote>\n",
			_("Create New Group"));

	if (mysql_query(db, "SELECT * FROM groups"))
		aucError(mysql_error(db));
	else {
		res = mysql_store_result(db);

		fprintf(output, "<p><table cellpadding=2 cellspacing=0 border=0 width=100%%>\n");
		fprintf(output, "<tr bgcolor=#CCCCCC>\n");
		fprintf(output, "<td><b>%s</b></td><td><b>%s</b></td><td><b>%s</b></td>\n\n",
				_("groups"), _("users"), _("options"));
		fprintf(output, "</tr>\n");

		while ((row = mysql_fetch_row(res))) {
			fprintf(output, "<tr valign=top>\n");
			fprintf(output, "<td>%s</td>\n", row[0]);
			snprintf(buffer, sizeof(buffer), "SELECT uname FROM groupmembers WHERE grp = \"%s\"",
					 row[0]);
			if (mysql_query(db, buffer))
				aucError(mysql_error(db));
			else {
				usrs = mysql_store_result(db);
				fprintf(output, "<td>\n");
				while ((usr = mysql_fetch_row(usrs)))
					fprintf(output, "%s (<a href=\"group-deluser?group=%s&user=%s\">d</a>)\n",
							usr[0], row[0], usr[0]);
				fprintf(output, "</td>\n<td><a href=\"group-edit?group=%s\">%s</a><br>\n",
						row[0], _("Add User(s)"));
				fprintf(output, "<a href=\"group-delete?group=%s\">%s</a>\n",
						row[0], _("Delete Group"));
				fprintf(output, "</td>\n");
				mysql_free_result(usrs);
			}
			fprintf(output, "</tr>\n\n");
		}

		mysql_free_result(res);

		fprintf(output, "</table></p>\n");
	}

	aucHtmlSlimFoot();
	return 0;
}

int
aucPageAddGroup(FILE * output, char *extrapath)
{
	int x, edit = 0, nogood = 0;
	char buffer[1024], buffer2[1024], group[17], *fields[32], *a;

	strcpy(group, "");
	for (x = 0; x < 32; x++)
		fields[x] = NULL;

	if (!strcmp(cgiRequestMethod, "GET")) {
		if (cgiFormSuccess == cgiFormString("group", group, sizeof(group))) {
			snprintf(buffer, sizeof(buffer), "grp = \"%s\"", group);
			if (aucDBNumRows("groups", buffer))
				edit = 1;
		}
	} else {
		aucHtmlReadForm("group users", fields, 0);
		strcpy(buffer2, "");
		if (strlen(fields[0])) {
			check_group(output, fields[0]);
			for (a = fields[1]; *a; a += strlen(buffer2)) {
				while ((*a <= 32) && (*a != 0))
					a++;
				if (*a == 0)
					break;
				sscanf(a, "%s", buffer2);
				aucSanitizeUsername(buffer2);
				snprintf(buffer, sizeof(buffer), "INSERT INTO groupmembers \
               (grp, uname) VALUES (\"%s\", \"%s\")", fields[0], buffer2);
				if (mysql_query(db, buffer))
					aucError(mysql_error(db));
			}
			cgiHeaderLocation("group-manager");
			return 0;
		} else
			nogood = 1;
	}

	cgiHeaderContentType("text/html");

	if (edit)
		aucHtmlSlimHead("%s", _("Add Users to Group"));
	else
		aucHtmlSlimHead("%s", _("Add New Group"));

	if (nogood)
		fprintf(output, "<p><b>%s:</b> %s</p>\n\n",
				_("Error"), _("You must specify a group name."));

	aucDrawTableForm(output, "POST", "group-edit");
	if (edit) {
		aucDrawHidden(output, "group", group);
		fprintf(output, "<tr><td align=right>%s:</td><td><b>%s</b></td></tr>\n\n",
				_("Group"), group);
	} else
		aucDrawTextbox(output, "group", 16, aucMaxGroupLen, fields[0], _("Group Name"));
	aucDrawTextarea(output, "users", 10, 32, fields[1], _("Users to Add"));
	if (edit)
		aucDrawSubmit(output, _("Add Users"));
	else
		aucDrawSubmit(output, _("Create Group"));
	aucDrawTableFormEnd(output);
	aucHtmlSlimFoot();

	return 0;
}

int
aucPageDeleteGroup(FILE * output, char *extrapath)
{
	char force[8], group[17], buf[512];
	int nogood = 0;

	if (cgiFormSuccess != cgiFormString("group", group, sizeof(group)))
		nogood = 1;

	if (nogood) {
		cgiHeaderLocation("group-manager");
		return -1;
	}

	if (cgiFormSuccess != cgiFormString("force", force, sizeof(force))) {
		cgiHeaderContentType("text/html");
		aucHtmlSlimHead("%s", _("Confirmation"));
		fprintf(output, "<p>%s <b>%s</b>?</p>\n", _("Are you sure you want to delete the group"), group);
		fprintf(output, "<ul><p><b><a href=\"group-delete?force=yes&group=%s\">%s</a></b></p>\n",
				group, _("Yes"));
		fprintf(output, "<p><b><a href=\"group-manager\">No</a></b></p></ul>\n");
		aucHtmlSlimFoot();
		return 0;
	}

	snprintf(buf, sizeof(buf), "DELETE FROM groups WHERE grp = \"%s\"", group);
	if (mysql_query(db, buf))
		aucError(mysql_error(db));

	snprintf(buf, sizeof(buf), "DELETE FROM groupmembers WHERE grp = \"%s\"", group);
	if (mysql_query(db, buf))
		aucError(mysql_error(db));

	cgiHeaderLocation("group-manager");
	return 0;
}

int
aucPageDeleteUserGroup(FILE * output, char *extrapath)
{
	char user[65], group[17], buf[512];
	int nogood = 0;

	if (cgiFormSuccess != cgiFormString("user", user, sizeof(user)))
		nogood = 1;
	if (cgiFormSuccess != cgiFormString("group", group, sizeof(group)))
		nogood = 1;

	if (nogood) {
		cgiHeaderLocation("group-manager");
		return -1;
	}

	snprintf(buf, sizeof(buf), "DELETE FROM groupmembers WHERE grp = \"%s\" AND uname = \"%s\"",
			 group, user);
	if (mysql_query(db, buf))
		aucError(mysql_error(db));

	cgiHeaderLocation("group-manager");
	return 0;
}

int
aucAuthInGroup(char *group)
{
	int x;

	if (!strncasecmp(group, aucUserCache->role, strlen(aucUserCache->role)))
		return 1;
	for (x = 0; strlen(aucUserCache->groups[x]); x++)
		if (!strncasecmp(group, aucUserCache->groups[x], strlen(aucUserCache->groups[x])))
			return 1;
	return 0;
}

int
store_groups(aucAuthCachePtr cache, int maxgrps)
{
	MYSQL_RES *grps;
	MYSQL_ROW row;
	char buffer[1024];
	int x = 0;

	snprintf(buffer, sizeof(buffer), "SELECT grp FROM groupmembers WHERE \
      uname = \"%s\"", cache->uname);
	if (mysql_query(db, buffer))
		aucError(mysql_error(db));
	else {
		grps = mysql_store_result(db);
		while ((row = mysql_fetch_row(grps))) {
			snprintf(cache->groups[x], sizeof(cache->groups[x]), "%s", row[0]);
			x++;
		}
		mysql_free_result(grps);
	}
	cache->groups[x][0] = '\0';
	return x;
}

int
aucAuthGetFullName(char *fullname, size_t len, char *uname, unsigned long flags)
{
	char buf[1024], prefname[128];
	MYSQL_RES *res;
	MYSQL_ROW row;
	int retval = -1;

	strcpy(prefname, "");
	snprintf(fullname, len, "%s", uname);
	snprintf(buf, sizeof(buf), "SELECT title, fnamecfg, mnamecfg FROM prefs WHERE uname=\"%s\"",
			 uname);
	if (mysql_query(db, buf))
		aucError(mysql_error(db));
	else {
		res = mysql_store_result(db);
		if ((row = mysql_fetch_row(res)))
			sprintf(prefname, "%s%s%s%s%s%s", row[0], strlen(row[0]) ? " " : "",
					row[1], strlen(row[1]) ? " " : "", row[2], strlen(row[2]) ? " " : "");
		mysql_free_result(res);
	}

	snprintf(buf, sizeof(buf), "SELECT fname, mname, lname FROM userinfo WHERE uname=\"%s\"",
			 uname);
	if (mysql_query(db, buf))
		aucError(mysql_error(db));
	else {
		res = mysql_store_result(db);
		if ((row = mysql_fetch_row(res))) {
			retval = 0;
			
			if (strlen(prefname))
				snprintf(buf, sizeof(buf), "%s%s", prefname, row[2]);
			else if (strlen(row[1]))
				snprintf(buf, sizeof(buf), "%s %c. %s", row[0], *row[1], row[2]);
			else
				snprintf(buf, sizeof(buf), "%s %s", row[0], row[2]);

			if (flags & NAME_LINK)
				snprintf(fullname, len,
						 "%s <a href=\"%s/users/%s\"><img src=\"%s/%s\" alt=\"[Q]\" border=0 align=absmiddle></a>",
						 buf, aucPagesRoot, uname, aucGraphDir, aucQueryIcon);
			else
				snprintf(fullname, len, "%s", buf);

		}
		mysql_free_result(res);
	}

	return retval;
}

