/*
 * Authenticated User Community
 * Copyright (c) 2000 David C. Moore
 *
 * You may distribute and/or use for any purpose modified or unmodified
 * copies of this software if you preserve the copyright notice above.
 *
 * auth.h
 *
 * Authentication functions
 *
 */

/* Make sure this is only included once. */

#ifndef AUTH_C
#define AUTH_C 1

#define AUTH_NONE    (0x1 <<0)
#define AUTH_INVALID (0x1 <<1)
#define AUTH_VALID   (0x1 <<2)
#define AUTH_EXPIRED (0x1 <<3)
#define AUTH_DBERROR (0x1 <<4)

#define AUTH_TYPE_UNKNOWN  (0x1 <<0)
#define AUTH_TYPE_PAM      (0x1 <<1)
#define AUTH_TYPE_DATABASE (0x1 <<2)
#define AUTH_TYPE_DBPASSWD (0x1 <<3)

#define NAME_LINK    (0x1 <<1)

#define MAXGRPS 32

typedef struct _aucAuthCache aucAuthCache;
typedef aucAuthCache *aucAuthCachePtr;
struct _aucAuthCache
{
	char uname[65], passwd[17], permissions[9], fullname[64], homedir[64], role[16];
	int uid, gid, local, authtype;
	unsigned long id;
	unsigned int class;
	char groups[MAXGRPS][17];
	time_t login, last_request;
	char authstring[21];
};

extern char *aucDirectoryGrps;

extern aucAuthCachePtr aucAuthNewCache(void);
extern void aucAuthFreeCache(aucAuthCachePtr cache);
extern void aucAuthWriteCache(aucAuthCachePtr cache, char *uname);
extern void aucAuthClearCache(char *uname);
extern int aucAuthReadCache(aucAuthCachePtr cache, char *uname);
extern int aucAuthCheck(aucAuthCachePtr cache);
extern void aucAuthLoginPage(char *message);
extern int aucAuthPermsof(char *authlevel);
extern void aucAuthScramblePasswd(char *scrambled, size_t len, char *unscrambled);
extern void aucAuthUnscramblePasswd(char *unscrambled, size_t len, char *scrambled);
extern int aucAuthInGroup(char *group);
extern int aucAuthGetFullName(char *fullname, size_t len, char *uname, unsigned long flags);

extern int aucPageAccountManager(FILE * output, char *extrapath);
extern int aucPageAddAccount(FILE * output, char *extrapath);
extern int aucPageDeleteAccount(FILE * output, char *extrapath);
extern int aucPageBulkAddAccount(FILE * output, char *extrapath);
extern int aucPageGroupManager(FILE * output, char *extrapath);
extern int aucPageAddGroup(FILE * output, char *extrapath);
extern int aucPageDeleteGroup(FILE * output, char *extrapath);
extern int aucPageDeleteUserGroup(FILE * output, char *extrapath);
extern int aucPageSetPasswd(FILE * output, char *extrapath);



#endif
