/*
 * Authenticated User Community
 * Copyright (c) 2000 David C. Moore
 *
 * You may distribute and/or use for any purpose modified or unmodified
 * copies of this software if you preserve the copyright notice above.
 *
 * calendar.c
 *
 * Functions that manipulate the SQL representation of the school's
 * day to day calendar
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <auc/cgic.h>
#include <auc/auc.h>
#include <auc/pages.h>
#include <auc/html.h>
#include <auc/database.h>
#include <auc/error.h>
#include <auc/course.h>
#include "calendar.h"

int
aucCalendarInit(void)
{
	aucPagesRegister(NULL, "calendar-master", PAGE_FILE, aucCalendarMaster, "sysop", 0);

	aucDBRegister("calendar", "CREATE TABLE %s (eventdate DATE NOT NULL, \
         type VARCHAR(16) NOT NULL DEFAULT '', \
         name VARCHAR(64) NOT NULL, description TEXT, \
         KEY crono (eventdate,type))");

	return 0;
}

int
aucCalendarGetCurrTerm()
{
	MYSQL_RES *res;
	MYSQL_ROW row;
	char buf[1024];
	int result = 1, x, cond = 0;

	if (!db || aucDBConnectFailed)
		return result;

	for (x = 2; x <= aucNumYearDivisions; x++) {
		snprintf(buf, sizeof(buf), "SELECT (curdate() <= eventdate) FROM \
         calendar WHERE type=\"semend\" AND name=\"%d\"", x);
		if (mysql_query(db, buf))
			aucError(mysql_error(db));
		else {
			res = mysql_store_result(db);
			if ((row = mysql_fetch_row(res)))
				if (row[0] && !strcmp(row[0], "1"))
					cond = 1;
			mysql_free_result(res);
			if (!cond)
				continue;
		}
		snprintf(buf, sizeof(buf), "SELECT (curdate() > eventdate) FROM \
         calendar WHERE type=\"semend\" AND name=\"%d\"", x - 1);
		if (mysql_query(db, buf))
			aucError(mysql_error(db));
		else {
			res = mysql_store_result(db);
			if ((row = mysql_fetch_row(res)))
				if (row[0] && !strcmp(row[0], "1") && (cond == 1))
					result = x;
			mysql_free_result(res);
			if (result != 1)
				return result;
		}
	}
	return result;
}

int
aucGetSemBeg(char *str, size_t len, int sempref)
{
	MYSQL_RES *res;
	MYSQL_ROW row;
	char buf[512];

	strncpy(str, "", len);

	snprintf(buf, sizeof(buf), "SELECT eventdate FROM calendar WHERE \
      type=\"semstart\" AND name=\"%d\"", sempref);
	if (mysql_query(db, buf)) {
		aucError(mysql_error(db));
		return -1;
	}
	res = mysql_store_result(db);
	if ((row = mysql_fetch_row(res)))
		snprintf(str, len, "%s", row[0]);
	else
		return -1;
	mysql_free_result(res);

	return 0;
}

int
aucGetTodayDate(char *str, size_t len)
{
	MYSQL_RES *res;
	MYSQL_ROW row;
	char buf[512];

	snprintf(buf, sizeof(buf), "SELECT CURDATE()");
	if (mysql_query(db, buf))
		aucError(mysql_error(db));
	else {
		res = mysql_store_result(db);
		row = mysql_fetch_row(res);
		snprintf(str, len, "%s", row[0]);
		mysql_free_result(res);
	}
	return 0;
}

int
isschoolday(char *date)
{
	MYSQL_RES *res;
	MYSQL_ROW row;
	char buf[512];

	snprintf(buf, sizeof(buf), "SELECT DAYOFWEEK(\"%s\")", date);
	if (mysql_query(db, buf)) {
		aucError(mysql_error(db));
		return 0;
	}
	res = mysql_store_result(db);
	row = mysql_fetch_row(res);
	if (!strcmp(row[0], "1") || !strcmp(row[0], "7"))
		return 0;
	mysql_free_result(res);

	snprintf(buf, sizeof(buf), "SELECT COUNT(*) FROM calendar WHERE \
      type=\"Holiday\" AND eventdate=\"%s\"", date);
	if (mysql_query(db, buf)) {
		aucError(mysql_error(db));
		return 0;
	}
	res = mysql_store_result(db);
	row = mysql_fetch_row(res);
	if (strcmp(row[0], "0"))
		return 0;
	mysql_free_result(res);

	return 1;
}

int
aucIncDate(char *str, size_t len, int sempref)
{
	MYSQL_RES *res;
	MYSQL_ROW row;
	char buf[512];

	while (1) {
		snprintf(buf, sizeof(buf), "SELECT DATE_ADD(\"%s\", INTERVAL 1 DAY)", str);
		if (mysql_query(db, buf)) {
			aucError(mysql_error(db));
			break;
		} else {
			res = mysql_store_result(db);
			row = mysql_fetch_row(res);
			snprintf(str, len, "%s", row[0]);
			mysql_free_result(res);
		}

		snprintf(buf, sizeof(buf), "SELECT eventdate FROM calendar WHERE \
         type=\"semend\" AND name=\"%d\"", sempref);
		if (mysql_query(db, buf)) {
			aucError(mysql_error(db));
			break;
		} else {
			res = mysql_store_result(db);
			if (!(row = mysql_fetch_row(res)) || (strcmp(row[0], str) < 0)) {
				strcpy(str, "");
				return 0;
			}
			mysql_free_result(res);
		}

		if (isschoolday(str))
			break;
	}

	return 0;
}

char *
aucDateText(char *date)
{
	MYSQL_RES *res;
	MYSQL_ROW row;
	char buf[512];

	snprintf(buf, sizeof(buf), "SELECT DATE_FORMAT(\"%s\", \"%s\")", date, "%a, %b %d, %Y");
	if (mysql_query(db, buf)) {
		aucError(mysql_error(db));
		return NULL;
	}
	res = mysql_store_result(db);
	row = mysql_fetch_row(res);
	snprintf(buf, sizeof(buf), "%s", row[0]);
	mysql_free_result(res);

	return (char *) strdup(buf);
}

int
aucCalendarMaster(FILE * output, char *extrapath)
{
	MYSQL_RES *res = NULL, *res2 = NULL;
	MYSQL_ROW row, row2;
	char buf[512], name[64], caption[128], *values[32][6];
	int x, sem;

	for (x = 0; x < 32; x++)
		for (sem = 0; sem < 6; sem++)
			values[x][sem] = NULL;

	if (!strcmp(cgiRequestMethod, "POST")) {
		snprintf(buf, sizeof(buf), "DELETE FROM calendar WHERE \
         type=\"blockstart\" OR type=\"semstart\" OR type=\"semend\"");
		if (mysql_query(db, buf))
			aucError(mysql_error(db));
		for (sem = 1; sem <= aucNumYearDivisions; sem++) {
			snprintf(buf, sizeof(buf), "s%dstart", sem);
			aucHtmlReadForm(buf, &values[0][sem], STR_ESCAPE);
			snprintf(buf, sizeof(buf), "s%dend", sem);
			aucHtmlReadForm(buf, &values[2][sem], STR_ESCAPE);
			if (values[0][sem] && strlen(values[0][sem])) {
				snprintf(buf, sizeof(buf), "INSERT INTO calendar \
               (eventdate, type, name) VALUES (\"%s\", \"semstart\", \"%d\")", values[0][sem], sem);
				if (mysql_query(db, buf))
					aucError(mysql_error(db));
			}
			if (values[2][sem] && strlen(values[2][sem])) {
				snprintf(buf, sizeof(buf), "INSERT INTO calendar \
               (eventdate, type, name) VALUES (\"%s\", \"semend\", \"%d\")", values[2][sem], sem);
				if (mysql_query(db, buf))
					aucError(mysql_error(db));
			}
		}
		cgiHeaderLocation("home");
		return 0;
	}

	for (sem = 1; sem <= aucNumYearDivisions; sem++) {
		snprintf(buf, sizeof(buf), "SELECT eventdate FROM calendar \
         WHERE type=\"semstart\" AND name=\"%d\"", sem);
		if (mysql_query(db, buf))
			aucError(mysql_error(db));
		else {
			res = mysql_store_result(db);

			if ((row = mysql_fetch_row(res))) {
				values[0][sem] = (char *) strdup(row[0]);
				snprintf(buf, sizeof(buf), "SELECT eventdate FROM calendar \
               WHERE type=\"blockstart\" AND eventdate >= \"%s\" ORDER BY eventdate", row[0]);
				if (mysql_query(db, buf))
					aucError(mysql_error(db));
				else {
					res2 = mysql_store_result(db);
					if ((row2 = mysql_fetch_row(res2)))
						values[1][sem] = (char *) strdup(row2[0]);
					mysql_free_result(res2);
				}
			}

			mysql_free_result(res);
		}
		snprintf(buf, sizeof(buf), "SELECT eventdate FROM calendar \
         WHERE type=\"semend\" AND name=\"%d\"", sem);
		if (mysql_query(db, buf))
			aucError(mysql_error(db));
		else {
			res = mysql_store_result(db);

			if ((row = mysql_fetch_row(res))) {
				values[2][sem] = (char *) strdup(row[0]);
			}
			mysql_free_result(res);
		}
	}


	cgiHeaderContentType("text/html");

	aucHtmlSlimHead("%s", _("Master Calendar"));
	aucDrawTableForm(output, "POST", "calendar-master");
	for (sem = 1; sem <= aucNumYearDivisions; sem++) {
		snprintf(name, sizeof(name), "s%dstart", sem);
		snprintf(caption, sizeof(caption), "%s %d %s", aucYearDivisionName, sem, _("Start"));
		aucDrawDateselect(output, name, values[0][sem], caption);
		snprintf(name, sizeof(name), "s%dend", sem);
		snprintf(caption, sizeof(caption), "%s %d %s", aucYearDivisionName, sem, _("End"));
		aucDrawDateselect(output, name, values[2][sem], caption);
	}
	aucDrawSubmit(output, _("Save"));
	aucDrawTableFormEnd(output);
	aucHtmlSlimFoot();

	return 0;
}
