/*
 * Authenticated User Community
 * Copyright (c) 2000 David C. Moore
 *
 * You may distribute and/or use for any purpose modified or unmodified
 * copies of this software if you preserve the copyright notice above.
 *
 * course.c
 *
 * Routines relating to online iClassrooms display and manipulation
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <string.h>
#include <unistd.h>

#include <auc/cgic.h>
#include <auc/auc.h>
#include <auc/files.h>
#include <auc/calendar.h>
#include <auc/discussion.h>
#include <auc/coursenode.h>
#include <auc/conf.h>
#include <auc/pages.h>
#include <auc/database.h>
#include <auc/error.h>
#include <auc/html.h>
#include <auc/forms.h>
#include "course.h"

char **periods;
char *periods_default[] = {
	"1", "2", "3", "4", "5A", "5B", "6", "7", "8", NULL
};
char *periods_raw[] = {
	"1", "2", "3", "4", "5", "6", "7", "8", "9", NULL
};

char **departments;
char *departments_default[] = {
	"Computer Science",
	"English",
	"ESOL",
	"Fine Arts",
	"Foreign Language",
	"Mathematics",
	"Physical Education",
	"Science",
	"Social Studies",
	"Technology",
	"Other",
	NULL
};

char *aucScheduling = "regular";
char *aucYearDivisionName = "Semester";
char *aucYearDivisionNameP = "Semesters";
unsigned long aucNumYearDivisions = 2;
int aucPublicClassList = 1;

int autoschedule(char *);
int printannouncement(FILE *, char *);
char *getaides(char *, int);
int printscheduleshort(FILE *, char *);
int isaide(char *, char *);
int create_class_files(char *idstr);
int exist_class_files(char *idstr);
int course_page_header(FILE * output, aucCourseInfo clinfo);

void announcement_node(FILE * output, aucCourseNodeInfoPtr node, aucCourseInfoPtr clinfo);
void files_node(FILE * output, aucCourseNodeInfoPtr node, aucCourseInfoPtr clinfo);

void course_tools_node_frame(FILE * output, aucNodelettInfoPtr nodelett, char *arg);
void my_courses_node_frame(FILE * output, aucNodelettInfoPtr nodelett, char *arg);

int course_manager_page(FILE * output, char *extrapath);
int add_course_page(FILE * output, char *extrapath);
int delete_course_page(FILE * output, char *extrapath);
int view_course_page(FILE * output, char *extrapath);
int my_courses_config_page(FILE * output, char *extrapath);
int edit_course_annc_page(FILE * output, char *extrapath);
int edit_course_aides_page(FILE * output, char *extrapath);
int course_list_page(FILE * output, char *extrapath);


int
aucCourseInit(void)
{
	int count;
	char *buf;

	if (aucConfTokens("child::CourseScheduling/Department", &departments))
		departments = departments_default;
	if (aucConfTokens("child::CourseScheduling/Period", &periods))
		periods = periods_default;
	for (count = 0; periods[count]; count++);
	periods_raw[count] = NULL;
	aucConfToken("child::CourseScheduling/attribute::type", &aucScheduling);
	aucConfToken("child::CourseScheduling/YearDivisionName", &aucYearDivisionName);
	aucConfToken("child::CourseScheduling/YearDivNamePlural", &aucYearDivisionNameP);
	if (!aucConfToken("child::CourseScheduling/NumYearDivisions", &buf)) {
		aucNumYearDivisions = atol(buf);
		free(buf);
	}
	if (!aucConfToken("child::CourseScheduling/attribute::publiclist", &buf)) {
		if ((*buf == 'N') || (*buf == 'n'))
			aucPublicClassList = 0;
		free(buf);
	}

	aucCourseNodeInit();

	aucNodelettRegister("schedule",
						_("My Courses"),
						my_courses_node_frame, 2, 180, 20,
						_("Your class schedule that links to interactive classrooms."), "guest", 0);
	aucNodelettRegister("classes-admin",
						_("Course Tools"),
						course_tools_node_frame, 1, 80, 100,
						_("Administers the interactive classrooms"), "sysop", 0);

	aucPagesRegister(NULL, "course", PAGE_DIRFUNC, view_course_page, "guest", 0);
	aucPagesRegister(NULL, "courseannc-edit", PAGE_FILE, edit_course_annc_page, "normal", 0);
	aucPagesRegister(NULL, "aide-edit", PAGE_FILE, edit_course_aides_page, "normal", 0);
	aucPagesRegister(NULL, "course-manager", PAGE_FILE, course_manager_page, "sysop", 0);
	aucPagesRegister(NULL, "course-edit", PAGE_FILE, add_course_page, "sysop", 0);
	aucPagesRegister(NULL, "course-delete", PAGE_FILE, delete_course_page, "sysop", 0);
	aucPagesRegister(NULL, "schedule-config", PAGE_FILE, my_courses_config_page, "normal", 0);
	aucPagesRegister(NULL, "courselist", PAGE_FILE, course_list_page, "normal", 0);

	aucDBRegister("classes", "CREATE TABLE %s (id VARCHAR(16) DEFAULT '' NOT NULL, \
         title VARCHAR(64) NOT NULL, teacher VARCHAR(64) DEFAULT '' NOT NULL, \
         room VARCHAR(32) NOT NULL, department VARCHAR(32) NOT NULL, \
			period CHAR(2) NOT NULL, semester SMALLINT NOT NULL, \
			classtime VARCHAR(64) NOT NULL, description TEXT NOT NULL, \
         PRIMARY KEY (id,semester))");
	aucDBRegister("classannc", "CREATE TABLE %s (idstr VARCHAR(16) DEFAULT '' NOT NULL, \
         text TEXT DEFAULT '' NOT NULL, time DATETIME, \
         PRIMARY KEY (idstr))");
	aucDBRegister("classaides", "CREATE TABLE %s (idstr VARCHAR(16) DEFAULT '' NOT NULL, \
         aide VARCHAR(64) DEFAULT '' NOT NULL, PRIMARY KEY (idstr, aide))");
	aucDBRegister("classsched", "CREATE TABLE %s (idstr VARCHAR(16) DEFAULT '' NOT NULL, \
         sid BIGINT UNSIGNED DEFAULT 0 NOT NULL AUTO_INCREMENT, \
         name VARCHAR(64) DEFAULT '' NOT NULL, comment TEXT DEFAULT '' \
         NOT NULL, assndate DATE, assntype VARCHAR(32) DEFAULT '' \
         NOT NULL, block CHAR(8) NOT NULL DEFAULT '', \
         instime BIGINT UNSIGNED DEFAULT 0 NOT NULL, \
         KEY autoinc (sid), PRIMARY KEY (idstr, sid))");
	aucDBRegister("scheds", "CREATE TABLE %s (uname VARCHAR(64) DEFAULT '' NOT NULL, \
         public ENUM('Y','N'), p1 VARCHAR(16) DEFAULT '', \
         p2 VARCHAR(16) DEFAULT '', p3 VARCHAR(16) DEFAULT '', \
         p4 VARCHAR(16) DEFAULT '', p5 VARCHAR(16) DEFAULT '', \
         p6 VARCHAR(16) DEFAULT '', p7 VARCHAR(16) DEFAULT '', \
         p8 VARCHAR(16) DEFAULT '', p9 VARCHAR(16) DEFAULT '', \
         semester SMALLINT NOT NULL, PRIMARY KEY(uname,semester))");
	aucDBRegister("rosters", "CREATE TABLE %s (id VARCHAR(16) DEFAULT '' NOT NULL, \
			uname VARCHAR(64) DEFAULT '' NOT NULL, \
			schoolyear VARCHAR(16) DEFAULT '' NOT NULL, \
			semester SMALLINT NOT NULL, grade CHAR(4), \
			comments TEXT, PRIMARY KEY(id,uname,schoolyear,semester))");


	aucCourseNodeRegister("announcements",
						  _("Announcements"),
						  announcement_node, 0, _("Displays course announcements"), 0);
	aucCourseNodeRegister("files",
						  _("Documents and Files"),
						  files_node,
						  20,
						  _("A place where files can be made available to students in the course."),
						  0);

	aucCurrSemester = aucCalendarGetCurrTerm();

	return 0;
}

void
announcement_node(FILE * output, aucCourseNodeInfoPtr node, aucCourseInfoPtr clinfo)
{
	printannouncement(output, clinfo->uniquestr);
	if (clinfo->admin)
		fprintf(output,
				"<div align=right><small><b><a href=\"../courseannc-edit?course=%s\">%s</a></b></small></div>\n\n",
				aucLinkText(clinfo->uniquestr), _("Change Announcement"));
}

void
files_node(FILE * output, aucCourseNodeInfoPtr node, aucCourseInfoPtr clinfo)
{
	char buf[64];

	fprintf(output, "<div align=right><small><b><a href=\"%s/\">%s</a></b></small></div>\n\n",
			aucLinkText(clinfo->uniquestr), _("View Detailed List"));
	snprintf(buf, sizeof(buf), "course/%s", clinfo->uniquestr);
	aucFileListView(output, buf, "", -1, 0, 1);
}

char *
aucCoursePeriodName(int num)
{
	if ((num > 0) && (num < 9) && periods[num - 1])
		return periods[num - 1];
	else
		return "";
}

int
isaide(char *uniquestr, char *uname)
{
	char buf[512];

	snprintf(buf, sizeof(buf), "idstr=\"%s\" AND aide=\"%s\"", uniquestr, uname);
	return (int) aucDBNumRows("classaides", buf);
}

void
my_courses_node_frame(FILE * output, aucNodelettInfoPtr nodelett, char *arg)
{
	int classes;

	fprintf(output, "<table width=100%% height=%d cellpadding=2 cellspacing=0 border=0 bgcolor=#FFFFFF>\n",
			nodelett->height);

	fprintf(output, "<tr valign=top><td colspan=2 height=%d>\n", nodelett->height - 24);
	classes = aucCoursePrintList(output, (arg) ? arg : aucUserCache->uname, aucCurrSemester);
	fprintf(output, "</td></tr>\n\n");
	fprintf(output, "<tr bgcolor=#CCCCCC>\n");
	fprintf(output, "<td height=24>\n");
	if (!arg)
		fprintf(output, "<a href=\"schedule-config\">%s...</a>\n", _("Customize My Courses"));
	else
		fprintf(output, "&nbsp;");
	fprintf(output, "</td><td>\n");
	fprintf(output, "<div align=right><b>%s %ld</b>: %d %s</div>\n",
			aucYearDivisionName, aucCurrSemester, classes,
			(classes == 1) ? _("course") : _("courses"));
	fprintf(output, "</td></tr></table>\n\n");
}

int
my_courses_config_page(FILE * output, char *extrapath)
{
	MYSQL_RES *res = NULL;
	MYSQL_ROW row[7];
	MYSQL_FIELD *fields = NULL;
	char buf[2048], buf2[2048], buf3[2048], *values[32][6], id[17];
	int x, sem;
	char public = 'Y';

	for (x = 0; x < 32; x++)
		for (sem = 0; sem < 6; sem++)
			values[x][sem] = NULL;

	if (!strcmp(cgiRequestMethod, "POST")) {
		if (cgiFormSuccess != cgiFormCheckboxSingle("public"))
			public = 'N';
		for (sem = 1; sem <= aucNumYearDivisions; sem++) {
			snprintf(buf, sizeof(buf), "DELETE FROM scheds WHERE uname=\"%s\" AND semester=%d",
					 aucUserCache->uname, sem);
			if (mysql_query(db, buf))
				aucError(mysql_error(db));
			strcpy(buf, "");
			strcpy(buf3, "");
			for (x = 1; x <= 9; x++) {
				snprintf(buf2, sizeof(buf2), "s%dp%d", sem, x);
				if (cgiFormSuccess == cgiFormString(buf2, id, sizeof(id))) {
					snprintf(buf2, sizeof(buf2), ", \"%s\"", id);
					strcat(buf, buf2);
					snprintf(buf2, sizeof(buf2), ", %c%d", 'p', x);
					strcat(buf3, buf2);
				};
			}
			if (strlen(buf)) {
				snprintf(buf2, sizeof(buf2), "INSERT INTO scheds \
               (uname, public%s, semester) VALUES (\"%s\", '%c'%s, %d)", buf3, aucUserCache->uname, public, buf, sem);
				if (mysql_query(db, buf2))
					aucError(mysql_error(db));
			}
		}
		cgiHeaderLocation("home");
		return 0;
	}

	for (sem = 1; sem <= aucNumYearDivisions; sem++) {
		snprintf(buf, sizeof(buf), "SELECT * FROM scheds WHERE uname = \"%s\" AND semester = %d",
				 aucUserCache->uname, sem);
		if (mysql_query(db, buf))
			aucError(mysql_error(db));
		else {
			res = mysql_store_result(db);
			fields = mysql_fetch_fields(res);
			row[sem] = mysql_fetch_row(res);
			for (x = 0; row[sem] && (x < mysql_num_fields(res)); x++)
				values[x][sem] = row[sem][x];
		}
		if (values[1][sem] && (*values[1][sem] == 'N'))
			public = 'N';
	}


	cgiHeaderContentType("text/html");

	aucHtmlSlimHead("%s", _("Customize Interactive Schedule"));

	aucGetFullPath("schedule-mess.html", aucDataDir, buf, sizeof(buf));
	aucHtmlEchoFile(output, buf, 0);

	aucDrawTableForm(output, "POST", "schedule-config");
	aucDrawHidden(output, "username", aucUserCache->uname);
	fprintf(output, "<tr>\n");
	for (sem = 1; sem <= aucNumYearDivisions; sem++) {
		fprintf(output, "<td><p align=right><b>%s %d</b>\n", aucYearDivisionName, sem);

		if (aucPublicClassList)
			fprintf(output,
					"<br><small><a href=\"javascript: newWindow('courselist?sem=%d','courselist','540')\">%s</a></small><br>\n",
					sem, _("Course List"));

		fprintf(output, "</p><table cellpadding=2 cellspacing=0 border=0>\n\n");
		for (x = 2; x <= 10; x++) {
			snprintf(buf, sizeof(buf), "s%d%s", sem, fields[x].name);
			aucDrawTextbox(output, buf, 16, 16, values[x][sem], "ID");
		}
		fprintf(output, "</table></td>\n\n");
	}
	fprintf(output, "</tr></table>\n\n");
	fprintf(output, "<table cellpadding=2 cellspacing=0 border=0>\n");
	aucDrawCheckbox(output, fields[1].name, (public == 'Y') ? 1 : 0,
					_
					("Make my schedule visible to other students and teachers who would like to view it"),
					_("Options"));
	aucDrawSubmit(output, _("Save Schedule"));
	aucDrawTableFormEnd(output);
	aucHtmlSlimFoot();

	if (res)
		mysql_free_result(res);
	return 0;
}

int
course_list_page(FILE * output, char *extrapath)
{
	MYSQL_RES *res;
	MYSQL_ROW row;
	int sem = 1, count = 0;
	char buf[1024];

	cgiFormInteger("sem", &sem, 1);

	cgiStaticContentType("text/html");

	snprintf(buf, sizeof(buf), "%s %s %d", _("Course List for"), aucYearDivisionName, sem);
	aucHtmlPopupHead(buf);

	snprintf(buf, sizeof(buf), "SELECT * FROM classes \
      WHERE semester=%d ORDER BY period ASC, id ASC", sem);
	if (mysql_query(db, buf))
		aucError(mysql_error(db));
	else {
		res = mysql_store_result(db);
		fprintf(output, "<TABLE border=0 cellspacing=0 cellpadding=2 width=500>\n");
		fprintf(output, "<TR bgcolor=#CCCCCC><TD><b>%s</b></TD><TD><b>%s</b></TD>",
				_("ClassID"), _("Title"));
		fprintf(output, "<TD><b>%s</b></TD>\n", _("Instructor"));
		if (!strcmp(aucScheduling, "periods"))
			fprintf(output, "<TD><b>%s</b></TD>\n", _("Period"));
		else if (!strcmp(aucScheduling, "daytime"))
			fprintf(output, "<TD><b>%s</b></TD>\n", _("Time"));
		fprintf(output, "</TR>\n");

		while ((row = mysql_fetch_row(res))) {
			aucAuthGetFullName(buf, sizeof(buf), row[2], 0);
			fprintf(output, "<tr><TD>%s</TD><TD>%s</TD><TD>%s</TD>\n", row[0], row[1], buf);
			if (!strcmp(aucScheduling, "periods"))
				fprintf(output, "<TD>%s</TD>\n", aucCoursePeriodName(atoi(row[5])));
			else if (!strcmp(aucScheduling, "daytime"))
				fprintf(output, "<TD>%s</TD>\n", row[7]);
			fprintf(output, "</TR>\n");
			count++;
		}
		if (!count)
			fprintf(output, "<tr><td colspan=4><i>None Available</i></td></tr>\n");
		fprintf(output, "</TABLE>");
	}

	aucHtmlPopupFoot();
	return 0;
}

int
print_course_info_row(FILE * output, MYSQL_ROW row)
{
	char buf[128];
	int period;

	fprintf(output, "<tr>\n");

	if (!strcmp(aucScheduling, "periods")) {
		period = strtoul(row[5], NULL, 10);
		fprintf(output, "<td>&nbsp;%s</td>\n", aucCoursePeriodName(period));
	} else
		fprintf(output, "<td>&nbsp;%s</td>\n", row[0]);

	snprintf(buf, sizeof(buf), "%s@%s", row[0], row[6]);
	fprintf(output, "<td><a href=\"%s/course/%s\">%s</a></td>\n",
			aucPagesRoot, aucLinkText(buf), row[1]);

	if (aucAuthGetFullName(buf, sizeof(buf), row[2], 0))
		fprintf(output, "<td>%s</td>\n", buf);
	else
		fprintf(output, "<td><a href=\"%s/users/%s\">%s</a></td>\n", aucPagesRoot, row[2], buf);

	fprintf(output, "<td>%s</td>\n", row[3]);

	fprintf(output, "</tr>\n\n");
	return 0;
}

int
aucCoursePrintList(FILE * output, char *uname, int semester)
{
	MYSQL_RES *res;
	MYSQL_ROW row;
	char buf[512];
	int x, count = 0, found;

	fprintf(output, "<table cellpadding=0 border=0 cellspacing=0 width=\"100%%\">\n\n");
	fprintf(output, "<tr bgcolor=#CCCCCC>\n");
	fprintf(output, "<td><font size=-1>&nbsp;</font></td>\n");
	fprintf(output, "<td><font size=-1><u><b>%s</b></u></font></td>\n", _("Course"));
	fprintf(output, "<td><font size=-1><u><b>%s</b></u></font></td>\n", _("Instructor"));
	fprintf(output, "<td><font size=-1><u><b>%s</b></u></font></td>\n", _("Location"));
	fprintf(output, "</tr>\n\n");

	snprintf(buf, sizeof(buf), "SELECT * FROM classes WHERE teacher=\"%s\" \
		AND semester=%d", uname, semester);
	if (mysql_query(db, buf))
		aucError(mysql_error(db));
	else {
		res = mysql_store_result(db);
		found = 0;
		while ((row = mysql_fetch_row(res))) {
			if (!found)
				fprintf(output, "<tr><td colspan=4><small><b>%s:</b></small></td></tr>\n\n",
						_("Courses taught"));
			print_course_info_row(output, row);
			count++;
			found = 1;
		}
		mysql_free_result(res);
	}

	snprintf(buf, sizeof(buf), "SELECT * FROM classes LEFT OUTER JOIN rosters ON \
		classes.id=rosters.id WHERE rosters.uname=\"%s\" AND rosters.semester=%d \
		AND rosters.schoolyear=\"%s\"", uname, semester, "");
	if (mysql_query(db, buf))
		aucError(mysql_error(db));
	else {
		res = mysql_store_result(db);
		found = 0;
		while ((row = mysql_fetch_row(res))) {
			if (!found)
				fprintf(output, "<tr><td colspan=4><small><b>%s:</b></small></td></tr>\n\n",
						_("Enrolled in"));
			print_course_info_row(output, row);
			count++;
			found = 1;
		}
		mysql_free_result(res);
	}

	snprintf(buf, sizeof(buf), "SELECT * \
      FROM scheds WHERE uname = \"%s\" AND semester=%d", uname, semester);
	if (mysql_query(db, buf))
		aucError(mysql_error(db));
	else {
		res = mysql_store_result(db);
		if ((row = mysql_fetch_row(res)) &&
			(!strcmp(row[1], "Y") || !strcmp(uname, aucUserCache->uname))) {
			found = 0;
			for (x = 2; x < 10; x++) {
				MYSQL_RES *res2;
				MYSQL_ROW row2;

				if (!row[x] || !strlen(row[x]))
					continue;

				if (!found && count)
					fprintf(output, "<tr><td colspan=4><small><b>%s:</b></small></td></tr>\n\n",
							_("Additional courses"));

				snprintf(buf, sizeof(buf), "SELECT * FROM classes WHERE id=\"%s\" AND \
					semester=%d", row[x], semester);
				if (mysql_query(db, buf))
					aucError(mysql_error(db));
				else {
					res2 = mysql_store_result(db);
					if ((row2 = mysql_fetch_row(res2)))
						print_course_info_row(output, row2);
					else
						fprintf(output, "<tr><td>&nbsp;%s</td><td colspan=3>%s</td></tr>\n\n",
								row[x], _("Unknown Course"));
					mysql_free_result(res2);
					count++;
					found = 1;
				}
			}
		}
		mysql_free_result(res);
	}
	if (!count)
		fprintf(output, "<tr><td colspan=4><i>%s</i></td></tr>\n\n",
				_("Schedule is Empty or Unavailable"));

	fprintf(output, "</table>\n\n");
	return count;
}

void
course_db_status(FILE * output)
{
	int classes = aucDBNumRows("classes", NULL);

	fprintf(output, "<b>%d %s</b>\n", classes, (classes == 1) ? _("course") : _("courses"));
}

void
course_tools_node_frame(FILE * output, aucNodelettInfoPtr nodelett, char *arg)
{
	fprintf(output, "<table width=100%% height=%d cellpadding=2 cellspacing=0 border=0 bgcolor=#FFFFFF>\n",
			nodelett->height);

	fprintf(output, "<tr valign=top><td height=%d>\n", nodelett->height - 24);
	fprintf(output, "<a href=\"course-manager\">%s</a><br>\n", _("Course Manager"));
	fprintf(output, "<a href=\"course-edit\">%s</a><br>\n", _("Add Course"));
	fprintf(output, "<a href=\"calendar-master\">%s</a>\n", _("Master Calendar"));
	fprintf(output, "</td></tr>\n\n");
	fprintf(output, "<tr bgcolor=#CCCCCC>\n");
	fprintf(output, "<td height=24>\n");
	fprintf(output, "<div align=right>\n");
	course_db_status(output);
	fprintf(output, "</div>\n");
	fprintf(output, "</td></tr></table>\n");
}

int
course_manager_page(FILE * output, char *extrapath)
{
	char buffer[2048], sort[2048], *deplist[64], **a;
	MYSQL_RES *res;
	MYSQL_ROW row;
	MYSQL_FIELD *fields;
	int num, x;

	aucSortPattern sorts[] = {
		{"id", "ORDER BY classes.id ASC", SORT_PAGES, 100, NULL, NULL},
		{"title", "ORDER BY classes.title ASC, classes.id ASC", SORT_ALPHA, 0, NULL, NULL},
		
			{"teacher",
		 "ORDER BY userinfo.lname ASC, userinfo.fname ASC, classes.teacher ASC, classes.semester ASC, classes.period ASC",
			 SORT_ALPHA, 0, NULL, NULL},
		
			{"department",
		 "ORDER BY classes.department ASC, userinfo.lname ASC, userinfo.fname ASC, classes.teacher ASC, classes.semester ASC, classes.period ASC",
			 SORT_CUSTOM, 0, deplist, deplist},
		
			{"period",
		 "ORDER BY classes.period ASC, classes.department ASC, userinfo.lname ASC, userinfo.fname ASC, classes.teacher ASC, classes.semester ASC",
			 SORT_CUSTOM, 0, periods_raw, periods},
		{NULL}
	};

	deplist[0] = NULL;
	if (mysql_query(db, "SELECT DISTINCT department from classes \
         ORDER BY department ASC"))
		aucError(mysql_error(db));
	else {
		res = mysql_store_result(db);
		a = deplist;
		while ((row = mysql_fetch_row(res))) {
			if (row[0])
				*a++ = (char *) strdup(row[0]);
			else
				*a++ = (char *) strdup(_("unknown"));
		}
		*a = NULL;
		mysql_free_result(res);
	}

	aucHtmlSetSortCookies(output, "course-manager");

	cgiHeaderContentType("text/html");

	aucHtmlSlimHead("%s", _("Course Manager"));

	fprintf(output, "<table border=0 cellpadding=0 cellspacing=0 width=100%%>\n");
	fprintf(output, "<tr valign=top><td>\n\n");

	if (mysql_query(db, "SELECT COUNT(*) from classes"))
		aucError(mysql_error(db));
	else {
		res = mysql_store_result(db);
		row = mysql_fetch_row(res);
		aucHtmlProcessSort(output, sort, sizeof(sort), "course-manager",
					   strtoul(row[0], NULL, 10), sorts, "department");
		mysql_free_result(res);
	}

	fprintf(output, "</td><td align=right>\n\n");

	fprintf(output, "<p>%s</p>\n", _("For authorized users only"));
	fprintf(output, "<b><a href=\"course-edit\">%s</a></b>\n", _("Create New Course"));

	fprintf(output, "</td></tr></table>\n\n");

	snprintf(buffer, sizeof(buffer), "SELECT * FROM classes LEFT OUTER JOIN \
      userinfo ON classes.teacher=userinfo.uname %s", sort);
	if (mysql_query(db, buffer))
		aucError(mysql_error(db));
	else {
		res = mysql_store_result(db);
		fields = mysql_fetch_fields(res);

		fprintf(output, "<p><table cellpadding=2 cellspacing=0 border=0 width=100%%>\n");
		fprintf(output, "<tr bgcolor=#CCCCCC>\n");
		for (x = 0, num = 7; x <= num; x++)
			if (x == num)
				fprintf(output, "<td><b>%s</b></td>\n", _("options"));
			else if (!strcmp(fields[x].name, "semester"))
				fprintf(output, "<td><b>%s</b></td>\n", aucYearDivisionName);
			else if (!strcmp(fields[x].name, "period")) {
				if (!strcmp(aucScheduling, "periods"))
					fprintf(output, "<td><b>%s</b></td>\n", fields[x].name);
				else if (!strcmp(aucScheduling, "daytime"))
					fprintf(output, "<td><b>%s</b></td>\n", _("time"));
				else
					fprintf(output, "<td>&nbsp;</td>\n");
			} else
				fprintf(output, "<td><b>%s</b></td>\n", fields[x].name);
		fprintf(output, "</tr>\n");

		while ((row = mysql_fetch_row(res))) {
			fprintf(output, "<tr>\n");
			for (x = 0; x <= num; x++)
				if (x == 0) {
					snprintf(buffer, sizeof(buffer), "%s@%s", row[0], row[6]);
					fprintf(output, "<td><a href=\"course/%s\">%s</a></td>\n",
							aucLinkText(buffer), row[0]);
				} else if (x == num) {
					snprintf(buffer, sizeof(buffer), "id=%s&term=%s", row[0], row[6]);
					fprintf(output, "<td><a href=\"course-edit?%s\">%s</a>\n",
							aucLinkText(buffer), _("Edit"));
					snprintf(buffer, sizeof(buffer), "%s@%s", row[0], row[6]);
					fprintf(output, "<a href=\"course-delete?course=%s\">%s</a></td>\n",
							aucLinkText(buffer), _("Delete"));
				} else if (x == 2) {
					aucAuthGetFullName(buffer, sizeof(buffer), row[x], NAME_LINK);
					fprintf(output, "<td>%s</td>\n", buffer);
				} else if (x == 5) {
					int period = strtoul(row[x], NULL, 10);
					if (!strcmp(aucScheduling, "periods"))
						fprintf(output, "<td>%s</td>\n", aucCoursePeriodName(period));
					else if (!strcmp(aucScheduling, "daytime"))
						fprintf(output, "<td>%s</td>\n", row[7]);
					else
						fprintf(output, "<td>&nbsp;</td>\n");
				} else
					fprintf(output, "<td>%s</td>\n", row[x]);
			fprintf(output, "</tr>\n");
		}

		mysql_free_result(res);

		fprintf(output, "</table></p>\n");
	}

	for (x = 0; deplist[x]; x++)
		free(deplist[x]);

	aucHtmlSlimFoot();
	return (0);
}

void
copyclassinfo(aucCourseInfoPtr info, MYSQL_ROW data)
{
	char buf[128];
	info->id = (char *) strdup(data[0]);
	info->title = (char *) strdup(data[1]);
	info->teacher = (char *) strdup(data[2]);
	info->room = (char *) strdup(data[3]);
	info->department = (char *) strdup(data[4]);
	info->period = (char *) strdup(aucCoursePeriodName(atoi(data[5])));
	info->semester = strtoul(data[6], NULL, 10);
	info->time = (char *) strdup(data[7]);
	info->description = (char *) strdup(data[8]);

	snprintf(buf, sizeof(buf), "%s@%d", info->id, info->semester);
	info->uniquestr = strdup(buf);
}

void
aucCourseFreeInfo(aucCourseInfoPtr info)
{
	free(info->id);
	free(info->title);
	free(info->teacher);
	free(info->room);
	free(info->department);
	free(info->period);
	free(info->time);
	free(info->description);
	free(info->uniquestr);
}

int
aucCourseGetInfo(aucCourseInfoPtr clinfo, char *idstr, int semester)
{
	MYSQL_RES *res;
	MYSQL_ROW row;
	char buf[1024], id[32], *s;
	int val = -1;

	clinfo->id = NULL;

	snprintf(id, sizeof(id), "%s", idstr);
	if ((s = strchr(id, '@'))) {
		*s = '\0';
		semester = strtol(s + 1, NULL, 10);
	}

	if (!semester) {
		snprintf(buf, sizeof(buf), "id=\"%s\"", id);
		if (aucDBNumRows("classes", buf) > 1)
			semester = aucCurrSemester;
	}
	if (semester)
		snprintf(buf, sizeof(buf), "id=\"%s\" AND semester=%d", id, semester);

	if (!aucDBQuery(db, "SELECT * FROM classes WHERE %s", buf)) {
		res = mysql_store_result(db);
		if ((row = mysql_fetch_row(res))) {
			copyclassinfo(clinfo, row);
			val = 0;
			if (!strcmp(clinfo->teacher, aucUserCache->uname) ||
				aucAuthPermsof("sysop") || isaide(clinfo->uniquestr, aucUserCache->uname))
				clinfo->admin = 1;
			else
				clinfo->admin = 0;
		}
		mysql_free_result(res);
	}


	return val;
}

int
view_course_page(FILE * output, char *extrapath)
{
	aucCourseInfo clinfo;
	char idstr[512], *pathinfo;
	char buf[2048];
	int i;

	int classhead(FILE * output, char *path, char *name)
	{
		aucCourseCompactHeader(output, &clinfo, "File Listing");
		return 0;
	}
	int classfoot(FILE * output, char *path, char *name)
	{
		aucHtmlSlimFoot();
		return 0;
	}

	pathinfo = NULL;
	snprintf(idstr, sizeof(idstr), "%s", extrapath);

	for (i = 0; extrapath[i]; i++)
		if (idstr[i] == '/') {
			idstr[i] = '\0';
			pathinfo = &extrapath[i + 1];
			break;
		}


	if (!aucCourseGetInfo(&clinfo, idstr, 0)) {
		if (!exist_class_files(clinfo.uniquestr))
			create_class_files(clinfo.uniquestr);

		if (!pathinfo) {
			cgiHeaderContentType("text/html");
			course_page_header(output, clinfo);
			aucCourseNodeDrawAll(output, &clinfo);
			aucHtmlSlimFoot();
		} else {
			snprintf(buf, sizeof(buf), "course/%s", clinfo.uniquestr);
			aucViewPath(output, classhead, classfoot, _("Handouts and Documents"), buf, pathinfo,
						clinfo.admin?TREE_FORCEWRITE:TREE_READONLY, -1, 0);
		}
		aucCourseFreeInfo(&clinfo);
	} else {
		cgiHeaderContentType("text/html");
		aucHtmlSlimHead("%s", _("Error"));
		fprintf(output, "<h1>%s</h1>\n\n", _("Course Not Found"));
		fprintf(output, "<p><big><b>%s:</b> %s %s.  %s</big></p>\n\n", _("Error"),
				idstr, _("not found"),
				_("The requested course does not exist in the course database.  Please notify \
			the system administrator if it should exist."));
		aucHtmlSlimFoot();

	}
	return 0;
}

int
edit_course_aides_page(FILE * output, char *extrapath)
{
	int x;
	aucCourseInfo clinfo;
	char buffer[1024], buffer2[1024], uniquestr[32], *fields[32], *a;

	for (x = 0; x < 32; x++)
		fields[x] = NULL;

	if (cgiFormSuccess != cgiFormString("course", uniquestr, sizeof(uniquestr)))
		return -1;

	if (aucCourseGetInfo(&clinfo, uniquestr, 0))
		return -1;

	if (!clinfo.admin)
		return -2;

	if (!strcmp(cgiRequestMethod, "POST")) {
		aucHtmlReadForm("aides", fields, 0);
		strcpy(buffer2, "");
		snprintf(buffer, sizeof(buffer), "DELETE FROM classaides WHERE \
         idstr=\"%s\"", clinfo.uniquestr);
		if (mysql_query(db, buffer))
			aucError(mysql_error(db));
		for (a = fields[0]; *a; a += strlen(buffer2)) {
			while ((*a <= 32) && (*a != 0))
				a++;
			if (*a == 0)
				break;
			sscanf(a, "%s", buffer2);
			aucSanitizeUsername(buffer2);
			snprintf(buffer, sizeof(buffer), "INSERT INTO classaides \
            (idstr, aide) VALUES (\"%s\", \"%s\")", clinfo.uniquestr, buffer2);
			if (mysql_query(db, buffer))
				aucError(mysql_error(db));
		}
		snprintf(buffer, sizeof(buffer), "course/%s", aucLinkText(clinfo.uniquestr));
		cgiHeaderLocation(buffer);
	} else {

		cgiHeaderContentType("text/html");

		aucHtmlSlimHead("%s", _("Edit Course Aides"));

		aucGetFullPath("aide-edit-mess.html", aucDataDir, buffer, sizeof(buffer));
		aucHtmlEchoFile(cgiOut, buffer, 0);

		aucDrawTableForm(output, "POST", "aide-edit");
		aucDrawHidden(output, "course", clinfo.uniquestr);
		aucDrawTextarea(output, "aides", 10, 16,
						fields[0] ? fields[0] : getaides(clinfo.uniquestr, 0),
						_("Usernames of aides"));
		aucDrawSubmit(output, _("Update Aides"));
		aucDrawTableFormEnd(output);
		aucHtmlSlimFoot();
	}
	aucCourseFreeInfo(&clinfo);

	return 0;
}

char *
getaides(char *idstr, int verbose)
{
	MYSQL_RES *res;
	MYSQL_ROW row;
	char buf[512], buf2[512];
	char *ret;
	int count = 0;

	if (verbose)
		ret = (char *) strdup("none");
	else
		ret = (char *) strdup("");
	snprintf(buf, sizeof(buf), "SELECT aide FROM classaides WHERE \
      idstr=\"%s\"", idstr);
	if (mysql_query(db, buf)) {
		aucError(mysql_error(db));
		return ret;
	}
	res = mysql_store_result(db);
	while ((row = mysql_fetch_row(res))) {
		if (!count) {
			free(ret);
			ret = (char *) strdup("");
		} else if (verbose) {
			ret = (char *) realloc(ret, strlen(ret) + 6);
			strcat(ret, "<br>\n");
		} else if (!verbose) {
			ret = (char *) realloc(ret, strlen(ret) + 2);
			strcat(ret, "\n");
		}

		if (verbose)
			aucAuthGetFullName(buf2, sizeof(buf2), row[0], NAME_LINK);
		else
			strcpy(buf2, row[0]);
		ret = (char *) realloc(ret, strlen(ret) + strlen(buf2) + 1);
		strcat(ret, buf2);
		count++;
	}
	return ret;
}

int
printannouncement(FILE * output, char *uniquestr)
{
	MYSQL_RES *res;
	MYSQL_ROW row;
	char buf[512];

	snprintf(buf, sizeof(buf), "SELECT text, DATE_FORMAT(time, \"%s\") FROM \
      classannc WHERE idstr=\"%s\"", "%m/%d/%y %l:%i %p", uniquestr);
	if (mysql_query(db, buf)) {
		aucError(mysql_error(db));
		return -1;
	}

	res = mysql_store_result(db);
	if ((row = mysql_fetch_row(res))) {
		fprintf(output, "<div align=right><u><small>%s %s</small></u></div>\n", _("Updated"),
				row[1]);
		fprintf(output, "\n");
		aucPrintText(row[0], output, TEXT_BSPACES);
		fprintf(output, "<br>\n\n");
	} else
		fprintf(output, "<p><i>%s</i></p>\n", _("No Announcements"));

	mysql_free_result(res);
	return 0;
}

int
edit_course_annc_page(FILE * output, char *extrapath)
{
	MYSQL_RES *res = NULL;
	MYSQL_ROW row = NULL;
	int x, num;
	aucCourseInfo clinfo;
	char buffer[8192], buffer2[1024], uniquestr[32], *fields[4];

	for (x = 0; x < 4; x++)
		fields[x] = NULL;

	if (cgiFormSuccess != cgiFormString("course", uniquestr, sizeof(uniquestr)))
		return -1;

	if (aucCourseGetInfo(&clinfo, uniquestr, 0))
		return -1;

	if (!clinfo.admin)
		return -2;

	if (!strcmp(cgiRequestMethod, "GET")) {
		if (!aucDBQuery(db, "SELECT text FROM classannc WHERE idstr=\"%s\"", clinfo.uniquestr)) {
			res = mysql_store_result(db);
			if ((row = mysql_fetch_row(res)))
				for (x = 0, num = mysql_num_fields(res); x < num; x++)
					fields[x] = row[x];
		}
	} else {
		aucHtmlReadForm("text", fields, STR_ESCAPE);
		snprintf(buffer2, sizeof(buffer2), "DELETE FROM classannc WHERE \
         idstr=\"%s\"", clinfo.uniquestr);
		snprintf(buffer, sizeof(buffer), "INSERT INTO classannc (idstr, text, time) \
         VALUES (\"%s\", \"%s\", NOW())", clinfo.uniquestr, fields[0]);
		if (mysql_query(db, buffer2))
			aucError(mysql_error(db));
		if (strlen(fields[0]))
			if (mysql_query(db, buffer))
				aucError(mysql_error(db));
		snprintf(buffer, sizeof(buffer), "course/%s", aucLinkText(clinfo.uniquestr));
		cgiHeaderLocation(buffer);
		return 1;
	}

	cgiHeaderContentType("text/html");

	aucHtmlSlimHead("%s", _("Edit Class Announcement"));

	aucDrawTableForm(output, "POST", "courseannc-edit");
	aucDrawHidden(output, "course", clinfo.uniquestr);
	aucDrawTextarea(output, "text", 5, 50, fields[0], _("Announcement Text"));
	aucDrawSubmit(output, _("Save Announcement"));
	aucDrawTableFormEnd(output);
	aucHtmlSlimFoot();

	mysql_free_result(res);
	aucCourseFreeInfo(&clinfo);

	if (strcmp(cgiRequestMethod, "GET"))
		for (x = 0; fields[x]; x++)
			free(fields[x]);

	return 0;
}

int
add_course_page(FILE * output, char *extrapath)
{
	MYSQL_RES *res = NULL;
	MYSQL_ROW row = NULL;
	int edit = 0, nogood = 0;
	aucVarField vars[64];
	char buffer[1024], buffer2[1024], id[16], term[8];
	xmlDocPtr xmldoc;

	xmldoc = aucConfGetDoc();

	vars[0].name = strdup("form_action");
	vars[0].value = strdup("course-edit-POST");
	vars[1].name = strdup("actiontype");
	vars[1].value = strdup("add");
	vars[2].name = NULL;

	if (!strcmp(cgiRequestMethod, "GET")) {
		if ((cgiFormSuccess == cgiFormString("id", id, sizeof(id)))
			&& (cgiFormSuccess == cgiFormString("term", term, sizeof(term)))) {
			snprintf(buffer, sizeof(buffer),
					 "SELECT * FROM classes WHERE id=\"%s\" AND semester=%s", id, term);
			if (mysql_query(db, buffer))
				aucError(mysql_error(db));
			else {
				edit = 1;
				free(vars[1].value);
				vars[1].value = strdup("edit");
				res = mysql_store_result(db);
				row = mysql_fetch_row(res);
				aucDBStoreResults(vars, res, row);
				mysql_free_result(res);
			}
		}
	} else {
		edit = 1;
		free(vars[1].value);
		vars[1].value = strdup("edit");

		aucFormsXMLProcessFile(output, "addcourse.xml", vars, &xmldoc);

		aucDBEscapeResults(vars);

		snprintf(buffer2, sizeof(buffer2), "DELETE FROM classes WHERE \
         id=\"%s\" AND semester=\"%s\"", aucDBGetToken(vars, "oldid"), aucDBGetToken(vars, "oldsem"));

		aucSanitizeUsername(aucDBGetToken(vars, "teacher"));
		aucDBInsertQuery(buffer, sizeof(buffer), "classes", vars, "id", "title", "teacher",
						 "room", "department", "period", "semester", "classtime", "description",
						 NULL);
		if (strlen(aucDBGetToken(vars, "id")) && strlen(aucDBGetToken(vars, "semester"))) {
			char buf[64], buf2[64];
			char oldid[64], newid[64];

			if (mysql_query(db, buffer2))
				aucError(mysql_error(db));

			snprintf(oldid, sizeof(oldid), "%s@%s",
					 aucDBGetToken(vars, "oldid"), aucDBGetToken(vars, "oldsem"));
			snprintf(newid, sizeof(newid), "%s@%s",
					 aucDBGetToken(vars, "id"), aucDBGetToken(vars, "semester"));
			if (strcmp(oldid, newid)) {

				aucDBQuery(db, "UPDATE classaides SET idstr=\"%s\" WHERE idstr=\"%s\"", newid,
						   oldid);
				aucDBQuery(db, "UPDATE classannc SET idstr=\"%s\" WHERE idstr=\"%s\"", newid,
						   oldid);
				aucDBQuery(db, "UPDATE classsched SET idstr=\"%s\" WHERE idstr=\"%s\"", newid,
						   oldid);
				aucDBQuery(db, "UPDATE coursedisc SET idstr=\"%s\" WHERE idstr=\"%s\"", newid,
						   oldid);

				snprintf(buf, sizeof(buf), "%s/course/%s", aucVarRoot, oldid);
				snprintf(buf2, sizeof(buf2), "%s/course/%s", aucVarRoot, newid);

				aucSuRoot();
				rename(buf, buf2);
				aucSuUser();
			}

			if (mysql_query(db, buffer))
				aucError(mysql_error(db));
			cgiHeaderLocation("course-manager");
			return 1;
		} else
			nogood = 1;
	}

	cgiHeaderContentType("text/html");

	if (edit)
		aucHtmlSlimHead("%s", _("Edit Class"));
	else
		aucHtmlSlimHead("%s", _("Add New Class"));

	if (nogood)
		fprintf(output, "<p><b>%s:</b> %s</p>\n\n",
				_("Error"), _("You must specify an Identifier."));

	aucFormsXMLDisplayFile(output, "addcourse.xml", vars, xmldoc);

	aucHtmlSlimFoot();

	return 0;
}

int
delete_course_page(FILE * output, char *extrapath)
{
	char uniquestr[32], force[8];
	aucCourseInfo clinfo;

	if (cgiFormSuccess != cgiFormString("course", uniquestr, sizeof(uniquestr)))
		return -1;
	if (aucCourseGetInfo(&clinfo, uniquestr, 0))
		return -1;

	if (cgiFormSuccess != cgiFormString("force", force, sizeof(force))) {
		cgiHeaderContentType("text/html");
		aucHtmlSlimHead("%s", _("Confirmation"));
		fprintf(output, "<p>%s %s, <b>%s</b>?</p>\n",
				_("Are you sure you want to delete class"), clinfo.id, clinfo.title);
		fprintf(output, "<ul><p><b><a href=\"course-delete?force=yes&course=%s\">%s</a></b></p>\n",
				aucLinkText(clinfo.uniquestr), _("Yes"));
		fprintf(output, "<p><b><a href=\"course-manager\">%s</a></b></p></ul>\n", _("No"));
		aucHtmlSlimFoot();
	} else {

		aucDBQuery(db, "DELETE FROM classes WHERE id=\"%s\" AND semester=%d",
				   clinfo.id, clinfo.semester);
		aucDBQuery(db, "DELETE FROM classaides WHERE idstr=\"%s\"", clinfo.uniquestr);
		aucDBQuery(db, "DELETE FROM classannc WHERE idstr=\"%s\"", clinfo.uniquestr);
		aucDBQuery(db, "DELETE FROM classsched WHERE idstr=\"%s\"", clinfo.uniquestr);
		aucDBQuery(db, "DELETE FROM coursedisc WHERE idstr=\"%s\"", clinfo.uniquestr);

		cgiHeaderLocation("course-manager");
	}

	aucCourseFreeInfo(&clinfo);
	return 0;
}

int
course_page_header(FILE * output, aucCourseInfo clinfo)
{
	char buf[2048];
	FILE *image;

	if (!strcmp(aucScheduling, "periods"))
		aucHtmlSlimHead("%s", clinfo.title);
	else
		aucHtmlSlimHead("%s: %s", clinfo.id, clinfo.title);

	fprintf(output, "<p><table width=600 cellpadding=2 cellspacing=0 border=0 bgcolor=#%s><tr><td>\n\n", aucTrimColor);
	fprintf(output, "<table width=100%% cellpadding=2 cellspacing=0 border=0 bgcolor=#CCCCCC><tr>\n\n");

	snprintf(buf, sizeof(buf), "%s/classes/dep%c%c%c.gif",
			 aucGraphDirLocal, clinfo.department[0], clinfo.department[1], clinfo.department[2]);
	if ((image = fopen(buf, "r"))) {
		fprintf(output,
				"<td width=90><img src=\"%s/classes/dep%c%c%c.gif\" width=80 height=80 alt=\"%s\"></td>\n\n",
				aucGraphDir, clinfo.department[0], clinfo.department[1], clinfo.department[2],
				clinfo.department);
		fclose(image);
	}

	aucAuthGetFullName(buf, sizeof(buf), clinfo.teacher, NAME_LINK);

	if (!strcmp(aucScheduling, "periods"))
		fprintf(output, "<td><p><font size=+2 %s><b>%s</b></font><br>\n",
				HT_SANSSERIF, clinfo.title);
	else
		fprintf(output, "<td><p><font size=+2 %s><b>%s: %s</b></font><br>\n",
				HT_SANSSERIF, clinfo.id, clinfo.title);

	fprintf(output, "<font %s>%s: <b>%s</b></font></p></td>\n", HT_SANSSERIF, _("Instructor"), buf);

	fprintf(output, "<td><p><font %s>%s: %s<br>\n", HT_SANSSERIF, _("Location"), clinfo.room);

	if (!strcmp(aucScheduling, "periods"))
		fprintf(output, "%s: %s<br>\n", _("Period"), clinfo.period);
	else if (!strcmp(aucScheduling, "daytime"))
		fprintf(output, "%s: %s<br>\n", _("Time"), clinfo.time);

	if (clinfo.admin)
		fprintf(output, "<a href=\"%s/aide-edit?course=%s\">%s</a>:<br>%s</font></p></td>\n",
				aucPagesRoot, aucLinkText(clinfo.uniquestr), _("Online Aide(s)"),
				getaides(clinfo.uniquestr, 1));
	else
		fprintf(output, "%s:<br>%s</font></p></td>\n",
				_("Online Aide(s)"), getaides(clinfo.uniquestr, 1));
	fprintf(output, "</tr></table></td></tr></table></p>\n\n");
	return 0;
}

int
aucCourseCompactHeader(FILE * output, aucCourseInfoPtr clinfo, char *caption)
{
	aucHtmlSlimHead(clinfo->title);
	fprintf(output, "<table border=0 cellpadding=0 cellspacing=0 width=100%%>\n");
	fprintf(output, "<tr valign=top><td><big><b>%s</b></big></td>\n", caption);
	fprintf(output, "<td align=right><a href=\"%s/course/%s\">%s %s</a></td>\n",
			aucPagesRoot, aucLinkText(clinfo->uniquestr), _("Back to"), clinfo->title);
	fprintf(output, "</tr></table>\n\n");
	return 0;
}


int
exist_class_files(char *idstr)
{
	char buf[2048];
	struct stat fileinfo;

	snprintf(buf, sizeof(buf), "%s/course/%s", aucVarRoot, idstr);
	if (!stat(buf, &fileinfo))
		return 1;
	else
		return 0;
}

int
create_class_files(char *idstr)
{
	char buf[2048];
	FILE *outfile;
	int mask;

	aucSuRoot();
	mask = umask(022);

	snprintf(buf, sizeof(buf), "%s/course/%s/Handouts", aucVarRoot, idstr);
	if (aucCheckDir(buf, 0755, 0, 0)) {
		snprintf(buf, sizeof(buf), "%s/course/%s/Assignments", aucVarRoot, idstr);
		aucCheckDir(buf, 0755, 0, 0);
		snprintf(buf, sizeof(buf), "%s/course/%s/Web_Resources.txt", aucVarRoot, idstr);
		if ((outfile = fopen(buf, "w"))) {
			aucGetFullPath("class-init-weblinks.txt", aucDataDir, buf, sizeof(buf));
			aucHtmlEchoFile(outfile, buf, 0);
			fclose(outfile);
		}
	} else
		aucError("Could not create class files");

	umask(mask);
	aucSuUser();

	return 0;
}
