/*
 * Authenticated User Community
 * Copyright (c) 2000 David C. Moore
 *
 * You may distribute and/or use for any purpose modified or unmodified
 * copies of this software if you preserve the copyright notice above.
 *
 * coursenode.c
 *
 * Routines for the display of course nodes (feature panes in courses)
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <libxml/xpath.h>
#include <malloc.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>

#include <auc/cgic.h>
#include <auc/auc.h>
#include <auc/course.h>
#include <auc/util.h>

#include "coursenode.h"

aucCourseNodeListPtr nodes;

void drawnodeframe(FILE * output, aucCourseNodeInfoPtr node, aucCourseInfoPtr clinfo);

int
comp_priority_node(aucCourseNodeInfoPtr * n1, aucCourseNodeInfoPtr * n2)
{
	aucCourseNodeInfoPtr node1 = *n1;
	aucCourseNodeInfoPtr node2 = *n2;

	return node1->priority - node2->priority;
}

int
aucCourseNodeRegister(char *handle, char *title, aucCourseNodeDrawFunc drawfunc,
					  int priority, char *description, long unsigned int flags)
{
	aucCourseNodeInfoPtr info;
	nodes->len++;
	nodes->info =
		(aucCourseNodeInfoPtr *) realloc(nodes->info, sizeof(aucCourseNodeInfoPtr) * nodes->len);
	info = (aucCourseNodeInfoPtr) malloc(sizeof(aucCourseNodeInfo));

	info->handle = handle;
	info->title = title;
	info->drawfunc = drawfunc;
	info->priority = priority;
	info->description = description;
	info->flags = flags;

	nodes->info[nodes->len - 1] = info;

	qsort(nodes->info, nodes->len, sizeof(aucCourseNodeInfoPtr), (compareFunc) comp_priority_node);

	return 0;
}


int
aucCourseNodeInit(void)
{
	nodes = aucCourseNodeNewList();
	return 0;
}

void
aucCourseNodeRemoveListItem(aucCourseNodeListPtr list, int index)
{
	memmove(list->info + index, list->info + index + 1,
			(list->len - index - 1) * sizeof(aucCourseNodeInfoPtr));
	list->len--;
	list->info = realloc(list->info, list->len * sizeof(aucCourseNodeInfoPtr));
}

void
aucCourseNodeInsertListItem(aucCourseNodeListPtr list, int index,
							aucCourseNodeInfoPtr node, char *arg)
{
	list->len++;
	list->info = realloc(list->info, list->len * sizeof(aucCourseNodeInfoPtr));
	memmove(list->info + index + 1, list->info + index,
			(list->len - index - 1) * sizeof(aucCourseNodeInfoPtr));
	list->info[index] = node;
}

aucCourseNodeListPtr
aucCourseNodeNewList(void)
{
	aucCourseNodeListPtr list;

	list = (aucCourseNodeListPtr) malloc(sizeof(aucCourseNodeList));
	list->info = NULL;
	list->len = 0;
	return list;
}

aucCourseNodeListPtr
aucCourseNodeCopyList(aucCourseNodeListPtr list)
{
	aucCourseNodeListPtr newlist;

	newlist = aucCourseNodeNewList();
	newlist->info = (aucCourseNodeInfoPtr *) malloc(sizeof(aucCourseNodeInfoPtr) * list->len);
	memcpy(newlist->info, list->info, sizeof(aucCourseNodeInfoPtr) * list->len);
	newlist->len = list->len;

	return newlist;
}

void
aucCourseNodeFreeList(aucCourseNodeListPtr list)
{
	free(list->info);
	free(list);
}

void
drawnodecolstart(FILE * output)
{
	fprintf(output, "<table cellpadding=0 cellspacing=0 border=0 width=600>\n");
	fprintf(output, "<tr valign=top><td width=295>\n\n");
	fprintf(output, "<table cellpadding=1 cellspacing=0 border=0 width=295>\n");
	fprintf(output, "<tr valign=top><td bgcolor=%s width=295>\n\n", aucTrimColor);
}

void
drawnodecolmid(FILE * output)
{
	fprintf(output, "</td></tr></table></td>\n");
	fprintf(output, "<td width=10>&nbsp;</td>\n");
	fprintf(output, "<td width=295>\n");
	fprintf(output, "<table cellpadding=1 cellspacing=0 border=0 width=295>\n");
	fprintf(output, "<tr valign=top><td bgcolor=%s width=295>\n\n", aucTrimColor);
}

void
drawnodecolend(FILE * output)
{
	fprintf(output, "</td></tr></table></td></tr></table>\n\n");
}

int
aucCourseNodeDrawAll(FILE * output, aucCourseInfoPtr clinfo)
{
	aucCourseNodeInfoPtr node;
	int count = 0, x;

	drawnodecolstart(output);
	for (x = 0; x < nodes->len; x++) {
		node = nodes->info[x];
		drawnodeframe(output, node, clinfo);
		count++;
		if ((nodes->len - count) == (nodes->len / 2))
			drawnodecolmid(output);
	}
	drawnodecolend(output);


	return count;
}

void
drawnodeframe(FILE * output, aucCourseNodeInfoPtr node, aucCourseInfoPtr clinfo)
{
	fprintf(output, "<tr valign=top><td width=100%% bgcolor=%s>\n", aucTrimColor);
	fprintf(output, "<table cellpadding=2 cellspacing=0 border=0 width=100%%>\n");

	/* print title */
	fprintf(output, "<tr bgcolor=%s>\n", aucTrimColor);
	fprintf(output, "<td><b><font color=#FFFFFF>%s</font></b></td>\n", node->title);
	fprintf(output, "</tr>\n");

	fprintf(output, "<tr bgcolor=#FFFFFF><td>\n");

	(*(node->drawfunc)) (output, node, clinfo);

	fprintf(output, "</td></tr></table></td></tr>\n\n");

	fprintf(output, "<tr><td bgcolor=#FFFFFF><font size=-2>&nbsp;</font></td></tr>\n\n");
}
