/*
 * Authenticated User Community
 * Copyright (c) 2000 David C. Moore
 *
 * You may distribute and/or use for any purpose modified or unmodified
 * copies of this software if you preserve the copyright notice above.
 *
 * directory.c
 *
 * Routines pertaining to the student/staff directory
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <malloc.h>
#include <string.h>

#include <auc/cgic.h>
#include <auc/auc.h>
#include <auc/email.h>
#include <auc/pages.h>
#include <auc/database.h>
#include <auc/error.h>
#include <auc/html.h>
#include <auc/course.h>

#include "directory.h"

int checkfriend(char *);
int printpeoplelist(FILE *, char *);
void printsearchpane(FILE *, char *);

int
aucDirInit(void)
{
	aucNodelettRegister("people",
						_("People"),
						aucPeopleNodeFrame, 1, 180, 30,
						_("People or friends with which you interact"), "normal", 0);

	aucPagesRegister(NULL, "users", PAGE_DIRFUNC, aucViewDirectoryName, "normal", 0);
	aucPagesRegister(NULL, "dirinfo-edit", PAGE_FILE, aucAddDirInfo, "sysop", 0);
	aucPagesRegister(NULL, "directory-newfriend", PAGE_FILE, aucNewFriend, "normal", 0);
	aucPagesRegister(NULL, "directory-delfriend", PAGE_FILE, aucDelFriend, "normal", 0);
	aucPagesRegister(NULL, "directory-search", PAGE_FILE, aucSearchDir, "guest", 0);

	aucDBRegister("directory", "CREATE TABLE %s (uname VARCHAR(64) DEFAULT '' NOT NULL, \
         streetadd VARCHAR(64) NOT NULL, city VARCHAR(64) NOT NULL, \
         stateprov VARCHAR(30) NOT NULL, zip VARCHAR(10) NOT NULL, \
         phone CHAR(12) NOT NULL, email VARCHAR(64) NOT NULL, \
         homepage VARCHAR(128) NOT NULL, message TEXT NOT NULL, \
         PRIMARY KEY (uname))");

	aucDBRegister("peoplelinks", "CREATE TABLE %s (uname VARCHAR(64) DEFAULT '' NOT NULL, \
         friend VARCHAR(64) DEFAULT '' NOT NULL, PRIMARY KEY (uname,friend))");

	return 0;
}

void
aucPeopleNodeFrame(FILE * output, aucNodelettInfoPtr nodelett, char *arg)
{
	fprintf(output, "<table width=100%% height=%d cellpadding=2 cellspacing=0 border=0 bgcolor=#FFFFFF>\n",
			nodelett->height);
	fprintf(output, "<tr valign=top><td height=%d>\n", nodelett->height - 55);
	printpeoplelist(output, aucUserCache->uname);
	fprintf(output, "</td></tr>\n\n");

	fprintf(output, "<form action=\"directory-search\" method=POST>\n");
	fprintf(output, "<tr bgcolor=#CCCCCC><td height=55>\n");
	fprintf(output, "<small><a href=\"directory-search\">%s</a> %s:</small><br>\n", _("Search"), _("Student/Staff Directory"));
	fprintf(output, "<input type=text name=string size=11>\n");
	fprintf(output, "<input type=image name=search border=0 align=absmiddle src=\"%s/%s\" width=47 height=20 alt=\"%s\">\n",
			aucGraphDir, "nodeletts/searchbut.gif", _("Search"));
	fprintf(output, "</td></tr></form></table>\n\n");
}

int
aucSearchDir(FILE * output, char *extrapath)
{
	MYSQL_RES *res;
	MYSQL_ROW row;
	char string[1024], buf[1024], newstr[1024], *a, *b, uname[65], fullname[128];
	int count = 0, list = 0;

	if (!strcmp(cgiRequestMethod, "POST")) {
		if (cgiFormSuccess != cgiFormString("string", string, sizeof(string))) {
			cgiStaticContentType("text/html");
			aucHtmlSlimHead("%s", _("Invalid Search"));
			printsearchpane(output, NULL);
			fprintf(output, "<p><b>%s</b>: %s</p>\n",
					_("Error"), _("You did not specify a search string."));
			aucHtmlSlimFoot();
			return -1;
		}
		b = newstr;
		*b++ = '%';
		for (a = string; *a;)
			if (((*a <= 'z') && (*a >= 'a')) || ((*a <= 'Z') && (*a >= 'A'))) {
				*b++ = *a;
				a++;
			} else {
				*b++ = '%';
				while (*a && !(((*a <= 'z') && (*a >= 'a')) || ((*a <= 'Z') && (*a >= 'A'))))
					a++;
			}
		*b++ = '%';
		*b++ = '\0';

		snprintf(buf, sizeof(buf), "SELECT uname FROM userinfo WHERE \
         CONCAT(uname, fname, mname, lname) LIKE \"%s\" AND \
         FIND_IN_SET(grp, \"%s\") ORDER BY lname, fname", newstr, aucDirectoryGrps);
		if (mysql_query(db, buf))
			aucError(mysql_error(db));
		else {
			res = mysql_store_result(db);
			while ((row = mysql_fetch_row(res))) {
				count++;
				if (count == 2) {
					list = 1;
					cgiStaticContentType("text/html");
					aucHtmlSlimHead("%s", _("Search Results"));
					printsearchpane(output, string);
					fprintf(output, "<h2>%s</h2>\n\n", _("Possible Matches"));
					fprintf(output, "<ol>\n");
				}
				if (count > 1) {
					fprintf(output, "<li><a href=\"users/%s\">%s</a></li>\n", uname, fullname);
				}
				aucAuthGetFullName(fullname, sizeof(fullname), row[0], 0);
				snprintf(uname, sizeof(uname), "%s", row[0]);
			}
			mysql_free_result(res);
			if (count == 1) {
				snprintf(buf, sizeof(buf), "users/%s", uname);
				cgiHeaderLocation(buf);
				return 1;
			} else if (count > 1) {
				fprintf(output, "<li><a href=\"users/%s\">%s</a></li>\n", uname, fullname);
				fprintf(output, "</ol>\n\n");
				aucHtmlSlimFoot();
				return 0;
			} else {
				cgiStaticContentType("text/html");
				aucHtmlSlimHead("%s", _("Search Results"));
				printsearchpane(output, string);
				fprintf(output, "<h2>%s</h2>\n\n", _("Possible Matches"));
				fprintf(output, "<ol>%s</ol>\n\n", _("Sorry, but no matches were returned.  Please try a different search string."));
				aucHtmlSlimFoot();
				return 0;
			}
		}
	} else {
		cgiStaticContentType("text/html");
		aucHtmlSlimHead("%s", _("Search Student/Staff Directory"));

		fprintf(output, "<p>%s</p>\n\n", _("The student and staff directory can be searched \
by name.  In the space below, enter a search string using either part or all of someone's \
full name or username.  Any matching results will be returned."));

		printsearchpane(output, NULL);
		aucHtmlSlimFoot();
		return 0;
	}
	return 0;
}

void
printsearchpane(FILE * output, char *string)
{

	fprintf(output, "<form action=\"%s/directory-search\" method=POST>\n", aucPagesRoot);
	fprintf(output,
			"<blockquote>&nbsp;<img src=\"%s/fileops/magnify.gif\" border=0 align=absmiddle alt=\"\"> <b>%s</b><br>\n",
			aucGraphDir, _("Search Student/Staff Directory:"));
	fprintf(output, "<input type=text name=string size=30 value=\"%s\">\n", string ? string : "");
	fprintf(output, "<input type=submit value=Search></blockquote>\n");
	fprintf(output, "</form>\n\n");
}

int
printpeoplelist(FILE * output, char *uname)
{
	MYSQL_RES *res;
	MYSQL_ROW row;
	char buf[1024], fullname[128];
	int count = 0;
	struct dirinfotype dirinfo;

	fprintf(output, "<table cellpadding=0 cellspacing=0 border=0 width=170>\n\n");
	snprintf(buf, sizeof(buf), "SELECT friend, lname FROM peoplelinks LEFT OUTER JOIN \
      userinfo on peoplelinks.friend=userinfo.uname WHERE peoplelinks.uname=\"%s\" \
      ORDER BY userinfo.lname ASC, userinfo.fname ASC", uname);
	if (mysql_query(db, buf))
		aucError(mysql_error(db));
	else {
		res = mysql_store_result(db);
		while ((row = mysql_fetch_row(res))) {
			if (!row[1] || !strlen(row[1]))
				continue;
			count++;
			aucAuthGetFullName(fullname, sizeof(fullname), row[0], 0);
			aucGetDirInfo(&dirinfo, row[0]);
			fprintf(output, "<tr><td width=150><a href=\"users/%s\">%s</a></td>\n",
					row[0], fullname);
			fprintf(output, "<td width=20>");
			snprintf(buf, sizeof(buf), "<img src=\"%s/fileops/email.gif\" border=0 alt=\"(e)\">",
					 aucGraphDir);
			if ((dirinfo.email) && strlen(dirinfo.email))
				aucLinkComposeMessage(output, dirinfo.email, fullname, buf);
			fprintf(output, "</td></tr>\n\n");
		}
		mysql_free_result(res);
	}
	fprintf(output, "</table>\n\n");
	if (count == 0)
		fprintf(output, "<i>%s</i>\n\n", _("Empty List"));

	return count;
}

int
aucAddDirInfo(FILE * output, char *extrapath)
{
	MYSQL_RES *res = NULL;
	MYSQL_ROW row = NULL;
	int x, num, edit = 0, nogood = 0;
	char buffer[2048], uname[65], *fields[32];

	strcpy(uname, "");
	for (x = 0; x < 32; x++)
		fields[x] = NULL;

	if (!strcmp(cgiRequestMethod, "GET")) {
		if (cgiFormSuccess == cgiFormString("username", uname, sizeof(uname))) {
			snprintf(buffer, sizeof(buffer), "SELECT * FROM directory WHERE uname=\"%s\"", uname);
			if (mysql_query(db, buffer))
				aucError(mysql_error(db));
			else {
				edit = 1;
				res = mysql_store_result(db);
				if ((row = mysql_fetch_row(res)))
					for (x = 0, num = mysql_num_fields(res); x < num; x++)
						fields[x] = row[x];
			}
		}
	} else {
		aucHtmlReadForm("username streetadd city stateprov zip phone email homepage message",
						   fields, STR_ESCAPE);
		snprintf(buffer, sizeof(buffer), "REPLACE INTO directory \
         (uname, streetadd, city, stateprov, zip, phone, email, homepage, message) \
         VALUES (\"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\")", fields[0], fields[1], fields[2], fields[3], fields[4], fields[5], fields[6], fields[7], fields[8]);
		if (strlen(fields[0])) {
			if (mysql_query(db, buffer))
				aucError(mysql_error(db));
			cgiHeaderLocation("account-manager");
			return 0;
		} else
			nogood = 1;
	}

	cgiHeaderContentType("text/html");

	if (edit)
		aucHtmlSlimHead("%s", _("Edit Directory Information"));
	else
		aucHtmlSlimHead("%s", _("Add Directory Information"));

	if (nogood)
		fprintf(output, "<p><b>%s:</b> %s</p>\n\n", _("Error"), _("You must specify a Username."));

	aucDrawTableForm(output, "POST", "dirinfo-edit");
	if (strlen(uname)) {
		aucDrawHidden(output, "username", uname);
		fprintf(output, "<tr><td align=right>%s:</td><td><b>%s</b></td></tr>\n\n",
				_("Username"), uname);
	} else
		aucDrawTextbox(output, "username", 16, aucMaxUsernameLen, fields[0], _("Username"));
	aucDrawTextbox(output, "streetadd", 30, 64, fields[1], _("Street Address"));
	aucDrawTextbox(output, "city", 20, 64, fields[2], _("City"));
	aucDrawTextbox(output, "stateprov", 30, 30, fields[3], _("State/Province"));
	aucDrawTextbox(output, "zip", 10, 10, fields[4], _("Zip/Postal Code"));
	aucDrawTextbox(output, "phone", 12, 12, fields[5], _("Phone Number"));
/*   if (!fields[6] && strlen(uname))
      fields[6] = (char *) aucEmailClGetAddress(uname); */
	aucDrawTextbox(output, "email", 35, 64, fields[6], _("E-Mail Address"));
	aucDrawTextbox(output, "homepage", 35, 128, fields[7], _("Homepage"));
	aucDrawTextarea(output, "message", 4, 50, fields[8], _("Greeting Text"));
	if (edit)
		aucDrawSubmit(output, _("Save Entry"));
	else
		aucDrawSubmit(output, _("Add Entry"));
	aucDrawTableFormEnd(output);
	aucHtmlSlimFoot();

	if (edit)
		mysql_free_result(res);

	if (strcmp(cgiRequestMethod, "GET"))
		for (x = 0; fields[x]; x++)
			free(fields[x]);


	return 0;
}

int
aucViewDirectoryName(FILE * output, char *extrapath)
{
	MYSQL_RES *res;
	MYSQL_ROW row;
	char buf[1024], fullname[64], class[8], grp[32], lastlogin[64];
	struct dirinfotype dirinfo;
	int printdir = 1;

	cgiHeaderContentType("text/html");
	snprintf(buf, sizeof(buf), "SELECT class, grp, DATE_FORMAT(lastlogin, \"%s\") \
      FROM userinfo WHERE uname=\"%s\"", "%a, %d %b %Y, %l:%i %p", extrapath);
	if (mysql_query(db, buf)) {
		aucError(mysql_error(db));
		return 0;
	}
	res = mysql_store_result(db);
	if (!(row = mysql_fetch_row(res))) {
		aucHtmlSlimHead("%s", _("Directory Error"));
		fprintf(output, "<p>%s %s %s</p>\n", _("The username"), extrapath, _("was not found."));
		aucHtmlSlimFoot();
		return 1;
	}
	if (row[0])
		strcpy(class, row[0]);
	else
		strcpy(class, "");
	if (row[1])
		strcpy(grp, row[1]);
	else
		strcpy(grp, "");
	if (row[2])
		strcpy(lastlogin, row[2]);
	else
		sprintf(lastlogin, "<i>%s</i>", _("Never logged in"));
	mysql_free_result(res);

	aucGetDirInfo(&dirinfo, extrapath);

	aucAuthGetFullName(fullname, sizeof(fullname), extrapath, 0);

	aucHtmlSlimHead("%s: %s", _("User Info"), extrapath);

	fprintf(output, "<ul>\n");
	if (strcmp(aucUserCache->uname, extrapath)) {
		if (checkfriend(extrapath)) {
			fprintf(output, "<li><a href=\"%s/directory-delfriend?username=%s\">",
					aucPagesRoot, extrapath);
			fprintf(output, "%s %s %s</a></li>\n", _("Delete"), fullname, _("from my people list"));
		} else {
			fprintf(output, "<li><a href=\"%s/directory-newfriend?username=%s\">",
					aucPagesRoot, extrapath);
			fprintf(output, "%s %s %s</a></li>\n", _("Add"), fullname, _("to my people list"));
		}
	}

	if (aucAuthPermsof("sysop")) {
		fprintf(output, "<li><a href=\"%s/dirinfo-edit?username=%s\">", aucPagesRoot, extrapath);
		fprintf(output, _("Edit %s's directory information</a> (sysop-only)</li>"), fullname);
		fprintf(output, "<li><a href=\"%s/account-edit?username=%s\">", aucPagesRoot, extrapath);
		fprintf(output, _("Edit %s's account information</a> (sysop-only)</li>"), fullname);
	}
	fprintf(output, "</ul>\n\n");

	fprintf(output, "<ul>\n");
	fprintf(output, "<table cellpadding=2 cellspacing=0 border=0>\n");
	fprintf(output, "<tr><td bgcolor=%s>\n\n", aucTrimColor);

	fprintf(output, "<table cellpadding=3 cellspacing=0 border=0 width=500>\n");

	fprintf(output, "<tr bgcolor=%s>\n", aucTrimColor);
	fprintf(output, "<td><font color=#FFFFFF>%s: <b>%s</b></font></td>\n",
			_("User Info"), extrapath);
	fprintf(output, "</tr>\n\n");

	fprintf(output, "<tr bgcolor=#CCCCCC valign=top><td>\n\n");

	fprintf(output, "<table cellpadding=4 cellspacing=0 border=0 width=100%%>\n");
	fprintf(output, "<tr valign=top><td>\n");
	fprintf(output, "<p>%s: <b>%s</b><br>\n", _("Full Name"), fullname);
	fprintf(output, "%s: %s<br>\n", _("Role"), grp);
	if (strlen(class))
		fprintf(output, "%s %s<br>\n", _("Class of"), class);
	if ((dirinfo.email) && strlen(dirinfo.email)) {
		snprintf(buf, sizeof(buf), "%s <%s>", fullname, dirinfo.email);
		fprintf(output, "%s: ", _("Email"));
		aucLinkComposeMessage(output, dirinfo.email, fullname, dirinfo.email);
		fprintf(output, "<br>\n");
	}
	if ((dirinfo.homepage) && strlen(dirinfo.homepage)) {
		fprintf(output, "%s: ", _("WWW"));
		aucPrintText(dirinfo.homepage, output, TEXT_BSPACES);
		fprintf(output, "<br>\n");
	}
	fprintf(output, "</p>\n\n");


	fprintf(output, "</td><td>\n\n");


	fprintf(output, "<p>\n");
	if ((dirinfo.streetadd) && strlen(dirinfo.streetadd) && printdir) {
		fprintf(output, "%s<br>\n", dirinfo.streetadd);
		fprintf(output, "%s, %s %s<br><br>\n", dirinfo.city, dirinfo.stateprov, dirinfo.zip);
	}
	if ((dirinfo.phone) && strlen(dirinfo.phone) && printdir)
		fprintf(output, "%s: %s<br>\n", _("Phone"), dirinfo.phone);
	fprintf(output, "</p>\n\n");

	fprintf(output, "</td></tr></table>\n\n");

	fprintf(output, "<hr noshade size=1>\n\n");

	if ((dirinfo.message) && strlen(dirinfo.message)) {
		aucPrintText(dirinfo.message, output, TEXT_BSPACES);
		fprintf(output, "<hr noshade size=1>\n\n");
	}

	fprintf(output, "%s: %s<br>\n", _("Last Login"), lastlogin);

	fprintf(output, "%s:<div align=center>\n", _("Courses"));
	aucCoursePrintList(output, extrapath, aucCurrSemester);
	fprintf(output, "</div>\n");

	fprintf(output, "</td></tr></table>\n");
	fprintf(output, "</td></tr></table>\n\n");
	fprintf(output, "</ul>\n\n");

	printsearchpane(output, NULL);

	aucHtmlSlimFoot();
	return 1;
}

int
aucGetDirInfo(struct dirinfotype *dirinfo, char *uname)
{
	MYSQL_RES *res;
	MYSQL_ROW row;
	char buf[512];
	int flag = 0;

	dirinfo->uname = dirinfo->streetadd = dirinfo->city = dirinfo->stateprov = NULL;
	dirinfo->zip = dirinfo->phone = dirinfo->email = dirinfo->homepage = NULL;
	dirinfo->message = NULL;

	snprintf(buf, sizeof(buf),
			 "SELECT streetadd, city, stateprov, zip, phone, email, homepage, message \
      FROM directory WHERE uname=\"%s\"", uname);
	if (mysql_query(db, buf))
		aucError(mysql_error(db));
	else {
		res = mysql_store_result(db);
		if ((row = mysql_fetch_row(res))) {
			flag = 1;
			dirinfo->uname = (char *) strdup(uname);
			dirinfo->streetadd = (char *) strdup(row[0]);
			dirinfo->city = (char *) strdup(row[1]);
			dirinfo->stateprov = (char *) strdup(row[2]);
			dirinfo->zip = (char *) strdup(row[3]);
			dirinfo->phone = (char *) strdup(row[4]);
			dirinfo->email = (char *) strdup(row[5]);
			dirinfo->homepage = (char *) strdup(row[6]);
			dirinfo->message = (char *) strdup(row[7]);
		} else {
			dirinfo->uname = (char *) strdup(uname);
//         dirinfo->email = (char *) aucEmailClGetAddress(uname);
		}
		mysql_free_result(res);
	}
	return flag;
}

int
checkfriend(char *friend)
{
	MYSQL_RES *res;
	MYSQL_ROW row;
	char buf[1024];
	int found = 0;

	snprintf(buf, sizeof(buf), "SELECT COUNT(*) FROM peoplelinks WHERE \
      uname=\"%s\" AND friend=\"%s\"", aucUserCache->uname, friend);
	if (mysql_query(db, buf))
		aucError(mysql_error(db));
	else {
		res = mysql_store_result(db);
		row = mysql_fetch_row(res);
		if (!strcmp(row[0], "1"))
			found = 1;
		mysql_free_result(res);
	}
	return found;
}

int
aucNewFriend(FILE * output, char *extrapath)
{
	char buf[1024], friend[65];

	if (cgiFormSuccess == cgiFormString("username", friend, sizeof(friend))) {
		snprintf(buf, sizeof(buf), "REPLACE INTO peoplelinks (uname, friend) \
         VALUES (\"%s\", \"%s\")", aucUserCache->uname, friend);
		if (mysql_query(db, buf))
			aucError(mysql_error(db));
		snprintf(buf, sizeof(buf), "users/%s", friend);
		cgiHeaderLocation(buf);
	} else
		cgiHeaderContentType("text/html");

	return 1;
}

int
aucDelFriend(FILE * output, char *extrapath)
{
	char buf[1024], friend[65];

	if (cgiFormSuccess == cgiFormString("username", friend, sizeof(friend))) {
		snprintf(buf, sizeof(buf), "DELETE FROM peoplelinks WHERE \
         uname=\"%s\" AND friend=\"%s\"", aucUserCache->uname, friend);
		if (mysql_query(db, buf))
			aucError(mysql_error(db));
		snprintf(buf, sizeof(buf), "users/%s", friend);
		cgiHeaderLocation(buf);
	} else
		cgiHeaderContentType("text/html");

	return 1;
}
