/*
 * Authenticated User Community
 * Copyright (c) 2000 David C. Moore
 *
 * You may distribute and/or use for any purpose modified or unmodified
 * copies of this software if you preserve the copyright notice above.
 *
 * discussion.c
 *
 * Routines pertaining to AUC discussion forums
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>

#include <auc/cgic.h>
#include <auc/auc.h>
#include <auc/course.h>
#include <auc/coursenode.h>
#include <auc/pages.h>
#include <auc/database.h>
#include <auc/util.h>
#include <auc/error.h>
#include <auc/html.h>

#include "discussion.h"

void incrementhits(char *discid);
int getcommentcount(char *discid);
void getnewestcomment(char *discid, char *str, size_t len);
void discussion_node(FILE * output, aucCourseNodeInfoPtr node, aucCourseInfoPtr clinfo);
int comment_post_pane(FILE * output, char *name, char *text, char *cid, char *course);
int parse_comment_post(char *name, aucCourseInfoPtr clinfo);
int disc_admin_auth(char *discid);


int
aucDiscInit(void)
{
	aucPagesRegister(NULL, "discussion-edit", PAGE_FILE, aucDiscAdd, "normal", 0);
	aucPagesRegister(NULL, "disccourse-edit", PAGE_FILE, aucDiscCourseAdd, "normal", 0);
	aucPagesRegister(NULL, "discussion-delete", PAGE_FILE, aucDiscDelete, "normal", 0);
	aucPagesRegister(NULL, "discussion", PAGE_DIRFUNC, aucDiscPageView, "guest", 0);
	aucPagesRegister(NULL, "comment-post", PAGE_FILE, aucDiscCommentPost, "normal", 0);
	aucPagesRegister(NULL, "comment-delete", PAGE_FILE, aucDiscCommentDelete, "normal", 0);
	aucPagesRegister(NULL, "comment-modify", PAGE_FILE, aucDiscCommentModify, "normal", 0);

	aucDBRegister("discussions", "CREATE TABLE %s (discid VARCHAR(20) DEFAULT '' NOT NULL, \
         title VARCHAR(64), type VARCHAR(16), subtype VARCHAR(16), \
         description TEXT DEFAULT '' NOT NULL, \
         time DATETIME, hits MEDIUMINT NOT NULL, PRIMARY KEY (discid))");
	aucDBRegister("comments", "CREATE TABLE %s (discid VARCHAR(20) DEFAULT '' NOT NULL, \
         cid BIGINT UNSIGNED DEFAULT 0 NOT NULL AUTO_INCREMENT, \
         uname VARCHAR(64), time DATETIME, comment TEXT DEFAULT '' NOT NULL, \
         KEY autoinc (cid), PRIMARY KEY (discid,cid))");
	aucDBRegister("coursedisc", "CREATE TABLE %s (idstr VARCHAR(16) DEFAULT '' NOT NULL, \
			discid VARCHAR(20) DEFAULT '' NOT NULL, PRIMARY KEY (idstr,discid))");

	aucCourseNodeRegister("discussions",
						  _("Discussion Forums"),
						  discussion_node, 10, _("Interactive discussions about course topics"), 0);

	return 0;
}

void
discussion_node(FILE * output, aucCourseNodeInfoPtr node, aucCourseInfoPtr clinfo)
{
	int count;
	char buf[256];

	snprintf(buf, sizeof(buf), "LEFT OUTER JOIN coursedisc \
		ON coursedisc.discid=discussions.discid WHERE \
		coursedisc.idstr=\"%s\"", clinfo->uniquestr);
	count = aucDiscList(output, buf, clinfo->admin, clinfo);
	if (clinfo->admin)
		fprintf(output,
				"<small><b><a href=\"../disccourse-edit?course=%s&title=%s\">%s</a></b></small>\n",
				aucLinkText(clinfo->uniquestr), count ? "" : _("General%20Discussion"),
				_("Add Discussion"));
}

int
getcommentcount(char *discid)
{
	char buf[1024];

	snprintf(buf, sizeof(buf), "discid=\"%s\"", discid);
	return aucDBNumRows("comments", buf);
}

void
getnewestcomment(char *discid, char *str, size_t len)
{
	MYSQL_RES *res;
	MYSQL_ROW row;
	char buf[1024];

	snprintf(buf, sizeof(buf), "SELECT DATE_FORMAT(time, \"%s\") FROM comments \
      WHERE discid=\"%s\" ORDER BY cid DESC LIMIT 1", "%m/%d/%y %l:%i %p", discid);
	if (mysql_query(db, buf)) {
		aucError(mysql_error(db));
		return;
	}
	res = mysql_store_result(db);
	if ((row = mysql_fetch_row(res)))
		snprintf(str, len, "%s", row[0]);
	else
		snprintf(str, len, _("empty"));
	mysql_free_result(res);
}

int
aucDiscList(FILE * output, char *where, int admin, aucCourseInfoPtr clinfo)
{
	char buf[1024];
	MYSQL_RES *discres;
	MYSQL_ROW discrow;
	int count = 0;

	snprintf(buf, sizeof(buf), "SELECT discussions.discid, title, type, subtype, description, \
      DATE_FORMAT(time, \"%s\"), hits FROM discussions %s \
      ORDER BY discussions.discid", "%a, %d %b %Y", where);
	if (mysql_query(db, buf))
		aucError(mysql_error(db));
	else {
		discres = mysql_store_result(db);
		if ((discrow = mysql_fetch_row(discres)))
			while (discrow) {
				count++;
				getnewestcomment(discrow[0], buf, sizeof(buf));
				if (clinfo)
					fprintf(output,
							"<dt><b><a href=\"../discussion/%s?course=%s\">%s</a></b> <small>[%s]</small></dt>\n",
							discrow[0], aucLinkText(clinfo->uniquestr), discrow[1], buf);
				else
					fprintf(output,
							"<dt><b><a href=\"../discussion/%s\">%s</a></b> <small>[%s]</small></dt>\n",
							discrow[0], discrow[1], buf);
				fprintf(output, "<dd><small><b>%d %s</b> (%s %s)\n", getcommentcount(discrow[0]),
						_("comments"), discrow[6], _("hits"));
				if (admin) {
					if (clinfo) {
						fprintf(output,
								", <a href=\"%s/disccourse-edit?course=%s&discid=%s\">%s</a>,\n",
								aucPagesRoot, aucLinkText(clinfo->uniquestr), discrow[0],
								_("Edit"));
						fprintf(output,
								"<a href=\"%s/discussion-delete?course=%s&discid=%s\">%s</a>\n",
								aucPagesRoot, aucLinkText(clinfo->uniquestr), discrow[0], _("Del"));
					} else {
						fprintf(output, ", <a href=\"%s/discussion-edit?discid=%s\">%s</a>,\n",
								aucPagesRoot, discrow[0], _("Edit"));
						fprintf(output, "<a href=\"%s/discussion-delete?discid=%s\">%s</a>\n",
								aucPagesRoot, discrow[0], _("Del"));
					}
				}
				fprintf(output, "</small></dd><br>\n");
				discrow = mysql_fetch_row(discres);
		} else {
			fprintf(output, "<i>%s</i>\n\n",
					_("No discussions found.  Please ask your teacher to create one."));
		}
		mysql_free_result(discres);
	}

	return count;
}

void
incrementhits(char *discid)
{
	char buf[1024];

	snprintf(buf, sizeof(buf), "UPDATE discussions SET hits=hits+1 WHERE \
      discid=\"%s\"", discid);
	if (mysql_query(db, buf))
		aucError(mysql_error(db));
}

int
disc_admin_auth(char *discid)
{
	MYSQL_RES *res;
	MYSQL_ROW row;
	char buf[1024];
	int val = 0;

	snprintf(buf, sizeof(buf), "SELECT * FROM discussions WHERE discid=\"%s\"", discid);
	if (mysql_query(db, buf)) {
		aucError(mysql_error(db));
		return 0;
	} else {
		res = mysql_store_result(db);
		if ((row = mysql_fetch_row(res))) {
		}
		if (aucAuthPermsof("sysop"))
			val = 1;
		mysql_free_result(res);
	}
	return val;
}

int
aucDiscCommentDelete(FILE * output, char *extrapath)
{
	char buf[1024], discid[1024], cid[1024], course[32];
	aucCourseInfo clinfo;

	if (cgiFormSuccess != cgiFormString("discid", discid, sizeof(discid)))
		return -1;

	strcpy(course, "");
	if (cgiFormSuccess == cgiFormString("course", course, sizeof(course))) {
		if (aucCourseGetInfo(&clinfo, course, 0))
			return -1;
		snprintf(buf, sizeof(buf), "discid=\"%s\" AND idstr=\"%s\"", discid, clinfo.uniquestr);
		if (!aucDBNumRows("coursedisc", buf))
			return -1;
	}

	if (cgiFormSuccess != cgiFormString("cid", cid, sizeof(cid)))
		return -1;
	if (!((strlen(course) && clinfo.admin) || disc_admin_auth(discid)))
		return -2;

	snprintf(buf, sizeof(buf), "DELETE FROM comments WHERE discid=\"%s\" AND \
      cid=\"%s\"", discid, cid);
	mysql_query(db, buf);
	if (strlen(course)) {
		snprintf(buf, sizeof(buf), "discussion/%s?course=%s", discid,
				 aucLinkText(clinfo.uniquestr));
		aucCourseFreeInfo(&clinfo);
	} else
		snprintf(buf, sizeof(buf), "discussion/%s", discid);
	cgiHeaderLocation(buf);
	return 0;
}

int
aucDiscCommentModify(FILE * output, char *extrapath)
{
	MYSQL_RES *res;
	MYSQL_ROW row;
	char buf[1024], discid[1024], cid[1024], course[32];
	aucCourseInfo clinfo;

	if (cgiFormSuccess != cgiFormString("discid", discid, sizeof(discid)))
		return -1;

	strcpy(course, "");
	if (cgiFormSuccess == cgiFormString("course", course, sizeof(course))) {
		if (aucCourseGetInfo(&clinfo, course, 0))
			return -1;
		snprintf(buf, sizeof(buf), "discid=\"%s\" AND idstr=\"%s\"", discid, clinfo.uniquestr);
		if (!aucDBNumRows("coursedisc", buf))
			return -1;
	}

	if (cgiFormSuccess != cgiFormString("cid", cid, sizeof(cid)))
		return -1;
	if (!((strlen(course) && clinfo.admin) || disc_admin_auth(discid)))
		return -2;

	cgiHeaderContentType("text/html");

	snprintf(buf, sizeof(buf), "SELECT uname, DATE_FORMAT(time,\"%s\"), comment FROM comments \
      WHERE discid=\"%s\" AND cid=\"%s\"", "%a, %d %b %Y, %l:%i %p", discid, cid);
	if (mysql_query(db, buf)) {
		aucError(mysql_error(db));
		return -1;
	} else {
		res = mysql_store_result(db);
		if (!(row = mysql_fetch_row(res)))
			return -1;
		aucHtmlSlimHead("%s", _("Modify Comment"));
		aucAuthGetFullName(buf, sizeof(buf), row[0], NAME_LINK);
		fprintf(output, "<p>%s <b>%s</b> %s %s</p>\n\n", _("Posted by"), buf, _("on"), row[1]);
		comment_post_pane(output, discid, row[2], cid, course);
		aucHtmlSlimFoot();
	}
	if (strlen(course))
		aucCourseFreeInfo(&clinfo);
	return 0;
}

int
comment_post_pane(FILE * output, char *name, char *text, char *cid, char *course)
{
	if (cid)
		fprintf(output, "<p><form method=POST action=\"comment-post\">\n");
	else
		fprintf(output, "<p><form method=POST action=\"../comment-post\">\n");
	fprintf(output, "<input type=hidden name=discid value=\"%s\">\n", name);
	if (cid)
		fprintf(output, "<input type=hidden name=cid value=\"%s\">\n", cid);
	if (course && strlen(course))
		fprintf(output, "<input type=hidden name=course value=\"%s\">\n", course);
	fprintf(output, "<textarea name=text rows=5 cols=60 wrap=soft>");
	fprintf(output, "%s", text);
	fprintf(output, "</textarea><br>\n");
	if (cid)
		fprintf(output, "<input type=submit value=\"%s\"></form></p>\n\n", _("Repost Comment"));
	else
		fprintf(output, "<input type=submit value=\"%s\"></form></p>\n\n", _("Post Comment"));
	return 0;
}

int
parse_comment_post(char *name, aucCourseInfoPtr clinfo)
{
	char *query, *text, *text2, cid[1024];
	int size;

	if ((cgiFormSuccess == cgiFormStringSpaceNeeded("text", &size)) && (size > 1)) {
		text = (char *) malloc(size * sizeof(char));
		text2 = (char *) malloc((size * 2) * sizeof(char));
		cgiFormString("text", text, size);
		mysql_escape_string(text2, text, size - 1);
		query = (char *) malloc((size * 2 + 1024) * sizeof(char));
		if (cgiFormSuccess == cgiFormString("cid", cid, sizeof(cid))) {
			if (!((clinfo && clinfo->admin) || disc_admin_auth(name)))
				return -2;
			snprintf(query, size * 2 + 1024, "UPDATE comments SET comment=\"%s\" \
            WHERE discid=\"%s\" AND cid=\"%s\"", text2, name, cid);
		} else
			snprintf(query, size * 2 + 1024, "INSERT INTO comments (discid, uname, \
            time, comment) VALUES (\"%s\", \"%s\", NOW(), \"%s\")", name, aucUserCache->uname, text2);
		mysql_query(db, query);
		free(text);
		free(text2);
		free(query);
		return 1;
	}
	return 0;
}

int
aucDiscCommentPost(FILE * output, char *extrapath)
{
	char discid[1024], buffer[1024], course[32];
	aucCourseInfo clinfo;

	if (cgiFormSuccess != cgiFormString("discid", discid, sizeof(discid)))
		return -1;

	strcpy(course, "");
	if (cgiFormSuccess == cgiFormString("course", course, sizeof(course))) {
		if (aucCourseGetInfo(&clinfo, course, 0))
			return -1;
		snprintf(buffer, sizeof(buffer), "discid=\"%s\" AND idstr=\"%s\"",
				 discid, clinfo.uniquestr);
		if (!aucDBNumRows("coursedisc", buffer))
			return -1;
	}

	if (!strcmp(cgiRequestMethod, "POST")) {
		if (strlen(course)) {
			parse_comment_post(discid, &clinfo);
			snprintf(buffer, sizeof(buffer), "discussion/%s?course=%s",
					 discid, aucLinkText(clinfo.uniquestr));
			aucCourseFreeInfo(&clinfo);
		} else {
			parse_comment_post(discid, NULL);
			snprintf(buffer, sizeof(buffer), "discussion/%s", discid);
		}
		cgiHeaderLocation(buffer);
		return 0;
	} else
		return -1;
}

int
aucDiscPageView(FILE * output, char *extrapath)
{
	MYSQL_RES *res = NULL;
	MYSQL_ROW row = NULL;
	char buf[1024], caption[1024], code[1024], course[32];
	aucCourseInfo clinfo;
	int recs = 0;
	int maxcomments = 25, pageno = 1;

	strcpy(course, "");
	if (cgiFormSuccess == cgiFormString("course", course, sizeof(course))) {
		if (aucCourseGetInfo(&clinfo, course, 0))
			return -1;
		snprintf(buf, sizeof(buf), "discid=\"%s\" AND idstr=\"%s\"", extrapath, clinfo.uniquestr);
		if (!aucDBNumRows("coursedisc", buf))
			return -1;
	}

/*   if (!strcmp(cgiRequestMethod, "POST")) {
		if (strlen(course)) {
      	parse_comment_post(extrapath, &clinfo);
			snprintf(buf, sizeof(buf), "%s?course=%s", extrapath, aucLinkText(clinfo.uniquestr));
			aucClassesFreeInfo(&clinfo);
		} else {
      	parse_comment_post(extrapath, NULL);
			snprintf(buf, sizeof(buf), "%s", extrapath);
		}
      cgiHeaderLocation(buf);
      return 0;
   } */

	incrementhits(extrapath);

	cgiHeaderContentType("text/html");
	snprintf(buf, sizeof(buf), "SELECT discid, title, type, subtype, description, \
      DATE_FORMAT(time, \"%s\"), hits FROM discussions \
      WHERE discid=\"%s\"", "%a, %d %b %Y", extrapath);
	if (mysql_query(db, buf)) {
		aucError(mysql_error(db));
		return 0;
	}

	res = mysql_store_result(db);
	if ((row = mysql_fetch_row(res))) {
		strcpy(caption, row[1]);

		if (strlen(course))
			aucCourseCompactHeader(output, &clinfo, caption);
		else
			aucHtmlSlimHead("%s", caption);

		fprintf(output, "<small>&nbsp;&nbsp;&nbsp;%s %s.<br>\n", _("Created on"), row[5]);
		fprintf(output, "&nbsp;&nbsp;&nbsp;<i>%s %s %s</i>.</small>\n\n",
				_("This forum has been visited"), row[6], _("times"));
		if (strlen(row[4])) {
			fprintf(output, "<blockquote>%s</blockquote>\n\n", row[4]);
		}
	} else {
		aucHtmlSlimHead("%s", _("Error"));
		fprintf(output, "<h1>%s</h1>\n\n", _("Discussion not found"));
		fprintf(output, "<p>%s</p>\n\n",
				_("Sorry, the requested discussion could not be found in the database."));
		aucHtmlSlimFoot();
		mysql_free_result(res);
		return 0;
	}
	mysql_free_result(res);

	fprintf(output, "<blockquote>\n");
	if (aucAuthPermsof("normal"))
		comment_post_pane(output, extrapath, "", NULL, course);
	else
		fprintf(output, "%s\n",
				_("As a guest, you have the ability to read this forum but not post to it."));
	fprintf(output, "</blockquote>\n");

	recs = getcommentcount(extrapath);

	fprintf(output, "<p>\n");

	if (cgiFormSuccess == cgiFormString("page", buf, sizeof(buf)))
		pageno = strtol(buf, NULL, 10);
	else
		pageno = 1;

	if (strlen(course))
		snprintf(buf, sizeof(buf), "%s?course=%s", extrapath, aucLinkText(clinfo.uniquestr));
	else
		snprintf(buf, sizeof(buf), "%s", extrapath);
	aucPaginate(output, code, sizeof(code), buf, recs, maxcomments, pageno);

	snprintf(buf, sizeof(buf), "SELECT uname, DATE_FORMAT(time,\"%s\"), comment, cid \
      FROM comments WHERE discid=\"%s\" \
      ORDER BY cid DESC %s", "%a, %d %b %Y, %l:%i %p", extrapath, code);
	if (mysql_query(db, buf)) {
		aucError(mysql_error(db));
		return 0;
	} else {
		res = mysql_store_result(db);
		fprintf(output, "<hr>\n\n");
		while ((row = mysql_fetch_row(res))) {
			aucAuthGetFullName(buf, sizeof(buf), row[0], NAME_LINK);
			fprintf(output, "%s <b>%s</b> %s %s\n", _("From"), buf, _("on"), row[1]);
			if ((strlen(course) && clinfo.admin) || disc_admin_auth(extrapath)) {
				if (strlen(course))
					snprintf(buf, sizeof(buf), "&course=%s", aucLinkText(clinfo.uniquestr));
				else
					strcpy(buf, "");

				fprintf(output, "<br><a href=\"../comment-modify?discid=%s&cid=%s%s\">%s</a>\n",
						extrapath, row[3], buf, _("Modify"));
				fprintf(output, "<a href=\"../comment-delete?discid=%s&cid=%s%s\">%s</a>\n",
						extrapath, row[3], buf, _("Delete"));
			}
			fprintf(output, "<br>\n\n");
			fprintf(output, "<table border=0><tr valign=top><td>\n");
			aucPrintText(row[2], output, TEXT_BSPACES);
			fprintf(output, "</td></tr></table>\n");
			fprintf(output, "<hr>\n\n");
		}
		mysql_free_result(res);

		if (strlen(course))
			snprintf(buf, sizeof(buf), "%s?course=%s", extrapath, aucLinkText(clinfo.uniquestr));
		else
			snprintf(buf, sizeof(buf), "%s", extrapath);
		aucPaginate(output, code, sizeof(code), buf, recs, maxcomments, pageno);
	}
	fprintf(output, "</p>\n");

	aucHtmlSlimFoot();
	if (strlen(course))
		aucCourseFreeInfo(&clinfo);
	return 0;
}

int
aucDiscCourseAdd(FILE * output, char *extrapath)
{
	MYSQL_RES *res;
	MYSQL_ROW row;
	aucCourseInfo clinfo;
	char course[20], *courses[32], *coursenames[32], buf[1024], *fields[32];
	int coursestates[32], x, edit = 0, nogood = 0, num = 0;
	int invalid[32];
	char discid[32];

	for (x = 0; x < 32; x++)
		fields[x] = NULL;

	courses[0] = NULL;
	coursenames[0] = NULL;

	if (cgiFormSuccess != cgiFormString("course", course, sizeof(course)))
		return -1;
	if (aucCourseGetInfo(&clinfo, course, 0))
		return -1;
	if (!clinfo.admin)
		return -2;

	if (!aucDBQuery(db, "SELECT id, semester, title, period FROM classes WHERE teacher=\"%s\" \
			AND NOT (id=\"%s\" AND semester=%d)", clinfo.teacher, clinfo.id, clinfo.semester)) {
		res = mysql_store_result(db);
		x = 0;
		while ((row = mysql_fetch_row(res))) {
			snprintf(buf, sizeof(buf), "%s@%s", row[0], row[1]);
			courses[x] = strdup(buf);
			if (!strcmp(aucScheduling, "periods"))
				snprintf(buf, sizeof(buf), "%s: <i>%s</i>, %s %s, %s %s",
						 row[0], row[2], _("Period"), aucCoursePeriodName(strtol(row[3], NULL, 10)),
						 aucYearDivisionName, row[1]);
			else
				snprintf(buf, sizeof(buf), "%s: <i>%s</i>, %s %s",
						 row[0], row[2], aucYearDivisionName, row[1]);
			coursenames[x] = strdup(buf);
			coursestates[x] = 0;
			x++;
			courses[x] = NULL;
			coursenames[x] = NULL;
		}
		num = x;
		mysql_free_result(res);
	}

	if (!strcmp(cgiRequestMethod, "GET")) {
		if (cgiFormSuccess == cgiFormString("discid", discid, sizeof(discid))) {
			if (!aucDBQuery(db, "SELECT discid, title, description FROM discussions \
					WHERE discid=\"%s\"", discid)) {
				res = mysql_store_result(db);
				if ((row = mysql_fetch_row(res))) {
					edit = 1;
					fields[0] = strdup(row[0]);
					fields[1] = strdup(row[1]);
					fields[2] = strdup(row[2]);
				}
				mysql_free_result(res);
			}
		}
		if (edit) {
			for (x = 0; x < num; x++) {
				snprintf(buf, sizeof(buf), "discid=\"%s\" AND idstr=\"%s\"", discid, courses[x]);
				coursestates[x] = aucDBNumRows("coursedisc", buf);
			}
		}
		if (cgiFormSuccess == cgiFormString("title", buf, sizeof(buf))) {
			free(fields[1]);
			fields[1] = (char *) strdup(buf);
		}
	} else {

		cgiFormCheckboxMultiple("courses", courses, num, coursestates, invalid);

		aucHtmlReadForm("discid title description", fields, STR_ESCAPE);
		if (strlen(fields[0])) {
			snprintf(discid, sizeof(discid), "%s", fields[0]);
			snprintf(buf, sizeof(buf), "UPDATE discussions \
            SET title=\"%s\", description=\"%s\" WHERE discid=\"%s\"", fields[1], fields[2], discid);
		} else {
			snprintf(discid, sizeof(discid), "%s%lu", aucUserCache->uname, aucTime);
			snprintf(buf, sizeof(buf), "INSERT INTO discussions \
				(discid, title, description, time) \
            VALUES (\"%s\", \"%s\", \"%s\", NOW())", discid, fields[1], fields[2]);
		}
		if (strlen(fields[1])) {
			if (mysql_query(db, buf))
				aucError(mysql_error(db));
			aucDBQuery(db, "DELETE FROM coursedisc WHERE discid=\"%s\"", discid);
			for (x = 0; x < num; x++)
				if (coursestates[x])
					aucDBQuery(db, "INSERT INTO coursedisc (idstr, discid) VALUES \
						(\"%s\", \"%s\")", courses[x], discid);
			aucDBQuery(db, "INSERT INTO coursedisc (idstr, discid) VALUES \
					(\"%s\", \"%s\")", clinfo.uniquestr, discid);
			snprintf(buf, sizeof(buf), "course/%s", aucLinkText(clinfo.uniquestr));
			cgiHeaderLocation(buf);
			aucCourseFreeInfo(&clinfo);
			return 1;
		} else
			nogood = 1;
	}


	cgiHeaderContentType("text/html");

	if (edit)
		aucHtmlSlimHead("%s", _("Edit Discussion Information"));
	else
		aucHtmlSlimHead("%s", _("Add New Discussion"));

	if (nogood)
		fprintf(output, "<p><b>%s:</b> %s</p>\n\n", _("Error"), _("You must specify a Title"));

	aucDrawTableForm(output, "POST", "disccourse-edit");
	if (edit)
		aucDrawHidden(output, "discid", discid);
	aucDrawHidden(output, "course", clinfo.uniquestr);
	aucDrawTextbox(output, "title", 32, 64, fields[1], _("Title"));
	if (courses[0])
		aucDrawCheckMult(output, "courses", courses, coursenames, coursestates,
						 _
						 ("In addition to the current course, make the discussion available to these courses"));
	aucDrawTextarea(output, "description", 8, 50, fields[2], _("Intro. text (optional)"));
	if (edit)
		aucDrawSubmit(output, _("Update Information"));
	else
		aucDrawSubmit(output, _("Create Discussion"));
	aucDrawTableFormEnd(output);

	aucHtmlSlimFoot();
	aucCourseFreeInfo(&clinfo);
	return 0;
}

int
aucDiscAdd(FILE * output, char *extrapath)
{
	MYSQL_RES *res = NULL;
	MYSQL_ROW row = NULL;
	int x, num, edit = 0, nogood = 0;
	char buffer[8192], buffer2[1024];
	char discid[20], *fields[32];

	strcpy(discid, "");
	for (x = 0; x < 32; x++)
		fields[x] = NULL;

	if (!strcmp(cgiRequestMethod, "GET")) {
		if (cgiFormSuccess == cgiFormString("discid", discid, sizeof(discid))) {
			snprintf(buffer, sizeof(buffer),
					 "SELECT discid, title, type, subtype, description FROM discussions WHERE discid=\"%s\"",
					 discid);
			if (mysql_query(db, buffer))
				aucError(mysql_error(db));
			else {
				edit = 1;
				res = mysql_store_result(db);
				row = mysql_fetch_row(res);
				for (x = 0, num = mysql_num_fields(res); x < num; x++)
					fields[x] = row[x];
			}
		}
		if (cgiFormSuccess == cgiFormString("title", buffer, sizeof(buffer))) {
			free(fields[1]);
			fields[1] = (char *) strdup(buffer);
		}
	} else {
		aucHtmlReadForm("discid title type subtype description olddiscid", fields, STR_ESCAPE);

		snprintf(buffer2, sizeof(buffer2), "DELETE FROM discussions WHERE \
         discid=\"%s\"", fields[5]);
		if (!strcmp(fields[5], fields[0]))
			snprintf(buffer, sizeof(buffer), "UPDATE discussions \
            SET title=\"%s\", type=\"%s\", subtype=\"%s\", description=\"%s\" \
            WHERE discid=\"%s\"", fields[1], fields[2], fields[3], fields[4], fields[0]);
		else
			snprintf(buffer, sizeof(buffer),
					 "INSERT INTO discussions (discid, title, type, subtype, description, time) \
            VALUES (\"%s\", \"%s\", \"%s\", \"%s\", \"%s\", NOW())", fields[0], fields[1], fields[2], fields[3], fields[4]);
		if (strlen(fields[0]) && strlen(fields[1])) {
			if (strcmp(fields[5], fields[0]) && mysql_query(db, buffer2))
				aucError(mysql_error(db));
			if (mysql_query(db, buffer))
				aucError(mysql_error(db));
			cgiHeaderLocation("discussion-manager");
			return 0;
		} else
			nogood = 1;
	}

	cgiHeaderContentType("text/html");

	if (edit) {
		aucHtmlSlimHead("%s", _("Edit Discussion Information"));
	} else {
		aucHtmlSlimHead("%s", _("Add New Discussion"));
	}

	if (nogood)
		fprintf(output, "<p><b>%s:</b> %s</p>\n\n",
				_("Error"), _("You must specify a Discussion ID and Title"));

	aucDrawTableForm(output, "POST", "discussion-edit");
	aucDrawHidden(output, "olddiscid", fields[5] ? fields[5] : fields[0]);
	aucDrawTextbox(output, "discid", 20, 20, fields[0], _("Discussion ID"));
	aucDrawTextbox(output, "title", 32, 64, fields[1], _("Name"));
	aucDrawTextbox(output, "type", 16, 16, fields[2], _("Type"));
	aucDrawTextbox(output, "subtype", 16, 16, fields[3], _("Subtype"));
	aucDrawTextarea(output, "description", 15, 50, fields[4], _("Intro. text (optional)"));
	if (edit)
		aucDrawSubmit(output, _("Update Information"));
	else
		aucDrawSubmit(output, _("Create Discussion"));
	aucDrawTableFormEnd(output);


	aucHtmlSlimFoot();

	if (edit)
		mysql_free_result(res);

	if (strcmp(cgiRequestMethod, "GET"))
		for (x = 0; fields[x]; x++)
			free(fields[x]);


	return 0;

}

int
aucDiscDelete(FILE * output, char *extrapath)
{
	MYSQL_RES *res;
	MYSQL_ROW row;
	aucCourseInfo clinfo;
	char discid[32], url[64], buf[512], force[8], course[32];
	int deleteall = 0;

	if (cgiFormSuccess != cgiFormString("discid", discid, sizeof(discid)))
		return -1;

	strcpy(course, "");
	if (cgiFormSuccess == cgiFormString("course", course, sizeof(course))) {
		if (aucCourseGetInfo(&clinfo, course, 0))
			return -1;
		snprintf(buf, sizeof(buf), "discid=\"%s\" AND idstr=\"%s\"", discid, clinfo.uniquestr);
		if (!aucDBNumRows("coursedisc", buf))
			return -1;
	}

	if (!((strlen(course) && clinfo.admin) || disc_admin_auth(discid)))
		return -2;

	if (cgiFormSuccess != cgiFormString("force", force, sizeof(force))) {
		if (aucDBQuery(db, "SELECT title FROM discussions WHERE discid=\"%s\"", discid))
			return -1;
		else {
			res = mysql_store_result(db);

			if ((row = mysql_fetch_row(res)))
				snprintf(buf, sizeof(buf), "%s", row[0]);
			else
				return -1;

			mysql_free_result(res);
		}

		cgiHeaderContentType("text/html");
		aucHtmlSlimHead("%s", _("Confirmation"));
		if (strlen(course)) {
			fprintf(output, "<p>%s \"%s\" %s %s, %s (%s)?</p>\n",
					_("Are you sure you want to delete"), buf, _("from course"),
					clinfo.id, clinfo.title,
					_("any occurences of this discussion in other courses will not be removed"));
			fprintf(output,
					"<ul><p><b><a href=\"discussion-delete?force=yes&discid=%s&course=%s\">%s</a></b></p>\n",
					discid, aucLinkText(clinfo.uniquestr), _("Yes"));
		} else {
			fprintf(output, "<p>%s \"%s\"?</p>\n", _("Are you sure you want to delete"), buf);
			fprintf(output,
					"<ul><p><b><a href=\"discussion-delete?force=yes&discid=%s\">%s</a></b></p>\n",
					discid, _("Yes"));
		}
		fprintf(output, "</ul>\n");
		aucHtmlSlimFoot();
		return 0;
	}

	if (strlen(course)) {
		snprintf(buf, sizeof(buf), "discid=\"%s\"", discid);
		if (aucDBNumRows("coursedisc", buf) == 1)
			deleteall = 1;
		aucDBQuery(db, "DELETE FROM coursedisc WHERE idstr=\"%s\" AND discid=\"%s\"",
					   clinfo.uniquestr, discid);
		snprintf(url, sizeof(url), "course/%s", aucLinkText(clinfo.uniquestr));
	} else {
		deleteall = 1;
		snprintf(url, sizeof(url), "discussion-manager");
	}

	if (deleteall) {
		aucDBQuery(db, "DELETE FROM discussions WHERE discid=\"%s\"", discid);
		aucDBQuery(db, "DELETE FROM comments WHERE discid=\"%s\"", discid);
	}

	cgiHeaderLocation(url);
	return 0;
}
