/*
 * Authenticated User Community
 * Copyright (c) 2000 David C. Moore
 *
 * You may distribute and/or use for any purpose modified or unmodified
 * copies of this software if you preserve the copyright notice above.
 *
 * email-pages.c
 *
 * Functions that display various pages in the web-based email
 * client
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <cgic.h>
#include <malloc.h>
#include <string.h>

#include <auc/auc.h>
#include <auc/files.h>
#include <auc/html.h>
#include <auc/auth.h>
#include <auc/util.h>
#include <auc/pages.h>
#include <auc/conf.h>

#include "email.h"

char *aucSMTPHost = "localhost";
char *aucMailTrashFolder = N_("Trash");
char *aucMailPineUserConfig = ".pinerc";
char *aucMailPostponedFolder = N_("postponed-msgs");
char *aucMailSavedFolder = N_("saved-messages");
char *aucMailSystem = "E-Mail";
int aucMailSourceType = LOCALSPOOL;
int aucMailAdvancedPrefs = 0;
int aucMailDebug = 0;

int process_init_error(FILE * output, int error, char *currpage);

int
aucEmailInit(void)
{
	aucPagePtr dir;
	char *buf;

	aucConfToken("child::Email/SMTPServer", &aucSMTPHost);
	aucConfToken("child::Email/TrashFolder", &aucMailTrashFolder);
	aucConfToken("child::Email/PineUserConfig", &aucMailPineUserConfig);
	aucConfToken("child::Email/PostponedFolder", &aucMailPostponedFolder);
	aucConfToken("child::Email/SavedFolder", &aucMailSavedFolder);
	aucConfToken("child::Email/MailSystemName", &aucMailSystem);

	if (!aucConfToken("child::Email/IMAPLogging", &buf)) {
		if (!strcmp(buf, "yes"))
			aucMailDebug = 1;
		free(buf);
	}

	if (!aucConfToken("child::Email/attribute::source", &buf)) {
		if (!strcmp(buf, "imap"))
			aucMailSourceType = IMAPSPOOL;
		else if (!strcmp(buf, "local"))
			aucMailSourceType = LOCALSPOOL;
		else
			aucMailSourceType = ALLLOCAL;
		free(buf);
	}

	if (!aucConfToken("child::Email/attribute::advancedprefs", &buf)) {
		if (!strcmp(buf, "enabled"))
			aucMailAdvancedPrefs = 1;
		free(buf);
	}

	aucNodelettRegister("email",
						_("E-Mail"),
						aucEmailClPrintNodeFrame, 1, 180, 10,
						_("This gives you access to your email stored on the server."),
						"normal", 0);

	dir = aucPagesRegisterSubDir(NULL, "email", "view", "normal", 0);
	aucPagesRegister(dir, "view", PAGE_FILE, EmailView, "normal", 0);
	aucPagesRegister(dir, "inject", PAGE_FILE, EmailInject, "normal", 0);
	aucPagesRegister(dir, "move", PAGE_FILE, EmailMove, "normal", 0);
	aucPagesRegister(dir, "compose", PAGE_FILE, EmailCompose, "normal", 0);
	aucPagesRegister(dir, "emptrash", PAGE_FILE, EmailEmpTrash, "normal", 0);
	aucPagesRegister(dir, "options", PAGE_FILE, EmailOptions, "normal", 0);
	aucPagesRegister(dir, "folders", PAGE_FILE, EmailFolders, "normal", 0);
	aucPagesRegister(dir, "delfolder", PAGE_FILE, EmailDelFolder, "normal", 0);
	aucPagesRegister(dir, "newfolder", PAGE_FILE, EmailNewFolder, "normal", 0);
	aucPagesRegister(dir, "firstnew", PAGE_FILE, aucEmailClFirstNew, "normal", 0);
	aucPagesRegister(dir, "attach", PAGE_DIRFUNC, EmailAttach, "normal", 0);
	aucPagesRegister(dir, "new-imap-login", PAGE_FILE, EmailNewLogin, "normal", 0);

	return 0;
}

int
EmailView(FILE * output, char *extrapath)
{
	int flags = 0, page = 1;
	char type[128], show[128], mailbox[128], buffer[128];
	unsigned int uid;
	int err;

	strcpy(type, "index");
	strcpy(show, "all");
	strcpy(mailbox, "INBOX");

	if (cgiFormSuccess == cgiFormInteger("UID", &uid, 0))
		strcpy(type, "message");
	if (cgiFormSuccess == cgiFormString("mailbox", buffer, sizeof(buffer)))
		strcpy(mailbox, buffer);
	if (cgiFormSuccess == cgiFormString("show", buffer, sizeof(buffer)))
		strcpy(show, buffer);

	if (strcmp(type, "index") == 0) {

		cgiStaticContentType("text/html");
		aucHtmlSlimHead(mailbox);
		flags = QN_MAILBOX;
		if (cgiFormSuccess == cgiFormStringNoNewlines("sort", buffer, sizeof(buffer)))
			if (!strcmp(buffer, "forward"))
				flags = flags | QN_FORWARDSORT;

		page = 1;
		if (cgiFormSuccess == cgiFormString("page", buffer, sizeof(buffer)))
			page = strtoul(buffer, NULL, 10);

		if (!(err = aucEmailClInit(mailbox, aucMailDebug))) {
			aucEmailClHeader(output, page, flags);
			aucEmailClPrintList(output, page, flags);
			aucEmailClFooter(output, flags);
			aucEmailClClose();
		} else
			process_init_error(output, err, "view");
		aucHtmlSlimFoot();
	} else if (strcmp(type, "message") == 0) {

		cgiStaticContentType("text/html");

		if (!strcmp(show, "printable"))
			flags = QN_SINGLE | QN_PRINTABLE;
		else if (!strcmp(show, "raw"))
			flags = QN_SINGLE | QN_RAWTEXT;
		else
			flags = QN_SINGLE;

		if (!(flags & QN_PRINTABLE) && !(flags & QN_RAWTEXT))
			aucHtmlSlimHead(mailbox);
		else
			aucHtmlBlankHead(mailbox);

		if (!(err = aucEmailClInit(mailbox, aucMailDebug))) {
			if (!(flags & QN_PRINTABLE) && !(flags & QN_RAWTEXT))
				aucEmailClHeader(output, 1, flags);
			aucEmailClPrintMessage(output, uid, flags);
			if (!(flags & QN_PRINTABLE) && !(flags & QN_RAWTEXT))
				aucEmailClFooter(output, flags);
			aucEmailClClose();
		} else
			process_init_error(output, err, "view");

		if (!(flags & QN_PRINTABLE) && !(flags & QN_RAWTEXT))
			aucHtmlSlimFoot();
		else
			aucHtmlBlankFoot();
	}
	return 0;
}


int
EmailInject(FILE * output, char *extrapath)
{
	int flags = 0, dosetflag = 0;
	char type[128], show[128], mailbox[128], buffer[256], *to, *cc, *subject, *text, *inreplyto;
	size_t to_len, cc_len, subject_len, text_len, inreplyto_len;
	unsigned int uid;

	strcpy(type, "index");
	strcpy(show, "new");
	strcpy(mailbox, "INBOX");

	cgiStaticContentType("text/html");

	if (cgiFormSuccess == cgiFormString("postpone", buffer, sizeof(buffer))) {
		aucHtmlSlimHead("%s", _("Postponing Message"));
		flags = QN_POSTPONE;
	} else
		aucHtmlSlimHead("%s", _("Sending Message"));

	if (cgiFormSuccess == cgiFormStringSpaceNeeded("to", &to_len) && to_len > 1) {
		to = (char *) malloc(to_len * sizeof(char));
		cgiFormStringNoNewlines("to", to, to_len);
	} else {
		fprintf(output, "<p><b>%s:</b> %s</p>\n", _("Error"), _("To: field not specified"));
		aucHtmlSlimFoot();
		return (1);
	}
	if (cgiFormSuccess == cgiFormStringSpaceNeeded("text", &text_len) && text_len > 1) {
		text = (char *) malloc(text_len * sizeof(char));
		cgiFormString("text", text, text_len);
	} else {
		fprintf(output, "<p><b>%s:</b> %s</p>\n", _("Error"), _("Message text empty"));
		aucHtmlSlimFoot();
		return (1);
	}

	if (cgiFormSuccess == cgiFormStringSpaceNeeded("cc", &cc_len) && cc_len > 1) {
		cc = (char *) malloc(cc_len * sizeof(char));
		cgiFormStringNoNewlines("cc", cc, cc_len);
	} else
		cc = (char *) strdup("");

	if (cgiFormSuccess == cgiFormStringSpaceNeeded("inreplyto", &inreplyto_len)
		&& inreplyto_len > 1) {
		inreplyto = (char *) malloc(inreplyto_len * sizeof(char));
		cgiFormStringNoNewlines("inreplyto", inreplyto, inreplyto_len);
	} else
		inreplyto = strdup("");

	if (cgiFormSuccess == cgiFormStringSpaceNeeded("subject", &subject_len) && subject_len > 1) {
		subject = (char *) malloc(subject_len * sizeof(char));
		cgiFormStringNoNewlines("subject", subject, subject_len);
	} else
		subject = (char *) strdup("no subject");

	if ((cgiFormSuccess == cgiFormInteger("ansmesg", &uid, 0))
		&& (cgiFormSuccess == cgiFormString("ansbox", buffer, sizeof(buffer))))
		dosetflag = 1;
	else
		strcpy(buffer, "INBOX");


	if (!aucEmailClInit(buffer, aucMailDebug)) {
		if (!aucEmailClInject(output, to, cc, subject, text, inreplyto, flags)) {
			fprintf(output, "<ul><li><a href=\"view\">%s</a></li>\n", _("Back to INBOX"));
			fprintf(output, "<li><a href=\"compose\">%s</a></li></ul>\n", _("Compose Message"));
			if (dosetflag)
				aucEmailClSetAnswered(output, uid);

		}
		aucEmailClClose();
	} else
		process_init_error(output, -1, "inject");


	free(to);
	free(cc);
	free(subject);
	free(text);
	free(inreplyto);

	aucHtmlSlimFoot();
	return 0;
}


int
EmailMove(FILE * output, char *extrapath)
{
	int size = 8192, i = 0;
	char source[128], dest[128], buffer[128], sequence[8192];
	char **delete;

	cgiFormString("dest", buffer, sizeof(buffer));
	snprintf(dest, sizeof(dest), "%s", buffer);
	cgiFormString("source", source, sizeof(source));
	if (cgiFormSuccess == cgiFormStringMultiple("mess", &delete)) {
		strcpy(sequence, "");
		while (delete[i]) {
			strncat(sequence, delete[i], size);
			size -= strlen(delete[i]);
			if (delete[i + 1])
				strncat(sequence, ",", size);
			size--, i++;
		}
		cgiStringArrayFree(delete);

		if (!aucEmailClInit(source, aucMailDebug)) {
			if (aucEmailClMoveTo(dest, sequence)) {
				cgiStaticContentType("text/html");
				aucHtmlSlimHead("%s", _("Error in Move"));
				fprintf(output, "<p><b>%s:</b> %s</p>",
						_("Error"), _("An internal error prevented the message from being moved"));
				aucHtmlSlimFoot();
				aucEmailClClose();
				return -1;
			}
			aucEmailClClose();
		} else {
			cgiStaticContentType("text/html");
			aucHtmlSlimHead("%s", _("Error in Move"));
			process_init_error(output, -1, "move");
			aucHtmlSlimFoot();
			return -1;
		}
	}
	if (cgiFormSuccess == cgiFormString("next", dest, sizeof(dest)))
		snprintf(buffer, sizeof(buffer), "view?mailbox=%s&UID=%s", aucLinkText(source), dest);
	else
		snprintf(buffer, sizeof(buffer), "view?mailbox=%s", aucLinkText(source));
	cgiHeaderLocation(buffer);
	return 0;
}


int
EmailCompose(FILE * output, char *extrapath)
{
	int flags = 0;
	char type[128], mailbox[128], to[512], buffer[128];
	int uid = -1;
	int err;

	strcpy(type, "compose");
	strcpy(mailbox, "INBOX");

	if (cgiFormSuccess != cgiFormString("to", to, sizeof(to)))
		strcpy(to, "");
	if (cgiFormSuccess == cgiFormInteger("UID", &uid, -1))
		strcpy(type, "replyfull");
	if (cgiFormSuccess == cgiFormString("mailbox", buffer, sizeof(buffer)))
		strcpy(mailbox, buffer);
	if (cgiFormSuccess == cgiFormString("type", buffer, sizeof(buffer)))
		strcpy(type, buffer);

	cgiStaticContentType("text/html");

	if (!(err = aucEmailClInit(mailbox, aucMailDebug)) && (uid >= 0)) {
		if (!strcmp(type, "forward"))
			flags = QN_FORWARD | QN_COMPOSE;
		else if (!strcmp(type, "replyfull"))
			flags = QN_REPLY | QN_REPLYFULL;
		else
			flags = QN_REPLY;
		aucHtmlSlimHead("%s", _("Reply to Message"));
		aucEmailClHeader(output, 1, flags);
		aucEmailClCompose(output, to, uid, flags);
		aucEmailClFooter(output, flags);
		aucEmailClClose();
		aucHtmlSlimFoot();
	} else if (!err) {
		aucHtmlSlimHead("%s", _("Compose Message"));
		aucEmailClHeader(output, 1, QN_COMPOSE);
		aucEmailClCompose(output, to, -1, 0);
		aucEmailClFooter(output, QN_COMPOSE);
		aucEmailClClose();
		aucHtmlSlimFoot();
	} else {
		aucHtmlSlimHead("%s", _("Error"));
		process_init_error(output, err, "compose");
		aucHtmlSlimFoot();
	}

	return 0;
}


int
EmailEmpTrash(FILE * output, char *extrapath)
{
	char buffer[256], mailbox[256];

	strcpy(mailbox, "INBOX");

	if (cgiFormSuccess == cgiFormString("mailbox", buffer, sizeof(buffer)))
		strcpy(mailbox, buffer);

	if (!aucEmailClInit(mailbox, aucMailDebug)) {
		if (!aucEmailClEmptyTrash(output)) {
			cgiStaticContentType("text/html");
			aucHtmlSlimHead("%s", _("Trash Emptied"));
			fprintf(output, "<p>%s</p>\n", _("The trash has been emptied successfully."));
			fprintf(output, "<p><a href=\"view\">%s</a></p>\n", _("Back to INBOX"));
			aucHtmlSlimFoot();
		} else {
			cgiStaticContentType("text/html");
			aucHtmlSlimHead("%s: %s", _("Error"), _("trash not emptied"));
			fprintf(output, "<p><b>%s:</b> %s</p>\n", _("Error"),
					_("There was an error emptying the trash"));
			aucHtmlSlimFoot();
		}
		aucEmailClClose();
	}
	return 0;
}


int
EmailOptions(FILE * output, char *extrapath)
{
	char buf[256], buf2[256], *sig;
	int level = 0, force = 0, post = 0, space;
	char *options[64], **currname;
	struct pinerckey pinerc[64], *currkey;

	if (cgiFormSuccess == cgiFormString("type", buf, sizeof(buf))) {
		if (!strcmp(buf, _("advanced")))
			level = 1;
		else
			force = 1;
	}

	if (!aucMailAdvancedPrefs && !aucAuthPermsof("sysop"))
		level = 0;

	if (!strcmp(cgiRequestMethod, "POST")) {
		aucEmailClShutdown();
		post = 1;

		if (cgiFormSuccess == cgiFormStringSpaceNeeded("sigtext", &space)) {
			sig = (char *) malloc((space + 1) * sizeof(char));
			cgiFormString("sigtext", sig, space + 1);
			aucEmailClWriteSigFile(sig);
			free(sig);
		}
		if (cgiFormSuccess == cgiFormStringSpaceNeeded("forwardemail", &space)) {
			sig = (char *) malloc((space + 1) * sizeof(char));
			cgiFormString("forwardemail", sig, space + 1);
			aucEmailClWriteForwardFile(sig);
			free(sig);
		}

		aucEmailClGetOptionNames(options, 64);
		for (currkey = pinerc, currname = options; *currname; currkey++, currname++) {
			snprintf(buf2, sizeof(buf2), "%s-custom", *currname);
			currkey->setting = *currname;
			currkey->set = 1;
			currkey->custom = 0;
			if (cgiFormSuccess == cgiFormString(buf2, buf, sizeof(buf))) {
				currkey->set = 0;
				if (!strcmp(buf, "yes")) {
					currkey->custom = 1;
					snprintf(buf2, sizeof(buf2), "%s-val", *currname);
					if (cgiFormSuccess == cgiFormString(buf2, buf, sizeof(buf)))
						currkey->value = (char *) strdup(buf);
					else
						currkey->custom = 0;
				}
			}
		}
		currkey->setting = *currname;
		aucEmailClWritePinerc(pinerc);
		aucEmailClFreePinerc(pinerc);
	}

	if (!post || level || force) {
		cgiHeaderContentType("text/html");
		aucHtmlSlimHead("%s", _("Email Preferences"));
		aucEmailClShowOptions(output, level);
		aucHtmlSlimFoot();
	} else {
		cgiHeaderLocation("../home");
	}
	return 0;
}


int
EmailFolders(FILE * output, char *extrapath)
{
	char mailbox[256];
	int err;

	strcpy(mailbox, "INBOX");

	cgiStaticContentType("text/html");
	aucHtmlSlimHead(_("Folders"));
	if (!(err = aucEmailClInit(mailbox, aucMailDebug))) {
		aucEmailClHeader(output, 1, QN_FOLDERS);
		aucEmailClViewFolders(output);
		aucEmailClFooter(output, QN_FOLDERS);
		aucEmailClClose();
	} else
		process_init_error(output, err, "folders");

	aucHtmlSlimFoot();
	return 0;
}


int
EmailDelFolder(FILE * output, char *extrapath)
{
	char buffer[256], mailbox[256];
	int force = 0;

	if (cgiFormSuccess == cgiFormString("mailbox", buffer, sizeof(buffer)))
		strcpy(mailbox, buffer);
	else {
		cgiStaticContentType("text/html");
		aucHtmlSlimHead("%s", _("Error Deleting Folder"));
		fprintf(output, "<h2>%s</h2>\n", _("Error Deleting Folder"));
		fprintf(output, "<p>%s</p>\n", _("A folder name was not specified"));
		aucHtmlSlimFoot();
		return -1;
	}

	if (cgiFormSuccess == cgiFormString("force", buffer, sizeof(buffer)))
		force = 1;

	if (!force) {
		cgiStaticContentType("text/html");
		aucHtmlSlimHead("%s", _("Confirm Folder Deletion"));
		fprintf(output, "<h2>%s</h2>\n", _("Confirmation"));
		fprintf(output, "<p>%s <b>%s</b>?\n",
				_("Are you sure you want to delete the folder"), mailbox);
		fprintf(output, "<p><b><a href=\"delfolder?mailbox=");
		aucPrintLinkText(output, mailbox);
		fprintf(output, "&force=yes\">%s</a></b></p>\n", _("Yes, delete it"));
		fprintf(output, "<p><b><a href=\"folders\">%s</a></b></p>\n", _("No, go back"));
		aucHtmlSlimFoot();
	} else if (!aucEmailClInit("INBOX", aucMailDebug) && !aucEmailClRemoveFolder(output, mailbox)) {
		char buf[256];
		snprintf(buf, sizeof(buf), "folders");
		cgiHeaderLocation(buf);
		aucEmailClClose();
	} else {
		cgiStaticContentType("text/html");
		aucHtmlSlimHead("%s", _("Error Deleting Folder"));
		fprintf(output, "<h2>%s</h2>\n", _("Error Deleting Folder"));
		fprintf(output, "<p>%s</p>\n", _("The specified folder could not be deleted"));
		aucHtmlSlimFoot();
	}
	return 0;
}


int
EmailNewFolder(FILE * output, char *extrapath)
{
	char buffer[256], folder[256];

	if (cgiFormSuccess == cgiFormString("folder", buffer, sizeof(buffer)))
		strcpy(folder, buffer);
	else {
		cgiStaticContentType("text/html");
		aucHtmlSlimHead("%s", _("Create New Folder"));
		fprintf(output, "<p>%s:</p>\n", _("Please specify a name for the new folder"));
		fprintf(output, "<form action=\"newfolder\" method=post>\n");
		fprintf(output, "<input type=text name=folder size=15><br>\n");
		fprintf(output, "<input type=submit value=\"%s\"></form>\n", _("Create"));
		aucHtmlSlimFoot();
		return 0;
	}

	if (!aucEmailClInit("INBOX", aucMailDebug)) {
		if (!aucEmailClNewFolder(output, folder)) {
			char buf[256];
			snprintf(buf, sizeof(buf), "view");
			cgiHeaderLocation(buf);
		} else {
			cgiStaticContentType("text/html");
			aucHtmlSlimHead("%s", _("Error Creating Folder"));
			fprintf(output, "<h2>%s</h2>\n", _("Error Creating Folder"));
			fprintf(output, "<p>%s</p>\n", _("The specified folder could not be created"));
			aucHtmlSlimFoot();
		}
		aucEmailClClose();
	}
	return 0;
}

int
EmailAttach(FILE * output, char *extrapath)
{
	char buf[1024];

	int emailhead(FILE * output, char *path, char *name)
	{
		aucHtmlPopupHead(_("Attachments"));
		if (!strncmp(name, _("Attachments"), strlen(_("Attachments"))))
			fprintf(output, "<p>%s</p>\n\n",
					_
					("To attach files, click \"Upload a file\" and choose a file to attach.  Click \
             \"Close Window\" after you are done adding attachments.  Any files \
             listed here will be sent with your current email message."));
		else
			fprintf(output, "<div align=right><a href=\"%s/email/attach/\">%s</a></div>\n",
					aucPagesRoot, _("Back to attachments"));
		return 0;
	}
	int emailfoot(FILE * output, char *path, char *name)
	{
		aucHtmlPopupFoot();
		return 0;
	}

	snprintf(buf, sizeof(buf), "homedir/%s/attach", aucUserDir);
	aucViewPath(output, emailhead, emailfoot, _("Attachments"), buf, extrapath, TREE_USEUNIX, 1, 0);
	return 0;
}

int
EmailNewLogin(FILE * output, char *extrapath)
{
	struct pinerckey pinerc[4];
	char buf[256];

	pinerc[0].setting = "imap-name";
	pinerc[0].set = 0;
	pinerc[0].custom = 1;
	if (cgiFormSuccess == cgiFormString("imap-name", buf, sizeof(buf)))
		pinerc[0].value = strdup(buf);
	else
		pinerc[0].value = strdup("");

	pinerc[1].setting = "imap-passwd";
	pinerc[1].set = 0;
	pinerc[1].custom = 1;
	if (cgiFormSuccess == cgiFormString("imap-passwd", buf, sizeof(buf)))
		pinerc[1].value = strdup(buf);
	else
		pinerc[1].value = strdup("");

	pinerc[2].setting = NULL;

	aucEmailClWritePinerc(pinerc);
	aucEmailClFreePinerc(pinerc);

	if (cgiFormSuccess == cgiFormString("url", buf, sizeof(buf)))
		cgiHeaderLocation(buf);
	else
		cgiHeaderLocation("view");

	return 0;
}

int
process_init_error(FILE * output, int error, char *currpage)
{
	char buf[256];

	aucEmailClShutdown();
	switch (error) {
	case -1:
		fprintf(output, "<p><big><b>%s:</b> %s</big></p>\n\n", _("Error"),
				_("Could not open mailbox"));
		break;
	case -2:
		fprintf(output, "<p><big>%s</big></p>\n\n",
				_("Your current username and password failed to authenticate to the\n"
				  "IMAP server.  Please type a new username and password to use for\n"
				  "the mail server."));

		aucDrawTableForm(output, "POST", "new-imap-login");

		if (strlen(cgiQueryString))
			snprintf(buf, sizeof(buf), "%s?%s", currpage, cgiQueryString);
		else
			snprintf(buf, sizeof(buf), "%s", currpage);
		aucDrawHidden(output, "url", buf);

		aucDrawTextbox(output, "imap-name", 16, 32, NULL, _("Username"));
		aucDrawPassword(output, "imap-passwd", 16, 32, NULL, _("Password"));

		aucDrawSubmit(output, _("Login"));
		aucDrawTableFormEnd(output);

		break;
	}
	return 0;
}
