/*
 * Authenticated User Community
 * Copyright (c) 2000 David C. Moore
 *
 * You may distribute and/or use for any purpose modified or unmodified
 * copies of this software if you preserve the copyright notice above.
 *
 * email.h
 *
 * Headers necessary for the use of functions in email.c
 *
 */

/* Make sure this is only included once. */

#ifndef EMAIL_C
#define EMAIL_C 1

#include <auc/nodelett.h>

#define EMAIL_INIT				0
#define EMAIL_CLOSE				1
#define EMAIL_MOVE_TO			2
#define EMAIL_OPTIONS			3
#define EMAIL_FIRST_NEW 		4
#define EMAIL_SET_ANSWERED 	5
#define EMAIL_REMOVE_FOLDER	6
#define EMAIL_NEW_FOLDER		7
#define EMAIL_EMPTY_TRASH		8
#define EMAIL_INJECT 			9
#define EMAIL_COMPOSE			10
#define EMAIL_PRINT_MESSAGE	11
#define EMAIL_HEADER 			12
#define EMAIL_FOOTER 			13
#define EMAIL_VIEW_FOLDERS 	14
#define EMAIL_PRINT_LIST		15
#define EMAIL_MAILSTAT			16

#define QN_NEW       (0x1 <<1)
#define QN_SINGLE    (0x1 <<2)
#define QN_COMPOSE   (0x1 <<3)
#define QN_MAILBOX   (0x1 <<4)
#define QN_REPLYFULL (0x1 <<7)
#define QN_NOSELF    (0x1 <<8)
#define QN_REPLY     (0x1 <<9)
#define QN_FORWARD   (0x1 <<10)
#define QN_FOLDERS   (0x1 <<11)
#define QN_PRINTABLE (0x1 <<12)
#define QN_RAWTEXT   (0x1 <<13)
#define QN_POSTPONE  (0x1 <<14)
#define QN_NODELETE  (0x1 <<15)
#define QN_FORWARDSORT (0x1 <<16)
#define QN_OPTIONS   (0x1 <<17)

#define IMAPSPOOL 	1
#define LOCALSPOOL	2
#define ALLLOCAL		3

struct pinerckey
{
	char *setting;
	int custom;
	char *value;
	int set;
};

extern char *aucSMTPHost, *aucMailTrashFolder;
extern char *aucMailPineUserConfig, *aucMailPostponedFolder, *aucMailSavedFolder;
extern char *aucMailSystem;
extern int aucMailSourceType, aucMailAdvancedPrefs;

extern int aucEmailInit(void);

extern void aucComposeHref(char *email, FILE * output);

extern void aucEmailClPrintNodeFrame(FILE * output, aucNodelettInfoPtr nodelett, char *arg);

extern int EmailView(FILE * output, char *extrapath);
extern int EmailInject(FILE * output, char *extrapath);
extern int EmailMove(FILE * output, char *extrapath);
extern int EmailCompose(FILE * output, char *extrapath);
extern int EmailEmpTrash(FILE * output, char *extrapath);
extern int EmailOptions(FILE * output, char *extrapath);
extern int EmailFolders(FILE * output, char *extrapath);
extern int EmailDelFolder(FILE * output, char *extrapath);
extern int EmailNewFolder(FILE * output, char *extrapath);
extern int aucEmailClFirstNew(FILE * output, char *extrapath);
extern int EmailAttach(FILE * output, char *extrapath);
extern int EmailNewLogin(FILE * output, char *extrapath);

extern char *aucEmailClGetAddress(char *uname);
extern void aucLinkComposeMessage(FILE * output, char *email, char *fullname, char *text);
extern int aucEmailClInit(char *mailbox, int debug);
extern int aucEmailClCompose(FILE * outfile, char *to, long messid, int func_flags);
extern int aucEmailClQuickInject(FILE * output, char *from_head, char *from_in, char *to_in,
								 char *cc_in, char *subj_in, char *text_in);
extern int aucEmailClInject(FILE * outfile, char *to, char *cc, char *subj, char *text,
							char *inreplyto_in, unsigned long func_flags);
extern int aucEmailClHeader(FILE * outfile, int pageno, int func_flags);
extern int aucEmailClFooter(FILE * outfile, int func_flags);
extern int aucEmailClPrintList(FILE * outfile, int page, int func_flag);
extern int aucEmailClPrintMessage(FILE * outfile, unsigned long uid, unsigned long func_flags);
extern int aucEmailClMoveTo(char *dest, char *sequence);
extern int aucEmailClClose(void);
extern int aucEmailClShutdown(void);
extern int aucEmailClRemoveFolder(FILE * outfile, char *folder);
extern int aucEmailClNewFolder(FILE * outfile, char *folder);
extern int aucEmailClEmptyTrash(FILE * output);
extern int aucEmailClViewFolders(FILE * output);
extern int aucEmailClShowOptions(FILE * output, int level);
extern int aucEmailClSetAnswered(FILE * outfile, unsigned long uid);
extern void aucEmailClGetOptionNames(char **options, int max);
extern void aucEmailClFreePinerc(struct pinerckey *pinerc);
extern int aucEmailClWritePinerc(struct pinerckey *pinerc);
extern int aucEmailClWriteSigFile(char *sig);
extern int aucEmailClWriteForwardFile(char *email);
extern int aucEmailClParseOptions(void);
extern int aucEmailClStartAgent(char *uname, char *authstring);
extern int aucEmailClSendOptions(void);


#endif /* EMAIL_C */
