
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/time.h>
#include <signal.h>
#include <unistd.h>

#include <auc/auc.h>

#include "error.h"

static void stack_trace(char **);
static char *command_error = NULL;

int
aucErrorAppendStr(char *str)
{
	char buf[1024];
	if (command_error)
		snprintf(buf, sizeof(buf), "%s<br>%s", command_error, str);
	else
		snprintf(buf, sizeof(buf), "%s", str);

	free(command_error);
	command_error = (char *) strdup(buf);
	return 0;
}

int
aucErrorDisplayQueue(FILE * output)
{
	if (command_error) {
		fprintf(output, "<p>%s</p>", command_error);
		aucError(command_error);
		free(command_error);
		command_error = NULL;
	}
	return 0;
}

int
aucErrorSaveQueue()
{
	char buf[4096], *buf2;

	if (command_error) {
		buf2 = (char *) malloc(sizeof(char) * (2 * strlen(command_error) + 1));
		mysql_escape_string(buf2, command_error, strlen(command_error));
	} else
		buf2 = (char *) strdup("");

	snprintf(buf, sizeof(buf), "UPDATE prefs SET commanderror=\"%s\" \
      WHERE uname=\"%s\"", buf2, aucUserCache->uname);
	free(buf2);
	if (mysql_query(db, buf))
		aucError(mysql_error(db));
	return 0;
}

void
aucError(char *errstr, ...)
{
	va_list ap;
	char buf[128], str[2048];

	strftime(buf, sizeof(buf), "%a %b %d %T %Y", localtime(&aucTime));
	va_start(ap, errstr);
	vsnprintf(str, sizeof(str), errstr, ap);
	va_end(ap);

	if (isatty(2))
		fprintf(stderr, "%s\n", str);
	else if (aucAuthenticated)
		fprintf(stderr, "[%s] auc: (%s) %s\n", buf, aucUserCache->uname, str);
	else
		fprintf(stderr, "[%s] auc: %s\n", buf, str);
}

void
aucErrorStackTrace(const char *prg_name)
{
	pid_t pid;
	char buf[16];
	char *args[4] = { "gdb", NULL, NULL, NULL };

	if (!prg_name)
		return;

	sprintf(buf, "%u", (uint) getpid());

	args[1] = (char *) prg_name;
	args[2] = buf;

	pid = fork();
	if (pid == 0) {
		stack_trace(args);
		_exit(0);
	} else if (pid == (pid_t) -1) {
		perror("unable to fork gdb");
		return;
	}
	waitpid(pid, NULL, 0);
}

static int stack_trace_done = 0;

static void
stack_trace_sigchld(int signum)
{
	stack_trace_done = 1;
}

static void
stack_trace(char **args)
{
	pid_t pid;
	int in_fd[2];
	int out_fd[2];
	fd_set fdset;
	fd_set readset;
	struct timeval tv;
	int sel, index, state;
	char buffer[1024];
	char c;
//  FILE * outfile;

	stack_trace_done = 0;
	signal(SIGCHLD, stack_trace_sigchld);

	if ((pipe(in_fd) == -1) || (pipe(out_fd) == -1)) {
		perror("unable to open pipe");
		_exit(0);
	}

	pid = fork();
	if (pid == 0) {
		close(0);
		dup(in_fd[0]);			/* set the stdin to the in pipe */
		close(1);
		dup(out_fd[1]);			/* set the stdout to the out pipe */
		close(2);
		dup(out_fd[1]);			/* set the stderr to the out pipe */

		fprintf(stderr, "%d %d\n", geteuid(), getuid());
		
		if (0 == getuid())
			seteuid(0);
		else if (0 == geteuid())
			setuid(0);
			
		fprintf(stderr, "%d %d\n", geteuid(), getuid());

		execvp(args[0], args);	/* exec gdb */
		perror("exec failed");
		_exit(0);
	} else if (pid == (pid_t) - 1) {
		perror("unable to fork");
		_exit(0);
	}

	FD_ZERO(&fdset);
	FD_SET(out_fd[0], &fdset);

	write(in_fd[1], "backtrace\n", 10);
	write(in_fd[1], "p x = 0\n", 8);
	write(in_fd[1], "quit\n", 5);

	index = 0;
	state = 0;

//  snprintf(buffer, sizeof(buffer), "crash.%ld", time(NULL));
//  outfile = fopen(buffer, "w"); 
	while (1) {
		readset = fdset;
		tv.tv_sec = 1;
		tv.tv_usec = 0;

		sel = select(FD_SETSIZE, &readset, NULL, NULL, &tv);
		if (sel == -1)
			break;

		if ((sel > 0) && (FD_ISSET(out_fd[0], &readset))) {
			if (read(out_fd[0], &c, 1)) {
//				fprintf(stderr, "%c", c);
				switch (state) {
				case 0:
					if (c == '#') {
						state = 1;
						index = 0;
						buffer[index++] = c;
					}
					break;
				case 1:
					buffer[index++] = c;
					if (((c == '\n') || (c == '\r')) && (buffer[index - 2] != ',')
						&& (buffer[index - 3] != ',') && (buffer[index - 2] != ')')
						&& (buffer[index - 3] != ')') && (buffer[index - 2] != '(')
						&& (buffer[index - 3] != '(')) {
						buffer[index - 1] = 0;
//						aucError(buffer);
                        fprintf (stderr, "%s\n", buffer);
						state = 0;
						index = 0;
					}
					break;
				default:
					break;
				}
			}
		} else if (stack_trace_done)
			break;
	}

//  fclose(outfile);
	close(in_fd[0]);
	close(in_fd[1]);
	close(out_fd[0]);
	close(out_fd[1]);
	_exit(0);
}
