/*
 * Authenticated User Community
 * Copyright (c) 2000 David C. Moore
 *
 * You may distribute and/or use for any purpose modified or unmodified
 * copies of this software if you preserve the copyright notice above.
 *
 * files.h
 *
 * Headers necessary for the use of functions in files.c
 *
 */

/* Make sure this is only included once. */

#ifndef FILES_C
#define FILES_C 1

#include <auc/nodelett.h>
#include <libxml/tree.h>

#define FILE_ERROR      0
#define FILE_DEFAULT    1
#define FILE_DIR        2
#define FILE_WORDDOC    3
#define FILE_TEXTDOC    4
#define FILE_HTMLDOC    5

#define FMASK_ERROR	(1 << 0)
#define FMASK_DEFAULT	(1 << 1)
#define FMASK_DIR	(1 << 2)
#define FMASK_WORDDOC	(1 << 3)
#define FMASK_TEXTDOC	(1 << 4)
#define FMASK_HTMLDOC	(1 << 5)
#define FMASK_ANY	-1

#define	IS_WRITABLE	(1 << 0)
#define	IS_POST		(1 << 1)

#define FOP_WRITE       0
#define FOP_MKDIR       1
#define FOP_DELETE      2
#define FOP_RENAME      3
#define FOP_RMDIR       4
#define FOP_CLASSINIT   5
#define FOP_MOVE        6
#define FOP_NEWSINIT    7

#define TREE_FORCEWRITE	0
#define TREE_USEUNIX	1
#define TREE_READONLY	2

#define FOLDER_NORMAL	0
#define FOLDER_DROPBOX	1

typedef int (*shorthead_fn) (FILE *, char *, char *);
typedef int (*shortfoot_fn) (FILE *, char *, char *);

typedef struct _aucFolder aucFolder;
typedef aucFolder * aucFolderPtr;

/**
 * Structure used to hold properties of a folder
 */
 
struct _aucFolder
{
	int type;
	char * description;
	xmlDocPtr doc;
};

extern char *aucWordViewPath, *aucWordViewOptions;
extern unsigned long aucNonLocalDiskLimit;

extern aucFolderPtr aucFilesGetFolderProps(char * path);
extern void aucFilesFreeFolderProps(aucFolderPtr props);


extern int aucFilesInit(void);

extern void aucFilesNodeFrame(FILE * output, aucNodelettInfoPtr nodelett, char *arg);

extern int aucFileBrowserH(FILE * output, char *extrapath);

extern int aucRegisterFilePath(char *root, char *virtualmatch, int cutoff);
extern int aucChownUploadedFiles(uid_t uid, gid_t gid);
extern int aucExceededQuota();
extern int aucFileListView(FILE * output, char *virtualfs, char *dir, int expert, int hidden,
						   int cols);
extern int aucFileDetailsView(FILE * output, char *path, int expert, int hidden, int write);
extern int aucViewPath(FILE * output, shorthead_fn shorthead, shortfoot_fn shortfoot,
					   char *topdirname, char *virtualfs, char *dir, int requested_perm,
					   int expert, int hidden);
extern unsigned long aucGetFileType(char *path, char *name);
extern char *aucFormatPathText(char *str, size_t len, char *path, unsigned long ftype, int expert);
extern int aucLinkFileShort(FILE * output, char *link, char *name, unsigned long ftype, int expert);
extern int aucFileOp(char *path, char *path2, unsigned long fop);
extern int aucText2Filename(char *filename);
extern int aucNewFilePrompt(FILE * output, char *action, int expert);
extern int aucRenameFilePrompt(FILE * output, char *action, char *fname, unsigned long ftype,
							   int expert);
extern int aucFileRemoveDir(char * path, int contents_only);


#endif /* FILES_C */
