/*
 * Authenticated User Community
 * Copyright (c) 2000 David C. Moore
 *
 * You may distribute and/or use for any purpose modified or unmodified
 * copies of this software if you preserve the copyright notice above.
 *
 * template.c
 *
 * Routines for the consistent display of HTML elements in
 * AUC-generated web pages
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>

#include <auc/cgic.h>
#include <auc/auc.h>
#include <auc/conf.h>
#include <auc/error.h>
#include <auc/calendar.h>
#include <auc/pages.h>
#include <auc/email.h>
#include <auc/util.h>

#include "html.h"

void getcurrdate(int *, int *, int *, char *);

char *helptitles[] = {
	"login", N_("Login Help"),
	"start", N_("Start Page Help"),
	NULL
};

char *aucmonths[] = {
	N_("January"),
	N_("February"),
	N_("March"),
	N_("April"),
	N_("May"),
	N_("June"),
	N_("July"),
	N_("August"),
	N_("September"),
	N_("October"),
	N_("November"),
	N_("December"),
	NULL
};

void
getcurrdate(int *yr, int *mn, int *dy, char *str)
{
	time_t timep;
	struct tm *times;

	if (str && (3 == sscanf(str, "%4u-%2u-%2u", yr, mn, dy))) {
		return;
	}
	timep = time(NULL);
	times = localtime(&timep);
	*yr = 1900 + times->tm_year;
	*mn = 1 + times->tm_mon;
	*dy = times->tm_mday;
}

int colorval(char * color, int * rgb)
{
	char buf[4], * end;
	int i;
	
	if (!color || strlen(color) != 6)
		return -1;
	
	for (i=0; i<3; i++) {
		strncpy(buf, color+(2*i), 2);
		rgb[i] = strtol(buf, &end, 16);
		if (end != buf+2)
			return -1;
	}
	return 0;
}

int colordiff(char * c1, char * c2)
{
	int rgb1[3], rgb2[3];
	
	if (colorval(c1, rgb1) < 0)
		return -1;
	if (colorval(c2, rgb2) < 0)
		return -1;
	
	rgb1[0] -= rgb2[0];
	rgb1[1] -= rgb2[1];
	rgb1[2] -= rgb2[2];
	
	return rgb1[0]*rgb1[0] + rgb1[1]*rgb1[1] + rgb1[2]*rgb1[2];
}

void
aucDrawTableForm(FILE * output, char *method, char *action)
{
	fprintf(output, "<p><table cellpadding=2 cellspacing=0 border=0>\n");
	fprintf(output, "<form method=\"%s\" action=\"%s\">\n\n", method, action);
}

void
aucDrawHidden(FILE * output, char *name, char *value)
{
	fprintf(output, "<input type=\"hidden\" name=\"%s\" value=\"%s\">\n\n", name,
			value ? value : "");
}

void
aucDrawChooseName(FILE * output, char *name, char *uname,
				  char *title, char *fname, char *mname, char *caption)
{
	MYSQL_RES *res = NULL;
	MYSQL_ROW row = NULL;
	char buf[1024], **a, *title2, *fname2, *mname2;
	char *titles[] = { N_("Mrs."), N_("Mr."), N_("Dr."), N_("Ms."), N_("Miss"), "", NULL };

	(title) ? (title2 = title) : (title2 = "zzz");
	(fname) ? (fname2 = fname) : (fname2 = "zzz");
	(mname) ? (mname2 = mname) : (mname2 = "zzz");
	snprintf(buf, sizeof(buf), "SELECT fname, mname, lname, grp FROM \
      userinfo WHERE uname=\"%s\"", uname);
	if (mysql_query(db, buf))
		aucError(mysql_error(db));
	else {
		res = mysql_store_result(db);
		row = mysql_fetch_row(res);
	}

	fprintf(output, "<tr><td align=right>%s:</td>\n", caption);
	fprintf(output, "<td><p>\n");
	if (!strcmp(row[3], _("teacher"))) {
		fprintf(output, "<select name=\"%s-title\">\n", name);
		for (a = titles; *a; a++)
			fprintf(output, "<option%s>%s</option>\n",
					(!strcmp(_(*a), title2)) ? " selected" : "", _(*a));
		fprintf(output, "</select>\n");
	}
	fprintf(output, "<select name=\"%s-fname\">\n", name);
	snprintf(buf, sizeof(buf), "%s", row[0]);
	fprintf(output, "<option%s>%s</option>\n", (!strcmp(buf, fname2)) ? " selected" : "", buf);
	snprintf(buf, sizeof(buf), "%c.", row[0][0]);
	fprintf(output, "<option%s>%s</option>\n", (!strcmp(buf, fname2)) ? " selected" : "", buf);
	strncpy(buf, "", sizeof(buf));
	fprintf(output, "<option%s>%s</option>\n", (!strcmp(buf, fname2)) ? " selected" : "", buf);
	fprintf(output, "</select>\n");
	if (row[1] && strlen(row[1])) {
		fprintf(output, "<select name=\"%s-mname\">\n", name);
		snprintf(buf, sizeof(buf), "%s", row[1]);
		fprintf(output, "<option%s>%s</option>\n",
				(!strcmp(buf, mname2)) ? " selected" : "", buf);
		snprintf(buf, sizeof(buf), "%c.", row[1][0]);
		fprintf(output, "<option%s>%s</option>\n",
				(!strcmp(buf, mname2)) ? " selected" : "", buf);
		strncpy(buf, "", sizeof(buf));
		fprintf(output, "<option%s>%s</option>\n",
				(!strcmp(buf, mname2)) ? " selected" : "", buf);
		fprintf(output, "</select>\n");
	}
	fprintf(output, "%s", row[2]);
	fprintf(output, "</p></td></tr>\n");
	if (res)
		mysql_free_result(res);
}

void
aucDrawDateselect(FILE * output, char *name, char *value, char *caption)
{
	int i;
	char **a;
	int yr, mn, dy;

	getcurrdate(&yr, &mn, &dy, value);

	fprintf(output, "<tr><td align=right>%s:</td>\n", caption);
	fprintf(output, "<td><p>\n");
	fprintf(output, "<select name=\"%s-month\">\n", name);
	for (a = aucmonths, i = 1; *a; a++, i++)
		fprintf(output, "<option value=\"%02d\"%s>%s</option>\n",
				i, (mn == i) ? " selected" : "", _(*a));
	fprintf(output, "</select>\n");
	fprintf(output, "<select name=\"%s-day\">\n", name);
	for (i = 1; i <= 31; i++)
		fprintf(output, "<option%s>%02d</option>\n", (dy == i) ? " selected" : "", i);
	fprintf(output, "</select>\n");
	fprintf(output, "<select name=\"%s-year\">\n", name);
	for (i = yr - 1; i <= yr + 1; i++)
		fprintf(output, "<option%s>%04d</option>\n", (yr == i) ? " selected" : "", i);
	fprintf(output, "</select>\n");
	fprintf(output, "</p></td></tr>\n");
}

void
aucDrawDateSchool(FILE * output, char *name, char *value, int sem, char *caption)
{
	char currdate[16], nextdate[16];
	int foundcurr = 0;
	char buf[512];

	aucGetSemBeg(currdate, sizeof(currdate), sem);
	if (value && strlen(value))
		snprintf(nextdate, sizeof(nextdate), "%s", value);
	else
		aucGetTodayDate(nextdate, sizeof(nextdate));

	fprintf(output, "<tr><td align=right>%s:</td>\n", caption);
	fprintf(output, "<td><p>\n");
	fprintf(output, "<select name=\"%s\">\n", name);
	while (strlen(currdate)) {
		snprintf(buf, sizeof(buf), "%s", currdate);
		if (!foundcurr && (strcmp(nextdate, currdate) <= 0)) {
			foundcurr = 1;
			fprintf(output, "<option value=\"%s\" selected>%s</option>\n",
					buf, aucDateText(currdate));
		} else
			fprintf(output, "<option value=\"%s\">%s</option>\n", buf, aucDateText(currdate));

		aucIncDate(currdate, sizeof(currdate), sem);
	}
	fprintf(output, "</select>\n");
	fprintf(output, "</p></td></tr>\n");
}

void
aucDrawCheckbox(FILE * output, char *name, int state, char *description, char *caption)
{
	fprintf(output, "<tr><td align=right>%s:</td>\n", caption);
	fprintf(output, "<td><p><input type=\"checkbox\" name=\"%s\" %s> %s</p></td></tr>\n\n",
			name, state ? "checked" : "", description);
}

void
aucDrawCheckOpts(FILE * output, char *names[], char *descriptions[], int states[], char *caption)
{
	int x;

	fprintf(output, "<tr><td align=right>%s:</td>\n", caption);
	fprintf(output, "<td><p>\n");
	for (x = 0; names[x]; x++)
		fprintf(output, "<input type=\"checkbox\" name=\"%s\" %s> %s<br>\n",
				names[x], states[x] ? "checked" : "", descriptions[x]);
	fprintf(output, "</p></td></tr>\n\n");
}

void
aucDrawCheckMult(FILE * output, char *name, char *values[], char *descriptions[], int states[],
				 char *caption)
{
	int x;

	fprintf(output, "<tr><td align=right>&nbsp;</td>\n");
	fprintf(output, "<td><p>%s:<br>\n", caption);
	for (x = 0; values[x]; x++)
		fprintf(output, "<input type=\"checkbox\" name=\"%s\" value=\"%s\" %s> %s<br>\n",
				name, values[x], states[x] ? "checked" : "", descriptions[x]);
	fprintf(output, "</p></td></tr>\n\n");
}

void
aucDrawTextbox(FILE * output, char *name, int size, int max, char *value, char *caption)
{
	fprintf(output, "<tr><td align=right>%s:</td>\n", caption);
	fprintf(output,
			"<td><input type=\"text\" name=\"%s\" size=%d maxlength=%d value=\"%s\"></td></tr>\n\n",
			name, size, max, value ? value : "");
}

void
aucDrawPassword(FILE * output, char *name, int size, int max, char *value, char *caption)
{
	fprintf(output, "<tr><td align=right>%s:</td>\n", caption);
	fprintf(output,
			"<td><input type=\"password\" name=\"%s\" size=%d maxlength=%d value=\"%s\"></td></tr>\n\n",
			name, size, max, value ? value : "");
}

void
aucDrawTextarea(FILE * output, char *name, int rows, int cols, char *value, char *caption)
{
	fprintf(output, "<tr valign=top><td align=right width=180>%s:</td>\n", caption);
	fprintf(output, "<td><textarea name=\"%s\" rows=%d cols=%d wrap=soft>\n", name, rows, cols);
	fprintf(output, "%s", value ? value : "");
	fprintf(output, "</textarea></td></tr>\n\n");
}

void
aucDrawDropdown(FILE * output, char *name, char *values[], char *fields[], char *value,
				char *caption)
{
	int x;

	fprintf(output, "<tr><td align=right>%s:</td>\n", caption);
	fprintf(output, "<td><select name=\"%s\">\n", name);
	for (x = 0; values[x]; x++)
		if (value && (!strcmp(values[x], value) || !strcmp(fields[x], value)))
			fprintf(output, "<option value=\"%s\" selected>%s</option>\n", values[x], _(fields[x]));
		else
			fprintf(output, "<option value=\"%s\">%s</option>\n", values[x], _(fields[x]));
	fprintf(output, "</select></td></tr>\n\n");
}

void
aucDrawRadio(FILE * output, char *name, char *values[], char *fields[], char *value, char *caption)
{
	int x;

	fprintf(output, "<tr><td align=right>%s:</td><td>\n", caption);
	for (x = 0; values[x]; x++)
		if (value && (!strcmp(values[x], value) || !strcmp(fields[x], value)))
			fprintf(output, "<input type=radio name=\"%s\" value=\"%s\" checked>%s<br>\n",
					name, values[x], fields[x]);
		else
			fprintf(output, "<input type=radio name=\"%s\" value=\"%s\">%s<br>\n",
					name, values[x], fields[x]);
	fprintf(output, "</td></tr>\n\n");
}

void
aucDrawSubmit(FILE * output, char *value)
{
	fprintf(output, "<tr><td>&nbsp;</td><td><input type=\"submit\" value=\"%s\"></td></tr>\n\n",
			value);
}

void
aucDrawSubmitName(FILE * output, char *name, char *value)
{
	fprintf(output,
			"<tr><td>&nbsp;</td><td><input type=\"submit\" name=\"%s\" value=\"%s\"></td></tr>\n\n",
			name, value);
}

void
aucDrawTableFormEnd(FILE * output)
{
	fprintf(output, "</form></table></p>\n\n");
}

int
aucHtmlHelpPage(FILE * output, char *extrapath)
{
	char filename[128], buf[1024], **a, root[128];

	if (extrapath && strlen(extrapath))
		strncpy(filename, extrapath, sizeof(filename));
	else
		strcpy(filename, _("Error"));

	cgiStaticContentType("text/html");
	for (a = helptitles; *a; a += 2)
		if (!strcmp(*a, filename)) {
			aucHtmlPopupHead(_(*(a + 1)));
			break;
		}
	if (!*a)
		aucHtmlPopupHead(filename);


	snprintf(root, sizeof(root), "%s/help", aucDataDir);
	aucGetFullPath(filename, root, buf, sizeof(buf));
	if (aucHtmlEchoFile(output, buf, 0)) {
		fprintf(output, "<p><b>%s</b>:\n", _("Error"));
		fprintf(output, "%s", _("The requested help file could not be found."));
		fprintf(output, "</p>\n\n");
	}

	aucHtmlPopupFoot();
	return 0;
}

void
aucHtmlPopupHead(char *title)
{
	char buf[256];

	fprintf(cgiOut, "<html><head><title>%s</title>\n", title);

	aucGetFullPath("headtags.html", aucDataDir, buf, sizeof(buf));
	aucHtmlEchoFile(cgiOut, buf, 0);

	fprintf(cgiOut, "</head>\n\n");

	fprintf(cgiOut,
			"<body bgcolor=#FFFFFF text=#000000 link=#%s vlink=#%s alink=#FF0000 marginheight=0 marginwidth=0>\n",
			aucLinkColor, aucVLinkColor);
	fprintf(cgiOut, "<a name=\"auctop\"></a>\n");

	fprintf(cgiOut, "<table width=100%% border=0 cellpadding=4 cellspacing=0>\n");
	fprintf(cgiOut, "<tr valign=top bgcolor=#%s><td width=100%%>\n\n", aucHeaderBGColor);

	fprintf(cgiOut, "<table width=100%% border=0 cellpadding=0 cellspacing=0>\n");

	/* Choose black or white text to contrast with either a dark or light background */
	if (colordiff(aucHeaderBGColor, "000000") > colordiff(aucHeaderBGColor, "FFFFFF"))
		fprintf(cgiOut, "<tr><td><b><font %s><big>%s</big></font></b></td>\n", HT_SANSSERIF, title);
	else
		fprintf(cgiOut, "<tr><td><b><font %s color=#FFFFFF><big>%s</big></font></b></td>\n", HT_SANSSERIF, title);

	fprintf(cgiOut, "<td align=right><font %s>&nbsp;\n", HT_SANSSERIF);

	fprintf(cgiOut,
			"<a href=\"javascript: self.close()\"><img src=\"%s/fileops/cancel.gif\" border=0 align=absmiddle> %s</a>\n",
			aucGraphDir, _("Close Window"));

	fprintf(cgiOut, "</font></td></tr>\n");

	fprintf(cgiOut, "</table>\n");

	fprintf(cgiOut, "</td></tr></table>\n");
	fprintf(cgiOut, "<table width=100%% border=0 cellpadding=0 cellspacing=0>\n");
	fprintf(cgiOut, "<tr valign=top bgcolor=#000000><td>");
	fprintf(cgiOut, "<img src=\"%s/transpix.gif\" width=1 height=2 alt=\"\">", aucGraphDir);
	fprintf(cgiOut, "</td></tr></table>\n");

	fprintf(cgiOut, "<table width=100%% border=0 cellpadding=15 cellspacing=0>\n");
	fprintf(cgiOut, "<tr valign=top><td>\n\n");

	aucErrorDisplayQueue(cgiOut);
}

void
aucHtmlPopupFoot()
{
	fprintf(cgiOut, "</td></tr></table>\n");
	fprintf(cgiOut, "<table width=100%% border=0 cellpadding=0 cellspacing=0>\n");
	fprintf(cgiOut, "<tr valign=top bgcolor=#000000><td>");
	fprintf(cgiOut, "<img src=\"%s/transpix.gif\" width=1 height=2 alt=\"\">", aucGraphDir);
	fprintf(cgiOut, "</td></tr></table>\n");

	fprintf(cgiOut, "<table width=100%% border=0 cellpadding=4 cellspacing=0>\n");
	fprintf(cgiOut, "<tr valign=top bgcolor=#%s><td>\n\n", aucHeaderBGColor);

	fprintf(cgiOut, "<table width=100%% border=0 cellpadding=0 cellspacing=0>\n");
	fprintf(cgiOut,
			"<tr><td><b><font %s>&nbsp;<a href=\"#auctop\"><img src=\"%s/fileops/top.gif\" border=0 align=absmiddle>%s</a></font></b></td>\n",
			HT_SANSSERIF, aucGraphDir, _("Top of Window"));
	fprintf(cgiOut, "</tr></table>\n");

	fprintf(cgiOut, "</td></tr></table>\n");

	fprintf(cgiOut, "</body></html>\n");
}

void
aucHtmlSlimHead(char *tformat, ...)
{
	va_list ap;
	char buf[256];
	char title[256];

	va_start(ap, tformat);
	vsnprintf(title, sizeof(title), tformat, ap);
	va_end(ap);

	fprintf(cgiOut, "<html><head><title>%s</title>\n", title);

	aucGetFullPath("headtags.html", aucDataDir, buf, sizeof(buf));
	aucHtmlEchoFile(cgiOut, buf, 0);

	fprintf(cgiOut, "</head>\n\n");

	fprintf(cgiOut,
			"<body bgcolor=#FFFFFF text=#000000 link=#%s vlink=#%s alink=#FF0000 marginheight=0 marginwidth=0>\n",
			aucLinkColor, aucVLinkColor);
	fprintf(cgiOut, "<a name=\"auctop\"></a>\n");

	fprintf(cgiOut, "<table width=100%% border=0 cellpadding=4 cellspacing=0>\n");
	fprintf(cgiOut, "<tr valign=top bgcolor=#%s><td width=100%%>\n\n", aucHeaderBGColor);

	fprintf(cgiOut, "<table width=100%% border=0 cellpadding=0 cellspacing=0>\n");
	fprintf(cgiOut, "<tr valign=bottom><td><font %s color=#%s>\n",
			HT_SANSSERIF, aucHeaderTextColor);
	if (aucAuthenticated && strcmp(cgiPathInfo, "/logoff")) {
		aucAuthGetFullName(buf, sizeof(buf), aucUserCache->uname, NAME_LINK);
		fprintf(cgiOut, "%s %s (%s)\n", _("Logged on as"), aucUserCache->uname, buf);
	}
	fprintf(cgiOut, "</font></td>\n");
	fprintf(cgiOut, "<td align=right><b><font color=#%s %s><big>%s</big></font></b>\n",
			aucHeaderTextColor, HT_SANSSERIF, aucOrgName);
	fprintf(cgiOut, "</td><td rowspan=2>\n");
	fprintf(cgiOut, "&nbsp;&nbsp;\n");
	fprintf(cgiOut, "</td><td rowspan=2 width=60>\n");

	fprintf(cgiOut, "<img src=\"%s/site-logo-small.gif\" alt=\"%s\">\n",
			aucGraphDir, aucSiteName);

	fprintf(cgiOut, "</td></tr>\n");

	/* Choose black or white text to contrast with either a dark or light background */
	if (colordiff(aucHeaderBGColor, "000000") > colordiff(aucHeaderBGColor, "FFFFFF"))
		fprintf(cgiOut, "<tr><td><b><font %s><big>%s</big></font></b></td>\n", HT_SANSSERIF, title);
	else
		fprintf(cgiOut, "<tr><td><b><font %s color=#FFFFFF><big>%s</big></font></b></td>\n", HT_SANSSERIF, title);

	fprintf(cgiOut, "<td align=right><font %s>&nbsp;\n", HT_SANSSERIF);
	if (aucAuthenticated && strcmp(cgiPathInfo, "/logoff")) {
		fprintf(cgiOut,
				"<a href=\"%s/home\"><img src=\"%s/fileops/home.gif\" border=0 align=absmiddle> %s </a> - \n",
				aucPagesRoot, aucGraphDir, _("Start Page"));
		fprintf(cgiOut,
				"<a href=\"%s/prefs\"><img src=\"%s/fileops/profile.gif\" border=0 align=absmiddle> %s</a> - \n",
				aucPagesRoot, aucGraphDir, _("User Profile"));
		fprintf(cgiOut,
				"<a href=\"%s/logoff\"><img src=\"%s/fileops/logoff.gif\" border=0 align=absmiddle> %s</a>\n",
				aucPagesRoot, aucGraphDir, _("Logoff"));
	}

	fprintf(cgiOut, "</font></td></tr>\n");

	fprintf(cgiOut, "</table>\n");

	fprintf(cgiOut, "</td></tr></table>\n");
	fprintf(cgiOut, "<table width=100%% border=0 cellpadding=0 cellspacing=0>\n");
	fprintf(cgiOut, "<tr valign=top bgcolor=#000000><td>");
	fprintf(cgiOut, "<img src=\"%s/transpix.gif\" width=1 height=2 alt=\"\">", aucGraphDir);
	fprintf(cgiOut, "</td></tr></table>\n");

	fprintf(cgiOut, "<table width=100%% border=0 cellpadding=15 cellspacing=0>\n");
	fprintf(cgiOut, "<tr valign=top><td>\n\n");

	aucErrorDisplayQueue(cgiOut);
}

void
aucHtmlSlimFoot()
{
	fprintf(cgiOut, "</td></tr></table>\n");
	fprintf(cgiOut, "<table width=100%% border=0 cellpadding=0 cellspacing=0>\n");
	fprintf(cgiOut, "<tr valign=top bgcolor=#000000><td>");
	fprintf(cgiOut, "<img src=\"%s/transpix.gif\" width=1 height=2 alt=\"\">", aucGraphDir);
	fprintf(cgiOut, "</td></tr></table>\n");

	fprintf(cgiOut, "<table width=100%% border=0 cellpadding=4 cellspacing=0>\n");
	fprintf(cgiOut, "<tr valign=top bgcolor=#%s><td>\n\n", aucHeaderBGColor);

	fprintf(cgiOut, "<table width=100%% border=0 cellpadding=0 cellspacing=0>\n");
	fprintf(cgiOut,
			"<tr><td><b><font %s>&nbsp;<a href=\"#auctop\"><img src=\"%s/fileops/top.gif\" border=0 align=absmiddle alt=\"Top\">%s</a></font></b></td>\n",
			HT_SANSSERIF, aucGraphDir, _("Top of Page"));

	/* Choose black or white text to contrast with either a dark or light background */
	if (colordiff(aucHeaderBGColor, "000000") > colordiff(aucHeaderBGColor, "FFFFFF")) {
		fprintf(cgiOut, "<td><center>%s &lt;", _("Questions or Comments? Email"));
		aucComposeHref(aucWebmaster, cgiOut);
		fprintf(cgiOut, "%s</a>&gt;</center></td>\n", aucWebmaster);
	}
	else {
		fprintf(cgiOut, "<td><center><font color=#FFFFFF>%s &lt;", _("Questions or Comments? Email"));
		aucComposeHref(aucWebmaster, cgiOut);
		fprintf(cgiOut, "%s</a>&gt;</font></center></td>\n", aucWebmaster);
	}
	fprintf(cgiOut, "<td align=right><b><font %s><a href=\"%s/about\">%s %s</a></font></b>\n",
			HT_SANSSERIF, aucPagesRoot, _("About AUC"), VERSION);
	fprintf(cgiOut, "</td></tr></table>\n");

	fprintf(cgiOut, "</td></tr></table>\n");

	fprintf(cgiOut, "</body></html>\n");
}

void
aucHtmlBlankHead(char *title)
{
	fprintf(cgiOut, "<html><head><title>%s</title></head>\n", title);
	fprintf(cgiOut, "<body bgcolor=#FFFFFF text=#000000 link=#%s vlink=#%s alink=#FF0000>\n",
			aucLinkColor, aucVLinkColor);
}

void
aucHtmlBlankFoot()
{
	fprintf(cgiOut, "</body></html>\n");
}

void
aucHtmlReadForm(char *fields, char **values, unsigned long flags)
{
	char *tok, **value;
	char buffer[1024], *names, d1[64], d2[64], d3[64];
	char *text;
	int size;

	names = (char *) malloc((strlen(fields) + 2) * sizeof(char));
	sprintf(names, "%s ", fields);
	value = values;
	for (tok = strtok(names, " \n"); tok; tok = strtok(NULL, " \n"), value++) {
		snprintf(d1, sizeof(d1), "%s-month", tok);
		snprintf(d2, sizeof(d2), "%s-day", tok);
		snprintf(d3, sizeof(d3), "%s-year", tok);
		if (cgiFormSuccess != cgiFormStringSpaceNeeded(tok, &size)) {
			if ((cgiFormSuccess == cgiFormString(d1, d1, sizeof(d1))) &&
				(cgiFormSuccess == cgiFormString(d2, d2, sizeof(d2))) &&
				(cgiFormSuccess == cgiFormString(d3, d3, sizeof(d3)))) {
				snprintf(buffer, sizeof(buffer), "%s-%s-%s", d3, d1, d2);
			} else
				strcpy(buffer, "");
			size = strlen(buffer) + 1;
			text = (char *) malloc(sizeof(char) * size);
			strcpy(text, buffer);
		} else {
			text = (char *) malloc(sizeof(char) * size);
			cgiFormString(tok, text, size);
		}
		*value = (char *) malloc(sizeof(char) * (size * 2 + 1));
		if (flags & STR_ESCAPE)
			mysql_escape_string(*value, text, size - 1);
		else
			strcpy(*value, text);
		free(text);
	}
	free(names);
	*value = NULL;

}

int
aucHtmlSetSortCookies(FILE * output, char *name)
{
	char usersort[256];

	if (cgiFormSuccess == cgiFormString("newsort", usersort, sizeof(usersort))) {
		cgiSendCookie(name, usersort, aucPagesRoot, 0);
		return 1;
	}
	return 0;
}

int
aucHtmlProcessSort(FILE * output, char *sort, size_t len, char *name, int recs,
			   aucSortPatternPtr sorts, char *defsort)
{
	char usersort[256], userpage[256];
	aucSortPatternPtr mysort, asort;
	char *namelist[64], **a, b, *temp;
	int x, flag, pageno;

	if (cgiFormSuccess != cgiFormString("newsort", usersort, sizeof(usersort)))
		if (cgiFormSuccess != cgiFormString(name, usersort, sizeof(usersort)))
			snprintf(usersort, sizeof(usersort), "%s", defsort);
	if (cgiFormSuccess != cgiFormString("page", userpage, sizeof(userpage)))
		snprintf(userpage, sizeof(userpage), "%d", 1);

	mysort = sorts;
	for (asort = sorts, x = 0; asort->name; asort++, x++) {
		if (!strcmp(usersort, asort->name))
			mysort = asort;
		namelist[x] = asort->name;
	}
	namelist[x] = asort->name;

	aucDrawTableForm(output, "GET", name);
	aucDrawDropdown(output, "newsort", namelist, namelist, usersort, _("Sort by"));
	aucDrawSubmit(output, _("Resort"));
	aucDrawTableFormEnd(output);

	fprintf(output, "<p>%s: \n", _("Page"));
	switch (mysort->type) {
	case SORT_CUSTOM:
		for (a = mysort->pages, flag = 0; *a; a++)
			if (!strcmp(userpage, *a))
				flag = 1;
		if (!flag)
			snprintf(userpage, sizeof(userpage), "%s", *(mysort->pages));
		strtol(userpage, &temp, 10);
		if (!strcmp(name, "account-manager") && !strcmp(usersort, "grp") && (temp != userpage))
			snprintf(sort, len, "WHERE grp=\"student\" AND class=\"%s\" %s", userpage,
					 mysort->code);
		else if (!strcmp(name, "account-manager") && !strcmp(usersort, "grp")
				 && !strcmp(userpage, "other"))
			snprintf(sort, len, "WHERE grp=\"student\" AND ISNULL(class) %s", mysort->code);
		else
			snprintf(sort, len, "WHERE %s=\"%s\" %s", mysort->name, userpage, mysort->code);
		for (a = mysort->pages; *a; a++)
			if (!strcmp(userpage, *a))
				fprintf(output, "<b>%s</b>&nbsp;\n", *(mysort->pagenames + (a - mysort->pages)));
			else
				fprintf(output, "<a href=\"%s?page=%s\">%s</a>&nbsp;\n", name,
						aucLinkText(*a), *(mysort->pagenames + (a - mysort->pages)));
		break;
	case SORT_ALPHA:
		if ((*userpage < 'A') || (*userpage > 'Z'))
			snprintf(userpage, sizeof(userpage), "A");
		snprintf(sort, len, "WHERE UCASE(%s) LIKE \"%c%%\" %s", mysort->name, *userpage,
				 mysort->code);
		for (b = 'A'; b <= 'Z'; b++)
			if (b == *userpage)
				fprintf(output, "<b>%c</b>\n", b);
			else
				fprintf(output, "<a href=\"%s?page=%c\">%c</a>\n", name, b, b);
		break;
	case SORT_PAGES:
		pageno = 1;
		sscanf(userpage, "%d", &pageno);
		snprintf(sort, len, "%s LIMIT %d,%d", mysort->code,
				 (pageno - 1) * mysort->maxlines, mysort->maxlines);
		for (x = 1; (x * mysort->maxlines - mysort->maxlines) < recs; x++)
			if (pageno == x)
				fprintf(output, "<b>%d</b>\n", x);
			else
				fprintf(output, "<a href=\"%s?page=%d\">%d</a>\n", name, x, x);
		break;
	}
	fprintf(output, "</p>\n\n");
	return 1;
}

int
aucHtmlEchoComm(FILE * output, char * cmd, char * const args[])
{
	int fork_val, status;
	
	/* todo: This command always outputs the child process to stdout rather than output */
	
	fflush(output);
	fork_val = fork();
	if (fork_val < 0)
		return -1;
	
	if (fork_val) {
		waitpid(fork_val, &status, 0);
	} else {
		/* get rid of all root permissions */
		if (!getuid())
			setuid(geteuid());
		else if (!geteuid())
			seteuid(getuid());
		execv(cmd, args);
		/* execv only returns upon failure */
		_exit(1);
	}

	if (!WIFEXITED(status) || WEXITSTATUS(status))
		return -1;

	return 0;
}

int
aucHtmlEchoCommBuf(char *name, char *buf, size_t len)
{
	FILE *poptemp;
	char c;
	int x = 0;

	if ((poptemp = popen(name, "r"))) {
		while (((c = getc(poptemp)) != EOF) && (x < (len - 1)))
			buf[x++] = c;
		buf[x] = '\0';
		pclose(poptemp);
		return (1);
	} else
		return (0);
}

int
aucHtmlEchoFile(FILE * output, char *name, int nosubs)
{
	FILE *foptemp;
	char data[1024], buf[64], *buf2, buf3[128];
	char datum;
	int count, invalid;

	if ((foptemp = fopen(name, "r"))) {
		while (fread(&datum, sizeof(char), 1, foptemp)) {
			if (nosubs || (datum != '@'))
				fwrite(&datum, sizeof(char), 1, output);
			else
				while (datum == '@') {
					data[0] = '@';
					count = 1;
					invalid = 1;
					while ((count < 32) && fread(&datum, sizeof(char), 1, foptemp)) {
						data[count++] = datum;
						if ( !((datum >= '@') && (datum <= 'Z')) && 
							 !((datum >= 'a') && (datum <= 'z')) &&
							 !((datum >= '0') && (datum <= '9')) )
							break;
						if (datum == '@') {
							invalid = 0;
							strncpy(buf, data + 1, count - 2);
							buf[count - 2] = '\0';
							snprintf(buf3, sizeof(buf3), "descendant::%s", buf);
							if (!aucConfToken(buf3, &buf2)) {
								datum = ' ';
								fwrite(buf2, sizeof(char), strlen(buf2), output);
							} else
								fwrite(data, sizeof(char), count - 1, output);
							break;
						}
					}
					if (invalid)
						fwrite(data, sizeof(char), count, output);
				}
		}
		fclose(foptemp);
		return 0;
	} else
		return -1;
}

/*
int
aucHtmlEchoComm(FILE * output, char *name)
{
	FILE *poptemp;
	char c;
	int retval = 0;

	if ((poptemp = popen(name, "r"))) {
		while ((c = getc(poptemp)) != EOF) {
			retval++;
			fputc(c, output);
		}
		pclose(poptemp);
		return retval;
	} else
		return retval;
}
*/
