/*
 * Authenticated User Community
 * Copyright (c) 2000 David C. Moore
 *
 * You may distribute and/or use for any purpose modified or unmodified
 * copies of this software if you preserve the copyright notice above.
 *
 * install.c
 *
 * Functions used for the installation wizard
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <libxml/tree.h>
#include <libxml/parser.h>
#include <sys/stat.h>
#include <unistd.h>

#include <auc/cgic.h>
#include <auc/auc.h>
#include <auc/util.h>
#include <auc/database.h>
#include <auc/forms.h>
#include <auc/html.h>
#include <auc/conf.h>
#include <auc/plugin.h>
#include <auc/nodelett.h>
#include <auc/error.h>

int check_sample_conf(char *);
int copy_sample_conf(char *, char *, int);

int
aucPageFirstRun(FILE * output, char *extrapath)
{
	char buf[1024];
	aucVarField vars[16];
	xmlDocPtr xmldoc = NULL;

	if (!aucFirstRun) {
		cgiStaticContentType("text/html");
		aucHtmlSlimHead("%s", _("Setup Complete"));

		fprintf(output, "<p><big>%s</big></p>\n\n", _("AUC has already been configured for \
         this site.  An administrator can edit these preferences from his/her \
         Start Page."));

		aucHtmlSlimFoot();
		return 0;
	}

	if (check_sample_conf("aucconf.xml.sample")) {
		cgiStaticContentType("text/html");
		aucHtmlSlimHead("%s", _("Sample Config Not Found"));

		fprintf(output, "<p><big><b>%s:</b> %s/aucconf.xml.sample %s.  %s</big></p>\n\n",
				_("Error"), aucEtcRoot, _("not found"),
				_("This file is necessary for the setup of AUC.  Make sure that AUC was \
			properly installed prior to setup."));

		aucHtmlSlimFoot();
		return 0;
	}

	vars[0].name = "form_action";
	vars[0].value = "firstrun-POST";
	vars[1].name = NULL;

	if (!strcmp(cgiRequestMethod, "GET")) {
		cgiStaticContentType("text/html");
		aucHtmlSlimHead("%s", _("AUC First Run"));

		aucGetFullPath("firstrun.xml", aucDataDir, buf, sizeof(buf));
		aucFormsXMLDisplayFile(output, buf, vars, NULL);
		aucHtmlSlimFoot();
	} else {
		char *tmp;

		if ((cgiFormSuccess == cgiFormCheckboxSingle("createdbinfo"))
			&& (tmp = aucDBProcessFormCreate())) {
			cgiHeaderContentType("text/html");
			aucHtmlSlimHead("%s", _("Error"));
			fprintf(output, "<p><big><b>%s</b>: %s</big></p>\n\n", _("Error"), tmp);
			aucHtmlSlimFoot();
			return -1;
		}

		if (copy_sample_conf("aucconf.xml.sample", "aucconf.xml", 0)) {
			cgiHeaderContentType("text/html");
			aucHtmlSlimHead("%s", _("Error"));
			fprintf(output, "<p><big><b>%s</b>: %s %s/aucconf.xml</big></p>\n\n", _("Error"),
					_("Could not edit"), aucEtcRoot);
			aucHtmlSlimFoot();
			return -1;
		}
		aucConfParse();
		xmldoc = aucConfGetDoc();
		aucGetFullPath("firstrun.xml", aucDataDir, buf, sizeof(buf));
		aucFormsXMLProcessFile(output, buf, vars, &xmldoc);
		if (!check_sample_conf("auc.conf")) {
			copy_sample_conf("auc.conf", "auc.conf.old", 1);
			aucConfUpdateToken("/child::auc", "version", "0.6.2");
		} else
			aucConfUpdateToken("/child::auc", "version", DB_VERSION);
		aucConfSetChanged();

		cgiHeaderContentType("text/html");
		aucHtmlSlimHead("%s", _("Setup Complete"));

		fprintf(output, "<h2>%s</h2>\n", _("Setup is now complete!"));
		fprintf(output, "<p>%s</p>\n\n",
				_("If this is the first time AUC has been set up on this server, you may \
			proceed to the <a href=\"home\">login page</a>.  Log in with the same username \
			and password that you specified for the SQL server.  Once logged in, select \
			the \"Site Preferences\" link to futher customize AUC for your site."));
		aucHtmlSlimFoot();
	}

	return 0;
}

int
aucPageSitePrefs(FILE * output, char *extrapath)
{
	char buf[256];
	int retval;
	char *fields[2], **sect;
	aucVarField vars[64];
	xmlDocPtr xmldoc = NULL;
	char *siteconf = "siteconfig.xml";
	char *subsections[] = {
		"global", _("Global"),
		"database", _("Database"),
		"auth", _("Authentication"),
		"scheduling", _("Scheduling"),
		"paths", _("File Paths"),
		"colors", _("Colors"),
		"fileman", _("File Manager"),
		"email", _("E-Mail"),
		"roles", _("User Roles"),
		NULL
	};

	vars[0].name = "form_action";
	vars[0].value = "site-prefs-POST";
	vars[1].name = "subsection";
	vars[1].value = "global";
	vars[2].name = NULL;

	aucHtmlReadForm("subsection", fields, 0);
	if (strlen(fields[0]))
		vars[1].value = fields[0];

	xmldoc = aucConfGetDoc();

	if ((sect = aucPluginConfList())) {
		for (; *sect; sect += 3) {
			if (!strcmp(*sect, vars[1].value))
				siteconf = *(sect + 2);
		}
	}

	if (siteconf)
		aucGetFullPath(siteconf, aucDataDir, buf, sizeof(buf));
	else
		strcpy(buf, "");

	retval = aucFormsProcessListEdit(output, buf, xmldoc, "site-prefs");
	if (retval == 1)
		return 0;
	else if (retval == 2) {
		aucConfSetChanged();
		return 0;
	}

	if (!strcmp(cgiRequestMethod, "GET")) {
		cgiStaticContentType("text/html");
		aucHtmlSlimHead("%s", _("Site Preferences"));

		fprintf(output, "<table cellpadding=0 cellspacing=4 border=0>\n");
		fprintf(output, "<tr valign=top>\n");
		fprintf(output, "<td><p align=right><font size=+1><i>%s</i>:</font></p></td>\n",
				_("Sections"));
		fprintf(output, "<td><font size=+1>\n");
		for (sect = subsections; *sect; sect += 2) {
			if (!strcmp(vars[1].value, *sect))
				fprintf(output, "<b>");
			else
				fprintf(output, "<a href=\"site-prefs?subsection=%s\">", *sect);
			fprintf(output, "[%s]", *(sect + 1));
			if (!strcmp(vars[1].value, *sect))
				fprintf(output, "</b>\n");
			else
				fprintf(output, "</a>\n");
		}
		fprintf(output, "</font></td></tr>\n\n");
		if ((sect = aucPluginConfList())) {
			fprintf(output, "<tr valign=top>\n");
			fprintf(output, "<td><p align=right><font size=+1><i>%s</i>:</font></p></td>\n",
					_("Plugins"));
			fprintf(output, "<td><font size=+1>\n");
			for (; *sect; sect += 3) {
				if (!strcmp(vars[1].value, *sect))
					fprintf(output, "<b>");
				else
					fprintf(output, "<a href=\"site-prefs?subsection=%s\">", *sect);
				fprintf(output, "[%s]", *(sect + 1));
				if (!strcmp(vars[1].value, *sect))
					fprintf(output, "</b>\n");
				else
					fprintf(output, "</a>\n");
			}
			fprintf(output, "</font></td></tr>\n\n");
		}
		fprintf(output, "</table>\n\n");

		aucFormsXMLDisplayFile(output, buf, vars, xmldoc);
		aucHtmlSlimFoot();
	} else {
		char *tmp;

		if ((cgiFormSuccess == cgiFormCheckboxSingle("createdbinfo"))
			&& (tmp = aucDBProcessFormCreate())) {
			cgiHeaderContentType("text/html");
			aucHtmlSlimHead("%s", _("Error"));
			fprintf(output, "<p><big><b>%s</b>: %s</big></p>\n\n", _("Error"), tmp);
			aucHtmlSlimFoot();
			return -1;
		}

		aucFormsXMLProcessFile(output, buf, vars, &xmldoc);
		aucConfSetChanged();
		snprintf(buf, sizeof(buf), "site-prefs?subsection=%s", vars[1].value);
		cgiHeaderLocation(buf);
	}

	return 0;
}

void
aucSiteSetupNodeFrame(FILE * output, aucNodelettInfoPtr nodelett, char *arg)
{
	MYSQL_RES *res;
	MYSQL_ROW row;
	int pendingaccts = 0;
	char buf[512];
	int count;

	snprintf(buf, sizeof(buf), "SELECT COUNT(*) FROM registrations \
      WHERE uname=\"\"");
	if (mysql_query(db, buf))
		aucError(mysql_error(db));
	else {
		res = mysql_store_result(db);
		if ((row = mysql_fetch_row(res)))
			pendingaccts = strtol(row[0], NULL, 10);
		mysql_free_result(res);
	}

	fprintf(output, "<table width=100%% height=%d cellpadding=2 cellspacing=0 border=0 bgcolor=#FFFFFF border=0>\n",
			nodelett->height);

	fprintf(output, "<tr valign=top><td height=20 colspan=2>\n");
	fprintf(output, "<center><a href=\"site-prefs\">%s</a></center>\n", _("Site Preferences"));
	fprintf(output, "</td></tr>\n\n");

	fprintf(output, "<tr valign=top><td height=%d>\n", nodelett->height - 44);
	fprintf(output, "<a href=\"account-manager\">%s</a><br>\n", _("User Account Manager"));
	fprintf(output, "<a href=\"account-edit\">%s</a>\n", _("Add Account Entry"));
	fprintf(output, "</td><td>\n");
	fprintf(output, "<a href=\"group-manager\">%s</a><br>\n", _("Group Manager"));
	fprintf(output, "<a href=\"group-edit\">%s</a>\n", _("Add Group"));
	fprintf(output, "</td></tr>\n\n");
	fprintf(output, "<tr bgcolor=#CCCCCC><td height=24>\n");
	if (pendingaccts)
		fprintf(output, "<b><a href=\"acct-pending\">%d %s</a></b>\n",
				pendingaccts, _("Pending Guest Accounts"));
	else
		fprintf(output, "&nbsp;\n");
	fprintf(output, "</td><td height=24><div align=right>\n");

	count = aucDBNumRows("userinfo", NULL);
	fprintf(output, "<b>%d %s</b>\n", count, (count == 1) ? _("user") : _("users"));

	fprintf(output, ",\n");

	count = aucDBNumRows("groups", NULL);
	fprintf(output, "<b>%d %s</b>\n", count, (count == 1) ? _("group") : _("groups"));

	fprintf(output, "</div>\n");
	fprintf(output, "</td></tr></table>\n");

}

int
check_sample_conf(char *name)
{
	struct stat info;
	char buf[128];

	snprintf(buf, sizeof(buf), "%s/%s", aucEtcRoot, name);
	return stat(buf, &info);
}

int
copy_sample_conf(char * oldfile, char * newfile, int remove)
{
	FILE *outfile;
	struct stat info;
	char file[128], file2[128];

	snprintf(file, sizeof(file), "%s/%s", aucEtcRoot, newfile);
	snprintf(file2, sizeof(file2), "%s/%s", aucEtcRoot, oldfile);
	if (!stat(file, &info))
		return 0;

	aucSuRoot();
	outfile = fopen(file, "w");
	aucSuUser();

	if (!outfile)
		return -1;
	aucHtmlEchoFile(outfile, file2, 1);
	fclose(outfile);
	if (remove)
		unlink(file2);
	return 0;
}
