/*
 * Authenticated User Community
 * Copyright (c) 2000 David C. Moore
 *
 * You may distribute and/or use for any purpose modified or unmodified
 * copies of this software if you preserve the copyright notice above.
 *
 * mime.c
 *
 * Routines for use of MIME-types in AUC
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <imap-2001/c-client/mail.h>

#include "mime.h"

#define MIMETYPESFILE "mime.types"

struct mimepair
{
	char type[32], subtype[32], ext[8];
	struct mimepair *next;
};

void parsemimetypes(char *etcroot);

int parsed = 0;
struct mimepair *mimetypes;
struct mimepair *lasttype;

char mimetypebuf[256];

char *
aucMimeGetType(char *ext, char *etcroot)
{
	struct mimepair *ptr;

	if (!parsed)
		parsemimetypes(etcroot);

	for (ptr = mimetypes; ptr && strcasecmp(ptr->ext, ext); ptr = ptr->next);
	if (!ptr)
		return NULL;
	else {
		snprintf(mimetypebuf, sizeof(mimetypebuf), "%s/%s", ptr->type, ptr->subtype);
		return mimetypebuf;
	}
}

char *
aucMimeGetDefaultExt(char *type, char *subtype, char *etcroot)
{
	struct mimepair *ptr;
	if (!parsed)
		parsemimetypes(etcroot);

	for (ptr = mimetypes;
		 ptr && (strcasecmp(ptr->type, type) || strcasecmp(ptr->subtype, subtype));
		 ptr = ptr->next);

	if (!ptr)
		return NULL;
	else
		return ptr->ext;
}

char *
aucMimeInt2Type(unsigned short type)
{
	switch (type) {
	case TYPETEXT:
		return "text";
	case TYPEMULTIPART:
		return "multipart";
	case TYPEMESSAGE:
		return "message";
	case TYPEAPPLICATION:
		return "application";
	case TYPEAUDIO:
		return "audio";
	case TYPEIMAGE:
		return "image";
	case TYPEVIDEO:
		return "video";
	default:
		return "other";
	}
}

unsigned short
aucMimeType2Int(char *type)
{
	if (!strncmp(type, "text", 4))
		return TYPETEXT;
	if (!strncmp(type, "multipart", 9))
		return TYPEMULTIPART;
	if (!strncmp(type, "message", 7))
		return TYPEMESSAGE;
	if (!strncmp(type, "application", 11))
		return TYPEAPPLICATION;
	if (!strncmp(type, "audio", 5))
		return TYPEAUDIO;
	if (!strncmp(type, "image", 5))
		return TYPEIMAGE;
	if (!strncmp(type, "video", 5))
		return TYPEVIDEO;

	return TYPEOTHER;
}

char *
aucMimeType2Subtype(char *type)
{
	char *c;

	if (!type)
		return NULL;

	for (c = type; *c; c++)
		if (*c == '/')
			return c + 1;
	return c;
}

void
aucMimePrintType(FILE * output, unsigned short type)
{
	fprintf(output, "%s", aucMimeInt2Type(type));
}

void
parsemimetypes(char *etcroot)
{
	FILE *infile;
	char buffer[128], filename[512], typestr[32], subtype[32], ext[32];
	int len, t;

	snprintf(filename, sizeof(filename), "%s/%s", etcroot, MIMETYPESFILE);

	parsed = 1;
	mimetypes = NULL;
	lasttype = NULL;
	if ((infile = fopen(filename, "r"))) {
		while (fgets(buffer, sizeof(buffer), infile)) {
			if (2 <= sscanf(buffer, "%[^/]/%s %n", typestr, subtype, &len)) {
				while ((1 == sscanf(buffer + len, " %s %n", ext, &t)) && (t > 0)) {
					len += t;
					if (!mimetypes) {
						mimetypes = (struct mimepair *) malloc(sizeof(struct mimepair));
						lasttype = mimetypes;
					} else {
						lasttype->next = (struct mimepair *) malloc(sizeof(struct mimepair));
						lasttype = lasttype->next;
					}
					lasttype->next = NULL;
					snprintf(lasttype->type, sizeof(lasttype->type), "%s", typestr);
					snprintf(lasttype->subtype, sizeof(lasttype->subtype), "%s", subtype);
					snprintf(lasttype->ext, sizeof(lasttype->ext), "%s", ext);
				}
			}
		}
		fclose(infile);
	}
}
