/*
 * Authenticated User Community
 * Copyright (c) 2000 David C. Moore
 *
 * You may distribute and/or use for any purpose modified or unmodified
 * copies of this software if you preserve the copyright notice above.
 *
 * nodelett.c
 *
 * Routines for the display of nodeletts (panes) on every user's
 * start page
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <libxml/xpath.h>
#include <malloc.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>

#include <auc/cgic.h>
#include <auc/auc.h>
#include <auc/util.h>
#include <auc/conf.h>
#include <auc/auth.h>
#include <auc/registeracct.h>
#include <auc/html.h>

#include "nodelett.h"

/* struct nodelettinfo *nodeletts[64];
struct nodelettinfo **lastnodelett; */

aucNodelettListPtr nodeletts;

// struct nodelettinfo *getnodelettinfo(char *);
int drawnodelettrow(FILE *, aucNodelettListPtr);
void drawnodelettframe(FILE * output, int width, int height, aucNodelettInfoPtr nodelett,
					   char *arg);

int
aucNodelettInit(void)
{
	nodeletts = aucNodelettNewList();
	return 0;
}

aucNodelettListPtr
aucNodelettGetList(void)
{
	return nodeletts;
}

void
aucNodelettRemoveListItem(aucNodelettListPtr list, int index)
{
	memmove(list->info + index, list->info + index + 1,
			(list->len - index - 1) * sizeof(aucNodelettInfoPtr));
	free(list->arg[index]);
	memmove(list->arg + index, list->arg + index + 1, (list->len - index - 1) * sizeof(char *));
	list->len--;
	list->info = realloc(list->info, list->len * sizeof(aucNodelettInfoPtr));
	list->arg = realloc(list->arg, list->len * sizeof(char *));
}

void
aucNodelettInsertListItem(aucNodelettListPtr list, int index,
						  aucNodelettInfoPtr nodelett, char *arg)
{
	list->len++;
	list->info = realloc(list->info, list->len * sizeof(aucNodelettInfoPtr));
	list->arg = realloc(list->arg, list->len * sizeof(char *));
	memmove(list->info + index + 1, list->info + index,
			(list->len - index - 1) * sizeof(aucNodelettInfoPtr));
	memmove(list->arg + index + 1, list->arg + index, (list->len - index - 1) * sizeof(char *));
	list->info[index] = nodelett;
	list->arg[index] = arg;
}

int
comp_priority(aucNodelettInfoPtr * n1, aucNodelettInfoPtr * n2)
{
	aucNodelettInfoPtr node1 = *n1;
	aucNodelettInfoPtr node2 = *n2;

	return node1->priority - node2->priority;
}

aucNodelettListPtr
aucNodelettGetUserPrefList(char *uname)
{
	aucNodelettListPtr newlist;
	newlist = aucNodelettCopyList(nodeletts);
	qsort(newlist->info, newlist->len, sizeof(aucNodelettInfoPtr), (compareFunc) comp_priority);

	/* This is a horribly NASTY hack */
	if (!strcmp(aucUserCache->role, "parent")) {
		int x = 0;
		char **kids, **kid;
		aucNodelettInfoPtr sched;
		while (strcmp(newlist->info[x]->handle, "schedule")) {
			x++;
			if (x >= newlist->len)
				return newlist;
		}
		sched = newlist->info[x];
		aucNodelettRemoveListItem(newlist, x);
		if ((kids = aucRegGetParentSchedNodes(uname))) {
			for (kid = kids; *kid; kid++)
				aucNodelettInsertListItem(newlist, x, sched, *kid);
//          free(kids);
		}
	}

	return newlist;
}

int
aucNodelettVerifyPrefList(aucNodelettListPtr preflist, char *role)
{
	int retval = 0, x;
	char buf[256];

	snprintf(buf, sizeof(buf), "Roles/Role[@name=\"%s\"]", role);

	if (aucConfTokenExist(buf)) {
		for (x = 0; x < preflist->len;) {
			snprintf(buf, sizeof(buf),
					 "Roles/Role[@name='%s']/pane[@name='%s']",
					 role, preflist->info[x]->handle);
			if (aucConfTokenExist(buf) && aucAuthPermsof(preflist->info[x]->reqpriv))
				x++;
			else
				aucNodelettRemoveListItem(preflist, x);
		}
	} else
		retval = -1;

	return retval;
}

int
aucNodelettRegister(char *handle, char *title, aucNodelettDrawFunc drawfunc,
					int columns, int height, int priority, char *description, char *reqpriv,
					long unsigned int flags)
{
	aucNodelettInfoPtr info;
	nodeletts->len++;
	nodeletts->info =
		(aucNodelettInfoPtr *) realloc(nodeletts->info,
									   sizeof(aucNodelettInfoPtr) * nodeletts->len);
	nodeletts->arg = (char **) realloc(nodeletts->arg, sizeof(char *) * nodeletts->len);
	info = (aucNodelettInfoPtr) malloc(sizeof(aucNodelettInfo));

	info->handle = handle;
	info->title = title;
	info->drawfunc = drawfunc;
	info->columns = columns;
	info->height = height;
	info->priority = priority;
	info->description = description;
	info->reqpriv = reqpriv;
	info->flags = flags;

	nodeletts->info[nodeletts->len - 1] = info;
	nodeletts->arg[nodeletts->len - 1] = NULL;
	return 0;
}


aucNodelettListPtr
aucNodelettNewList(void)
{
	aucNodelettListPtr list;

	list = (aucNodelettListPtr) malloc(sizeof(aucNodelettList));
	list->info = NULL;
	list->arg = NULL;
	list->len = 0;
	return list;
}

aucNodelettListPtr
aucNodelettCopyList(aucNodelettListPtr list)
{
	aucNodelettListPtr newlist;
	int x;

	newlist = aucNodelettNewList();
	newlist->info = (aucNodelettInfoPtr *) malloc(sizeof(aucNodelettInfoPtr) * list->len);
	newlist->arg = (char **) malloc(sizeof(char *) * list->len);
	memcpy(newlist->info, list->info, sizeof(aucNodelettInfoPtr) * list->len);
	for (x = 0; x < list->len; x++) {
		if (list->arg[x])
			newlist->arg[x] = strdup(list->arg[x]);
		else
			newlist->arg[x] = NULL;
	}
	newlist->len = list->len;

	return newlist;
}

void
aucNodelettFreeList(aucNodelettListPtr list)
{
	int x;
	free(list->info);
	for (x = 0; x < list->len; x++)
		free(list->arg[x]);
	free(list->arg);
	free(list);
}

int
aucNodelettDrawList(FILE * output, aucNodelettListPtr list)
{
	aucNodelettListPtr newlist;

	newlist = aucNodelettCopyList(list);

	while (newlist->len)
		drawnodelettrow(output, newlist);
	aucNodelettFreeList(newlist);
	return 0;
}

int
drawnodelettrow(FILE * output, aucNodelettListPtr list)
{
	aucNodelettInfoPtr nodelett;
	aucNodelettInfoPtr queue[8];
	char *aqueue[8];
	int columnsleft, nodepercent, count = 0, x, height = 0, index = 0;

	columnsleft = aucNodeColumns;
	nodepercent = 100 / aucNodeColumns;

	while (columnsleft && (index < list->len)) {
		nodelett = list->info[index];
		if (nodelett->columns <= columnsleft) {
			queue[count] = nodelett;
			if (list->arg[index])
				aqueue[count] = strdup(list->arg[index]);
			else
				aqueue[count] = NULL;
			if (nodelett->height > height)
				height = nodelett->height;
			count++;
			columnsleft -= nodelett->columns;
//          aucError("QUEUE: %s", list->arg[index]);
			aucNodelettRemoveListItem(list, index);
//          aucError("columnsleft = %d", columnsleft);
		} else
			index++;
	}

	if (!count)
		return 0;

	fprintf(output, "<table cellpadding=2 cellspacing=0 border=0 width=%d><tr valign=top>\n", aucNodeColumns * 200);

	for (x = 0; x < count; x++) {
		nodelett = queue[x];
		nodelett->height = height;
		fprintf(output, "<td width=%d%%>\n\n\n", nodelett->columns * nodepercent);
		drawnodelettframe(output, (200 * (nodelett->columns)) - 4, height, nodelett, aqueue[x]);
		free(aqueue[x]);
		fprintf(output, "</td>\n");
	}

	if (columnsleft)
		fprintf(output, "<td width=%d%%>&nbsp;</td>\n", columnsleft * nodepercent);
	fprintf(output, "</tr></table>\n\n\n");
	return (count);
}

void
drawnodelettframe(FILE * output, int width, int height, aucNodelettInfoPtr nodelett, char *arg)
{
	char buf[512];
	int aucNodeBorderWidth = 2, aucNodeHeaderPadding = 2;
	char *aucNodeBorderColor = aucTrimColor, *aucNodeBgColor = "#FFFFFF";

	fprintf(output, "<table cellpadding=0 cellspacing=0 border=0 width=%d>\n", width);
	fprintf(output, "<tr valign=top><td colspan=5 width=%d height=%d bgcolor=%s>\n",
			width, aucNodeBorderWidth, aucNodeBorderColor);
	fprintf(output, "<img src=\"%s/transpix.gif\" width=1 height=%d alt=\"\"></td></tr>\n",
			aucGraphDir, aucNodeBorderWidth);

	fprintf(output, "<tr valign=top><td width=%d rowspan=4 bgcolor=%s>\n",
			aucNodeBorderWidth, aucNodeBorderColor);
	fprintf(output, "<img src=\"%s/transpix.gif\" width=%d height=1 alt=\"\"></td>\n",
			aucGraphDir, aucNodeBorderWidth);
	fprintf(output, "<td width=%d height=%d colspan=3 bgcolor=%s>\n",
			width - 2 * aucNodeBorderWidth, aucNodeHeaderPadding,
			(!strcmp(nodelett->reqpriv, "sysop")) ? "#000066" : aucTrimColor);
	fprintf(output, "<img src=\"%s/transpix.gif\" width=1 height=%d alt=\"\"></td>\n",
			aucGraphDir, aucNodeHeaderPadding);
	fprintf(output, "<td width=%d rowspan=4 bgcolor=%s>\n", aucNodeBorderWidth, aucNodeBorderColor);
	fprintf(output, "<img src=\"%s/transpix.gif\" width=%d height=1 alt=\"\"></td></tr>\n",
			aucGraphDir, aucNodeBorderWidth);

	fprintf(output, "<tr valign=top><td width=%d bgcolor=%s>\n",
			aucNodeHeaderPadding, (!strcmp(nodelett->reqpriv, "sysop")) ? "#000066" : aucTrimColor);
	fprintf(output, "<img src=\"%s/transpix.gif\" width=%d height=1 alt=\"\"></td>\n",
			aucGraphDir, aucNodeHeaderPadding);
	fprintf(output, "<td width=%d bgcolor=%s>\n",
			width - 2 * aucNodeBorderWidth - 2 * aucNodeHeaderPadding,
			(!strcmp(nodelett->reqpriv, "sysop")) ? "#000066" : aucTrimColor);

	/* This is a NASTY hack */
	if (!strcmp(nodelett->handle, "schedule") && arg) {
		aucAuthGetFullName(buf, sizeof(buf), arg, 0);
		fprintf(output, "<font color=#FFFFFF %s>%s <b>%s</font></b>\n",
				HT_SANSSERIF, aucSmallCaps(_("Courses for"), 0),
				aucSmallCaps(buf, 0));
	} else
		fprintf(output, "<b><font color=#FFFFFF %s>%s</font></b>\n",
				HT_SANSSERIF, aucSmallCaps(nodelett->title, 0));

	fprintf(output, "</td>\n");
	fprintf(output, "<td width=%d bgcolor=%s>\n",
			aucNodeHeaderPadding, (!strcmp(nodelett->reqpriv, "sysop")) ? "#000066" : aucTrimColor);
	fprintf(output, "<img src=\"%s/transpix.gif\" width=%d height=1 alt=\"\"></td></tr>\n",
			aucGraphDir, aucNodeHeaderPadding);

	fprintf(output, "<tr valign=top><td width=%d height=%d colspan=3 bgcolor=%s>\n",
			width - 2 * aucNodeBorderWidth, aucNodeHeaderPadding,
			(!strcmp(nodelett->reqpriv, "sysop")) ? "#000066" : aucTrimColor);
	fprintf(output, "<img src=\"%s/transpix.gif\" width=1 height=%d alt=\"\"></td></tr>\n",
			aucGraphDir, aucNodeHeaderPadding);

	fprintf(output, "<tr valign=top><td width=%d height=%d colspan=3 bgcolor=%s>\n",
			width - 2 * aucNodeBorderWidth, aucNodeHeaderPadding, aucNodeBgColor);
	(*(nodelett->drawfunc)) (output, nodelett, arg);
	fprintf(output, "</td></tr>\n");

	fprintf(output, "<tr valign=top><td colspan=5 width=%d height=%d bgcolor=%s>\n",
			width, aucNodeBorderWidth, aucNodeBorderColor);
	fprintf(output, "<img src=\"%s/transpix.gif\" width=1 height=%d alt=\"\"></td></tr>\n",
			aucGraphDir, aucNodeBorderWidth);
	fprintf(output, "</table>\n\n\n");

}

