/**
 * @file pages.h
 *
 * The mechanisms behind the "virtual" URL processing inside AUC.
 */ 

#ifndef PAGES_C
#define PAGES_C


#define PA_LOCAL      (0x1 <<1)

/** 
 * Declaration for functions used to generate a HTML page.
 */
typedef int (*aucPageDrawFunc) (FILE *, char *);


#define PAGE_DIR     (0x1 << 0)
#define PAGE_DIRFUNC (0x1 << 1)
#define PAGE_FILE    (0x1 << 2)


typedef struct _aucPage aucPage;
typedef aucPage * aucPagePtr;

/**
 * Structure that defines the properties of a "virtual" web page inside AUC.
 */

struct _aucPage
{
   char *name;
   long unsigned int type;
   aucPagePtr *pages;
   aucPagePtr *lastpage;
   aucPageDrawFunc drawfunc;
   char *defaultpage;
   char *reqpriv;
   long unsigned int flags; 
};



extern char * aucPagesRoot;

extern int aucPagesInit(void);

extern int aucPagesRegister(aucPagePtr parentdir, char *name,
				long unsigned int type, aucPageDrawFunc drawfunc, char *reqpriv,
				long unsigned int flags);
extern aucPagePtr aucPagesRegisterSubDir(aucPagePtr parentdir, char *name,
				  char *defaultpage, char *reqpriv, long unsigned int flags);
extern int aucPagesRun(FILE * output, char *path);


extern int aucPagePrefs(FILE * output, char *extrapath);
extern int aucPageLoginCheck(FILE * output, char *extrapath);
extern int aucPageLogoff(FILE * output, char *extrapath);
extern int aucPageStart(FILE * output, char *extrapath);
extern int aucPageAbout(FILE * output, char *extrapath);

#endif
