/*
 * Authenticated User Community
 * Copyright (c) 2000 David C. Moore
 *
 * You may distribute and/or use for any purpose modified or unmodified
 * copies of this software if you preserve the copyright notice above.
 *
 * plugin.c
 *
 * Routines for managing AUC plugins
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <dlfcn.h>
#include <dirent.h>
#include <malloc.h>
#include <string.h>

#include <auc/auc.h>
#include <auc/conf.h>
#include <auc/error.h>

#include "plugin.h"

static aucPluginInfoPtr *pluginlist = NULL;
static void **openlist = NULL;
static char **openconf = NULL;

char **
aucPluginConfList(void)
{
	return openconf;
}

int
aucPluginInitAll(char *path)
{
	char **names;
	char **p;
	int count = 0;
	char buf[256];

	if (openlist)
		return count;

	if (aucConfTokens("child::Plugins/Load", &names))
		return count;

	for (p = names; *p; p++) {
		void *handle;
		aucPluginInitFunc initfunc;
		aucPluginQueryFunc queryfunc;

		snprintf(buf, sizeof(buf), "%s/%s%s", path, *p, aucModExt);
		free(*p);
		handle = dlopen(buf, RTLD_LAZY);
		if (!handle) {
			aucError("%s", dlerror());
			continue;
		}

		openlist = realloc(openlist, (count + 2) * sizeof(void *));
		openlist[count] = handle;
		openconf = realloc(openconf, (3 * count + 4) * sizeof(char *));
		openconf[3 * count] = NULL;
		openconf[3 * count + 1] = NULL;
		openconf[3 * count + 2] = NULL;
		if ((queryfunc = dlsym(handle, "aucPluginQuery")) || (queryfunc = dlsym(handle, "_aucPluginQuery"))) {
			aucPluginInfoPtr info;
			if ((info = queryfunc())) {
				openconf[3 * count] = info->name;
				openconf[3 * count + 1] = info->confheading;
				openconf[3 * count + 2] = info->xmlconf;
			}
		}
		count++;
		openlist[count] = NULL;
		openconf[3 * count] = NULL;

		if ((initfunc = dlsym(handle, "aucPluginInit")) || (initfunc = dlsym(handle, "_aucPluginInit")))
			initfunc();
	}
	free(names);

	return count;
}

int
aucPluginCleanupAll(char *path)
{
	void **p;
	aucPluginInfoPtr *i;
	int count = 0;

	if (!openlist)
		return count;

	for (p = openlist; *p; p++) {
		aucPluginInitFunc cleanfunc;

		count++;
		if ((cleanfunc = dlsym(*p, "aucPluginCleanup")) || (cleanfunc = dlsym(*p, "_aucPluginCleanup")))
			cleanfunc();
		dlclose(*p);
	}
	free(openlist);
	free(openconf);

	if (!pluginlist)
		return count;

	for (i = pluginlist; *i; i++) {
		free((*i)->name);
		free((*i)->description);
		free((*i)->xmlconf);
		free((*i)->confheading);
		free(*i);
	}
	free(pluginlist);

	return count;
}

aucPluginInfoPtr *
aucPluginQueryAll(char *path)
{
	DIR *datadir;
	struct dirent *currfile;
	int count = 0;

	if (pluginlist)
		return pluginlist;

	datadir = opendir(path);
	if (!datadir)
		return NULL;

	while ((currfile = readdir(datadir))) {
		char *filename;
		int len;
		char buf[256];
		aucPluginQueryFunc query;
		void *handle;

		filename = currfile->d_name;
		len = strlen(filename);
		if ((len <= strlen(aucModExt))
			|| (strcmp(filename + len - strlen(aucModExt), aucModExt)))
			continue;

		snprintf(buf, sizeof(buf), "%s/%s", path, filename);
		handle = dlopen(buf, RTLD_LAZY);
		if (!handle)
			continue;

		if ((query = dlsym(handle, "aucPluginQuery")) || (query = dlsym(handle, "_aucPluginQuery"))) {
			aucPluginInfoPtr info;

			if ((info = query())) {
				pluginlist = realloc(pluginlist, (count + 2) * sizeof(aucPluginInfoPtr));
				pluginlist[count] = malloc(sizeof(aucPluginInfo));

				pluginlist[count]->name = NULL;
				if (info->name)
					pluginlist[count]->name = strdup(info->name);

				pluginlist[count]->description = NULL;
				if (info->description)
					pluginlist[count]->description = strdup(info->description);

				pluginlist[count]->xmlconf = NULL;
				if (info->xmlconf)
					pluginlist[count]->xmlconf = strdup(info->xmlconf);

				pluginlist[count]->confheading = NULL;
				if (info->confheading)
					pluginlist[count]->confheading = strdup(info->confheading);

				count++;
				pluginlist[count] = NULL;
			}

		}
		dlclose(handle);
	}
	closedir(datadir);
	return pluginlist;
}
