/*
 * Authenticated User Community
 * Copyright (c) 2000 David C. Moore
 *
 * You may distribute and/or use for any purpose modified or unmodified
 * copies of this software if you preserve the copyright notice above.
 *
 * registeracct.c
 *
 * Routines for guest account registration and approval
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <stdlib.h>

#include <auc/cgic.h>
#include <auc/auc.h>
#include <auc/email.h>
#include <auc/html.h>
#include <auc/database.h>
#include <auc/error.h>

#include "registeracct.h"

char *types[] = { NULL, NULL };

char randbuf[17];

char *randstring(int);

int
aucPageApproveAcct(FILE * output, char *extrapath)
{
	MYSQL_RES *res;
	MYSQL_ROW row;
	char *fields[16], buf[2048], pid[16], uname[65];
	int x, existing = 0, nogood = 0;

	if (types[0] == NULL)
		types[0] = _("parent");

	for (x = 0; x < 16; x++)
		fields[x] = NULL;

	strcpy(pid, "");
	strcpy(uname, "");
	cgiFormString("pidnum", pid, sizeof(pid));
	cgiFormString("username", uname, sizeof(uname));

	if (!strcmp(cgiRequestMethod, "POST")) {
		aucHtmlReadForm("pid fname lname mname type email streetadd city \
         stateprov zip phone students newuname olduname deny", fields, STR_ESCAPE);

		if (strlen(fields[14])) {
			aucDBQuery(db, "DELETE FROM registrations WHERE pid=%s", fields[0]);
			cgiHeaderLocation("acct-pending");
			return 0;
		}

		if (strlen(fields[13]))
			snprintf(uname, sizeof(uname), "%s", fields[13]);

		if (!strlen(fields[12]) && !strlen(fields[13]))
			nogood = -1;
		if (!strlen(fields[1]) || !strlen(fields[2]) || !strlen(fields[4]))
			nogood = -1;

		aucSanitizeUsername(fields[12]);

		/* make sure the username doesn't already exist if creating a new user */
		if (!nogood && strlen(fields[12])) {
			snprintf(buf, sizeof(buf), "SELECT COUNT(*) FROM userinfo WHERE \
            uname=\"%s\"", fields[12]);
			if (mysql_query(db, buf))
				aucError(mysql_error(db));
			else {
				res = mysql_store_result(db);
				if ((row = mysql_fetch_row(res)) && (atoi(row[0]) == 1))
					existing = nogood = 1;
				mysql_free_result(res);
			}
		}

		/* update or add the guest account information in the registration database */
		if (!nogood) {
			if (strlen(fields[0]))
				snprintf(buf, sizeof(buf), "UPDATE registrations SET fname=\"%s\", \
               lname=\"%s\", mname=\"%s\", type=\"%s\", email=\"%s\", \
               streetadd=\"%s\", city=\"%s\", stateprov=\"%s\", zip=\"%s\", \
               phone=\"%s\", students=\"%s\", uname=\"%s\" WHERE pid=%s", fields[1], fields[2], fields[3], fields[4], fields[5], fields[6], fields[7], fields[8], fields[9], fields[10], fields[11], (strlen(fields[12])) ? fields[12] : fields[13], fields[0]);
			else
				snprintf(buf, sizeof(buf), "INSERT INTO registrations \
               (fname, lname, mname, type, email, streetadd, city, stateprov, \
               zip, phone, students, uname) VALUES (\"%s\", \"%s\", \"%s\", \"%s\" \
               \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\")", fields[1], fields[2], fields[3], fields[4], fields[5], fields[6], fields[7], fields[8], fields[9], fields[10], fields[11], (strlen(fields[12])) ? fields[12] : fields[13]);

			if (mysql_query(db, buf)) {
				aucError(mysql_error(db));
				nogood = 1;
			}
		}

		/* add the user to the master user database or update old info */
		if (!nogood) {
			if (strlen(fields[12]))
				snprintf(buf, sizeof(buf), "INSERT INTO userinfo (uname, fname, \
               lname, mname, id, grp, permissions) VALUES (\"%s\", \
               \"%s\", \"%s\", \"%s\", \"\", \"%s\", \"normal\")", fields[12], fields[1], fields[2], fields[3], fields[4]);
			else
				snprintf(buf, sizeof(buf), "UPDATE userinfo \
               SET fname=\"%s\", lname=\"%s\", mname=\"%s\", grp=\"%s\" \
               WHERE uname=\"%s\"", fields[1], fields[2], fields[3], fields[4], fields[13]);
			if (mysql_query(db, buf)) {
				aucError(mysql_error(db));
				nogood = 1;
			}

			/* update the directory database */
			if (strlen(fields[12]))
				snprintf(buf, sizeof(buf), "INSERT INTO directory (uname, streetadd, \
               city, stateprov, zip, phone, email) VALUES (\"%s\", \"%s\", \
               \"%s\", \"%s\", \"%s\", \"%s\", \"%s\")", fields[12], fields[6], fields[7], fields[8], fields[9], fields[10], fields[5]);
			else
				snprintf(buf, sizeof(buf), "UPDATE directory \
               SET streetadd=\"%s\", city=\"%s\", stateprov=\"%s\", zip=\"%s\", \
               phone=\"%s\", email=\"%s\" WHERE uname=\"%s\"", fields[6], fields[7], fields[8], fields[9], fields[10], fields[5], fields[13]);
			if (mysql_query(db, buf)) {
				aucError(mysql_error(db));
				nogood = 1;
			}
		}

		if (!nogood && strlen(fields[12])) {
			char *password, from[64], to[64], text[2048];
			password = randstring(8);

			cgiStaticContentType("text/html");
			aucHtmlSlimHead("%s", _("New Account Approval"));

			/* generate a password and add it to the passwd database */
			fprintf(output, "<p>%s...\n", _("Generating Random Password"));
			snprintf(buf, sizeof(buf), "REPLACE INTO passwd (uname, passwd) \
            VALUES (\"%s\", PASSWORD(\"%s\"))", fields[12], password);
			if (mysql_query(db, buf)) {
				aucError(mysql_error(db));
				fprintf(output, "%s!</p>\n", _("failed"));
				aucHtmlSlimFoot();
				return -1;
			} else
				fprintf(output, "<i>%s</i></p>\n", _("Success"));

			/* email the user their login information */
			if (strlen(fields[5]) && strchr(fields[5], '@')) {
				fprintf(output, "<p>%s %s %s...\n",
						_("Emailing"), fields[5], _("with username and password"));
				snprintf(from, sizeof(from), "%s %s <%s>",
						 aucSiteName, _("Accounts"), aucWebmaster);
				snprintf(to, sizeof(to), "%s %s <%s>", fields[1], fields[2], fields[5]);
				snprintf(text, sizeof(text), _("\
Dear %s, \n\
\n\
Thanks for registering as a %s on %s.  Your account has\n\
been approved and you now have access to the password protected\n\
portions of the site.  Please use this information to log in\n\
for the first time:\n\
\n\
   Username: %s\n\
   Password: %s\n\
\n\
Type them exactly as they appear on the login page of %s.\n\
After you have logged in, please change your password to something\n\
easier to remember, using the \"User Profile\" link. If\n\
you have any questions, you may reply to this email or write to\n\
%s.  Thanks again for registering as a user.\n\
\n\
Regards,\n\
The Support Staff of %s\
"), fields[1], fields[4], aucSiteName, fields[12], password, aucSiteName, aucWebmaster, aucSiteName);
				aucEmailClQuickInject(output, aucWebmaster, from, to, "",
									  _("Your Account Information"), text);
				fprintf(output, "</p>\n");
			}

			/* return the comfirmation screen with new user information */
			fprintf(output, "<h2>%s</h2>\n\n", _("Account Created Successfully"));
			fprintf(output, "<p>%s <b>%s</b> %s <b>%s</b>.</p>\n", _("The account"), fields[12],
				_("has been created and the owner has been emailed with access \
instructions if an email address was provided.  For your reference, the password is"),
				password);
			fprintf(output, "<p><b><a href=\"acct-pending\">%s</a></b></p>\n\n",
					_("Return to the Registration Approval Page"));
			aucHtmlSlimFoot();
			return 0;
		}


	} else if (strlen(pid) || strlen(uname)) {
		if (strlen(pid))
			snprintf(buf, sizeof(buf), "SELECT pid, fname, lname, mname, type, \
            email, streetadd, city, stateprov, zip, phone, students FROM \
            registrations WHERE pid=%s", pid);
		else
			snprintf(buf, sizeof(buf), "SELECT pid, fname, lname, mname, type, \
            email, streetadd, city, stateprov, zip, phone, students FROM \
            registrations WHERE uname=\"%s\"", uname);
		if (mysql_query(db, buf)) {
			aucError(mysql_error(db));
			return -1;
		}
		res = mysql_store_result(db);
		if ((row = mysql_fetch_row(res))) {
			for (x = 0; x < 12; x++)
				fields[x] = (char *) strdup(row[x]);
		}
		mysql_free_result(res);
	}

	cgiStaticContentType("text/html");

	if (nogood)
		aucHtmlSlimHead("%s", _("Account Creation Error"));
	else if (strlen(pid))
		aucHtmlSlimHead("%s", _("Approve Account Registration"));
	else if (strlen(uname))
		aucHtmlSlimHead("%s", _("Update Visitor Account"));
	else
		aucHtmlSlimHead("%s", _("Add Visitor Account"));

	if (existing)
		fprintf(output, "<p><font color=#FF0000>%s <b>%s</b> %s</font></p>\n",
				_("The username"), fields[12],
				_("is already in use, and cannot be used for the new account.\n"
				  "Please choose another."));
	else if (nogood == -1)
		fprintf(output, "<p><font color=#FF0000>%s</font></p>\n",
				_("One of the required fields was not filled in.  Please\n"
				  "specify all relevant information."));
	else if (nogood) {
		fprintf(output, "<p><font color=#FF0000>%s</font></p>\n",
				_("An error occurred when updating the user information."));
	}

	if (!strlen(uname))
		fprintf(output, "<p>%s</p>\n\n",
				_("Any of the following information can be modified.\n"
				  "For any requests to be approved, a username must be\n"
				  "specified.  You will be warned if the desired username\n"
				  "has already been taken."));

	aucDrawTableForm(output, "POST", "acct-approve");
	aucDrawHidden(output, "pid", fields[0]);
	if (strlen(uname)) {
		fprintf(output, "<tr valign=top><td align=right>%s:</td>\n", _("Username"));
		fprintf(output, "<td><b>%s</b></td></tr>\n\n", uname);
		aucDrawHidden(output, "olduname", uname);
	}
	aucDrawTextbox(output, "fname", 20, 32, fields[1], _("First Name"));
	aucDrawTextbox(output, "lname", 20, 32, fields[2], _("Last Name"));
	aucDrawTextbox(output, "mname", 1, 1, fields[3], _("Middle Initial"));
	aucDrawDropdown(output, "type", types, types,
					(fields[4]) ? fields[4] : _("parent"), _("Relation"));
	aucDrawTextbox(output, "email", 35, 64, fields[5], _("Email Address"));
	aucDrawTextbox(output, "streetadd", 30, 64, fields[6], _("Address"));
	aucDrawTextbox(output, "city", 20, 64, fields[7], _("City"));
	aucDrawTextbox(output, "stateprov", 30, 30, fields[8], _("State/Province"));
	aucDrawTextbox(output, "zip", 10, 10, fields[9], _("Zip/Postal Code"));
	aucDrawTextbox(output, "phone", 12, 12, fields[10], _("Phone Number"));

	if (strlen(uname)) {
		aucDrawTextarea(output, "students", 3, 16, fields[11], _("Student Usernames"));
		aucDrawSubmit(output, _("Update Information"));
	} else {
		fprintf(output, "<tr valign=top><td>&nbsp;</td><td>%s\n",
				_("In the box below type the usernames of students whose course list this parent"
				  "should be able to see, one per line."));
		if (strlen(pid)) {
			if (fields[11] && strlen(fields[11])) {
				fprintf(output, "%s:<br>\n", _("This parent requested access to"));
				aucPrintText(fields[11], output, 0);
			}
			fprintf(output, "\n<a href=\"directory-search\">%s</a></td></tr>\n\n",
					_("Search Directory"));
			aucDrawTextarea(output, "students", 3, 16, "", _("Student Usernames"));
		} else
			aucDrawTextarea(output, "students", 3, 16, fields[11], _("Student Usernames"));
		aucDrawTextbox(output, "newuname", 16, aucMaxUsernameLen, fields[12],
					   _("New User's Username"));
		aucDrawSubmit(output, _("Create Account"));
		aucDrawSubmitName(output, _("deny"), _("Delete This Request"));
	}
	aucDrawTableFormEnd(output);

	aucHtmlSlimFoot();
	return 0;

}

int
aucPagePendingAcct(FILE * output, char *extrapath)
{
	char buf[1024];
	MYSQL_RES *res;
	MYSQL_ROW row;
	int count = 0;

	cgiHeaderContentType("text/html");
	aucHtmlSlimHead("%s", _("Registrations Pending Approval"));

	fprintf(output, "<p>%s:</p>\n\n", _("Select a name to view the details of the request"));

	fprintf(output, "<table width=300 cellpadding=2 cellspacing=1 border=0>\n");
	fprintf(output, "<tr bgcolor=#CCCCCC valign=top><td><b>%s</b></td></tr>\n\n",
		_("Requests"));

	snprintf(buf, sizeof(buf), "SELECT pid, fname, lname FROM registrations \
      WHERE uname=\"\"");
	if (mysql_query(db, buf))
		aucError(mysql_error(db));
	else {
		res = mysql_store_result(db);
		while ((row = mysql_fetch_row(res))) {
			count++;
			fprintf(output, "<tr valign=top bgcolor=#EEEEEE><td>\n");
			fprintf(output, "<a href=\"acct-approve?pidnum=%s\">%s %s</a>\n",
					row[0], row[1], row[2]);
			fprintf(output, "</td></tr>\n\n");
		}
		mysql_free_result(res);
	}

	if (!count)
		fprintf(output, "<tr><td><i>%s</i></td></tr>\n\n", _("No requests found"));

	fprintf(output, "</table>\n\n");

	aucHtmlSlimFoot();
	return 0;
}

int
aucPageRegisterAcct(FILE * output, char *extrapath)
{
	char buf[2048], *fields[16];
	int x, nogood = 0;

	if (types[0] == NULL)
		types[0] = _("parent");

	for (x = 0; x < 16; x++)
		fields[x] = NULL;

	if (!strcmp(cgiRequestMethod, "POST")) {
		aucHtmlReadForm
			("fname lname mname type email streetadd city stateprov zip phone students", fields,
			 STR_ESCAPE);
		if (strlen(fields[0]) && strlen(fields[1]) && strlen(fields[3]) && strlen(fields[4])
			&& strlen(fields[5]) && strlen(fields[6]) && strlen(fields[7]) && strlen(fields[8])
			&& strlen(fields[9])) {
			snprintf(buf, sizeof(buf), "INSERT INTO registrations \
            (fname, lname, mname, type, email, streetadd, city, stateprov, \
            zip, phone, students) VALUES (\"%s\", \"%s\", \"%s\", \"%s\", \
            \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\")", fields[0], fields[1], fields[2], fields[3], fields[4], fields[5], fields[6], fields[7], fields[8], fields[9], fields[10]);

			cgiHeaderContentType("text/html");
			aucHtmlSlimHead("%s", _("Account Request Submitted"));

			if (mysql_query(db, buf)) {
				fprintf(output, "<p>%s:</p>\n", _("There was an error submitting your request"));
				fprintf(output, "<p><b>%s</b></p>\n", mysql_error(db));
				fprintf(output, "<p>%s</p>\n\n", _("Please try again."));
			} else {
				aucGetFullPath("regsuccess-mess.html", aucDataDir, buf, sizeof(buf));
				aucHtmlEchoFile(output, buf, 0);
			}

			aucHtmlSlimFoot();
			return 0;
		} else
			nogood = 1;
	}

	cgiStaticContentType("text/html");
	aucHtmlSlimHead("%s", _("Register New Account"));

	if (!nogood) {
		aucGetFullPath("register-mess.html", aucDataDir, buf, sizeof(buf));
		aucHtmlEchoFile(output, buf, 0);
	} else {
		fprintf(output, "<p><b>%s</b>:  %s</p>\n\n",
				_("Error"), _("A required field was left blank."));
	}

	aucDrawTableForm(output, "POST", "register");

	aucDrawTextbox(output, "fname", 20, 32, fields[0], _("Your First Name*"));
	aucDrawTextbox(output, "lname", 20, 32, fields[1], _("Last Name*"));
	aucDrawTextbox(output, "mname", 1, 1, fields[2], _("Middle Initial"));
	aucDrawDropdown(output, "type", types, types,
					(fields[3]) ? fields[3] : _("parent"), _("I am a*"));
	aucDrawTextbox(output, "email", 35, 64, fields[4], _("Email Address*"));
	aucDrawTextbox(output, "streetadd", 30, 64, fields[5], _("Address*"));
	aucDrawTextbox(output, "city", 20, 64, fields[6], _("City*"));
	aucDrawTextbox(output, "stateprov", 30, 30, fields[7], _("State/Province*"));
	aucDrawTextbox(output, "zip", 10, 10, fields[8], _("Zip/Postal Code*"));
	aucDrawTextbox(output, "phone", 12, 12, fields[9], _("Phone Number*"));
	fprintf(output, "<tr valign=top><td>&nbsp;</td><td>\n");
	fprintf(output, "%s\n", _("The following option is only for users who selected \"I am a parent\" in the above option."));
	fprintf(output, "</td></tr>\n\n");
	aucDrawTextarea(output, "students", 3, 40, fields[10], _("Enter the full names \
      of any students in your family that attend this school"));
	aucDrawSubmit(output, _("Submit Account Request"));
	aucDrawTableFormEnd(output);

	fprintf(output, "<p>%s</p>\n\n", _("You will be emailed at the address provided \
above when your account is ready.  Please allow up to 2 days to receive confirmation."));

	aucHtmlSlimFoot();
	return 0;
}

char **
aucRegGetParentSchedNodes(char *user)
{
	MYSQL_RES *res;
	MYSQL_ROW row;
	char buf[1024], *a, *b, *name;
	char **str = NULL;
	int count = 0;

	snprintf(buf, sizeof(buf), "SELECT students FROM registrations \
      WHERE uname=\"%s\"", user);
	if (mysql_query(db, buf)) {
		aucError(mysql_error(db));
		return NULL;
	}
	res = mysql_store_result(db);
	if ((row = mysql_fetch_row(res))) {
		b = a = (char *) strdup(row[0]);
		while ((name = (char *) strtok(a, "\n\r \t,;"))) {
			snprintf(buf, sizeof(buf), "uname=\"%s\"", name);
			if (aucDBNumRows("userinfo", buf) == 0) {
				a = NULL;
				continue;
			}

			str = realloc(str, (count + 2) * sizeof(char *));
			str[count] = strdup(name);
			count++;
			str[count] = NULL;

			a = NULL;
		}
		free(b);
	}
	return str;

}

char *
randstring(int len)
{
	int c;
	char val;
	for (c = 0; c < len; c++) {
		val = rand() % 36;
		if (val <= 9)
			val += '0';
		else
			val += 'a' - 10;
		randbuf[c] = val;
	}
	randbuf[c] = '\0';
	return randbuf;
}
