/*
 * Authenticated User Community
 * Copyright (c) 2000 David C. Moore
 *
 * You may distribute and/or use for any purpose modified or unmodified
 * copies of this software if you preserve the copyright notice above.
 *
 * sock.c
 *
 * The AUC socket communication API
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <time.h>
#include <unistd.h>
#include <fcntl.h>
#include <malloc.h>
#include <string.h>
#include <errno.h>

#define AUTH_LENGTH 20

int
aucSockConnect(char *subdir, char *user)
{
	int sock;
	struct sockaddr_un servaddr;

	sock = socket(PF_UNIX, SOCK_STREAM, 0);
	if (sock == -1)
		return -1;

	servaddr.sun_family = AF_UNIX;
	snprintf(servaddr.sun_path, sizeof(servaddr.sun_path), "/tmp/%s/%s", subdir, user);

	if (connect(sock, (struct sockaddr *) &servaddr, sizeof(servaddr))) {
		close(sock);
		return -1;
	}
	return sock;
}

int
aucSockSendCommand(int sock, char *header, char *auth, int comm, char **args)
{
	char buf[AUTH_LENGTH + 1];
	int x;

	if (write(sock, header, strlen(header)) == -1)
		return -1;
	snprintf(buf, sizeof(buf), "%*.*s", AUTH_LENGTH, AUTH_LENGTH, auth);
	if (write(sock, buf, AUTH_LENGTH) == -1)
		return -1;
	if (write(sock, &comm, sizeof(int)) == -1)
		return -1;

	if (!args)
		x = 0;
	else
		for (x = 0; args[x]; x++);

	if (write(sock, &x, sizeof(int)) == -1)
		return -1;

	if (x) {
		for (x = 0; args[x]; x++) {
			int len = strlen(args[x]);
			if (write(sock, &len, sizeof(int)) == -1)
				return -1;
			if (write(sock, args[x], strlen(args[x])) == -1)
				return -1;
		}
	}

	return 0;
}

int
aucSockSendReply(int retval, FILE * output)
{
	if (fwrite(&retval, sizeof(int), 1, output) <= 0)
		return -1;
	return 0;
}

int
aucSockRecvReply(int sock)
{
	int retval = -1;

	read(sock, &retval, sizeof(int));
	return retval;
}

int
aucSockRecvCommand(int sock, char *header, char **auth, char ***args)
{
	char buf[256];
	int comm, nargs, x;

	if (read(sock, buf, strlen(header)) == -1)
		return -1;
	buf[strlen(header)] = '\0';
	if (strcmp(header, buf))
		return -1;

	if (read(sock, buf, AUTH_LENGTH) == -1)
		return -1;
	buf[AUTH_LENGTH] = '\0';
	if (*auth) {
		if (strcmp(*auth, buf))
			return -1;
	} else {
		*auth = malloc((AUTH_LENGTH + 1) * sizeof(char));
		strcpy(*auth, buf);
	}

	if (read(sock, &comm, sizeof(int)) == -1)
		return -1;

	if (read(sock, &nargs, sizeof(int)) == -1)
		return -1;

	if (!args)
		return comm;
	*args = NULL;

	for (x = 0; x < nargs; x++) {
		int len;
		char *arg;

		*args = realloc(*args, (x + 2) * sizeof(char *));
		if (read(sock, &len, sizeof(int)) == -1)
			return -1;
		arg = malloc(len + 1);
		if (read(sock, arg, len) == -1)
			return -1;
		arg[len] = '\0';
		*((*args) + x) = arg;
		*((*args) + x + 1) = NULL;
	}
	return comm;
}

int
aucSockRecvEcho(int sock, FILE * output)
{
	char buf[256];
	int len;

	while ((len = read(sock, buf, sizeof(buf)))) {
		if (len == -1)
			return -1;
		fprintf(output, "%.*s", len, buf);
	}
	return 0;
}

void
aucSockFreeArgs(char **args)
{
	int x;

	if (!args)
		return;

	for (x = 0; args[x]; x++)
		free(args[x]);
	free(args);
}
