/*
 * Authenticated User Community
 * Copyright (c) 2000 David C. Moore
 *
 * You may distribute and/or use for any purpose modified or unmodified
 * copies of this software if you preserve the copyright notice above.
 *
 * newspaper.c
 *
 * The school newspaper design and publication system for AUC
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <auc/cgic.h>
#include <auc/auc.h>
#include <auc/plugin.h>
#include <auc/conf.h>
#include <auc/pages.h>
#include <auc/files.h>
#include <auc/database.h>
#include <auc/html.h>
#include <auc/auth.h>
#include <auc/error.h>

#include "newspaper.h"

char *aucNewsDir = "/home/httpd/html/newspaper";
char *aucNewsUrl = "/news";
char *aucNewsName = "School Newspaper";
char *aucNewsEmail = "news@host.com";

static aucPluginInfo newspaper_info = {
	"newspaper",
	"Newspaper publishing engine",
	"newspaper.xml",
	"Newspaper Engine"
};

int staffmember(void);
int linktopstory(FILE *, char *, int);
int linkedition(FILE *, char *);
int createnewspaperdirectories(void);

aucPluginInfoPtr
aucPluginQuery(void)
{
	return &newspaper_info;
}

int
aucPluginInit(void)
{
	aucPagePtr dir;

	aucConfToken("child::Newspaper/Directory", &aucNewsDir);
	aucConfToken("child::Newspaper/Url", &aucNewsUrl);
	aucConfToken("child::Newspaper/Name", &aucNewsName);
	aucConfToken("child::Newspaper/Email", &aucNewsEmail);

	aucNodelettRegister("news",
						aucNewsName,
						aucNewsNodeFrame, 2, 180, 40,
						_("This is the news as published by the school newspaper."), "guest", 0);

	dir = aucPagesRegisterSubDir(NULL, "news", "admin", "normal", 0);
	aucPagesRegister(dir, "admin", PAGE_FILE, NewsAdmin, "normal", 0);
	aucPagesRegister(dir, "editarticle", PAGE_FILE, NewsEditArticle, "normal", 0);
	aucPagesRegister(dir, "newedition", PAGE_FILE, NewsCreateEdition, "normal", 0);
	aucPagesRegister(dir, "deledition", PAGE_FILE, NewsDeleteEdition, "normal", 0);
	aucPagesRegister(dir, "delarticle", PAGE_FILE, NewsDeleteArticle, "normal", 0);
	aucPagesRegister(dir, "publish", PAGE_FILE, NewsPublishEdition, "normal", 0);
	aucPagesRegister(dir, "editedition", PAGE_DIRFUNC, NewsEditEdition, "normal", 0);
	aucPagesRegister(dir, "files", PAGE_DIRFUNC, NewsFiles, "normal", 0);

	aucDBRegister("newspaper", "CREATE TABLE %s (issue DATE NOT NULL, name CHAR(16) NOT NULL, \
         author VARCHAR(64) NOT NULL, title VARCHAR(255) NOT NULL, \
         section VARCHAR(32) NOT NULL, image VARCHAR(64), \
         caption text NOT NULL, bodytext TEXT NOT NULL, \
         teaser text NOT NULL, priority INTEGER DEFAULT 99 NOT NULL, \
         PRIMARY KEY (issue, name))");
	aucDBRegister("newseditions",
				  "CREATE TABLE %s (issue DATE NOT NULL, status VARCHAR(32) NOT NULL, \
         PRIMARY KEY (issue))");

	aucRegisterFilePath(aucNewsDir, "news/", 5);

	if (!strcmp("/news/admin", cgiPathInfo))
		createnewspaperdirectories();

	return 0;
}

void
aucNewsNodeFrame(FILE * output, aucNodelettInfoPtr nodelett, char *arg)
{
	fprintf(output, "<table width=100%% height=%d cellpadding=2 cellspacing=0 border=0 bgcolor=#FFFFFF>\n",
			nodelett->height);
	fprintf(output, "<tr valign=top><td height=%d>\n", nodelett->height - 28);

	linktopstory(output, NULL, 1);


	fprintf(output, "</td></tr>\n\n");

	fprintf(output, "<tr bgcolor=#CCCCCC><td height=24>\n");

	fprintf(output, "<table border=0 width=100%% cellpadding=0 cellspacing=0>\n");
	fprintf(output, "<tr valign=top><td>\n");
	fprintf(output, "%s:\n<b>", _("Current Print Edition"));
	linkedition(output, NULL);
	fprintf(output, "</b></td>\n\n");

	if (staffmember()) {
		fprintf(output, "<td align=right><a href=\"news/admin\">%s...</a>\n", _("Staff Access"));
//      aucNewsDBIntegrityCheck(output);
		fprintf(output, "</td>\n");
	} else
		fprintf(output, "<td></td>\n");

	fprintf(output, "</tr></table></td>\n");
	fprintf(output, "</tr></form></table>\n\n");
}

int
NewsAdmin(FILE * output, char *extrapath)
{
	MYSQL_RES *res;
	MYSQL_ROW row;
	char buf[1024];
	int eds = 0;

	if (!staffmember())
		return -2;

	cgiHeaderContentType("text/html");
	snprintf(buf, sizeof(buf), _("%s Staff Access"), aucNewsName);
	aucHtmlSlimHead(buf);

	fprintf(output, "<p>%s</p>\n\n",
			_("From here, new newspaper editions can be created, and older editions \
may be modified.  To give others access to this staff-only area, \
add them to a group entitled \"newspaper\"."));

	fprintf(output, "<p><b><a href=\"newedition\">%s</a></b></p>\n\n", _("Create New Edition"));


	fprintf(output, "<table width=400 cellpadding=2 cellspacing=0 border=0>\n");
	fprintf(output, "<tr valign=top><td width=200><b><u>%s</u></b></td>\n", _("Edition Date"));
	fprintf(output, "<td width=200><b><u>%s</u></b></td></tr>\n", _("Status"));

	snprintf(buf, sizeof(buf), "SELECT DATE_FORMAT(issue, \"%s\"), issue, status FROM \
      newseditions ORDER BY issue", "%d %b %Y");
	if (mysql_query(db, buf))
		aucError(mysql_error(db));
	else {
		res = mysql_store_result(db);
		while ((row = mysql_fetch_row(res))) {
			eds++;
			fprintf(output,
					"<tr valign=top bgcolor=#EEEEEE><td><a href=\"editedition/%s\">%s</a></td>\n",
					row[1], row[0]);
			fprintf(output, "<td>%s</td></tr>\n\n", row[2]);
		}
		mysql_free_result(res);
	}

	if (!eds) {
		fprintf(output, "<tr bgcolor=#EEEEEE><td colspan=2><p><i>%s</i></p></td></tr>\n\n",
			_("No editions were found.  Please create one with the \"Create New Edition\" link above."));
	}
	fprintf(output, "</table>\n\n");

	aucHtmlSlimFoot();
	return 0;
}


int
NewsEditArticle(FILE * output, char *extrapath)
{
	MYSQL_RES *res;
	MYSQL_ROW row;
	char buf[8192];
	char *fields[32];
	int x, edit = 0, nogood = 0;
	char ssections[5][40];
	char *sections[] = { ssections[0], ssections[1], ssections[2], ssections[3],
		ssections[4], NULL
	};
	char sprioritydesc[5][40];
	char *prioritydesc[] = { sprioritydesc[0], sprioritydesc[1],
		sprioritydesc[2], sprioritydesc[3], sprioritydesc[4],
		NULL
	};
	char *priority[] = { "1", "2", "3", "4", "5", NULL };

	if (!staffmember())
		return -2;

	strcpy(ssections[0], _("Feature"));
	strcpy(ssections[1], _("Editorial"));
	strcpy(ssections[2], _("News"));
	strcpy(ssections[3], _("Sports"));
	strcpy(ssections[4], _("Column"));

	strcpy(sprioritydesc[0], _("1 (top story)"));
	strcpy(sprioritydesc[1], _("2"));
	strcpy(sprioritydesc[2], _("3"));
	strcpy(sprioritydesc[3], _("4"));
	strcpy(sprioritydesc[4], _("5 (lowest priority)"));

	for (x = 0; x < 32; x++)
		fields[x] = NULL;

	if (!strcmp(cgiRequestMethod, "GET")) {
		aucHtmlReadForm("name", &fields[10], STR_ESCAPE);
		aucHtmlReadForm("edition", fields, STR_ESCAPE);
		if (!fields[0] || !strlen(fields[0])) {
			cgiHeaderContentType("text/html");
			aucHtmlSlimHead("%s", _("Create New Article"));
			fprintf(output, "<p><b>%s:</b> %s</p>\n\n", _("Error"), _("No edition specified."));
			aucHtmlSlimFoot();
			return -1;
		}
		if (fields[10] && strlen(fields[10])) {
			snprintf(buf, sizeof(buf), "SELECT issue, title, name, author, section, \
            image, caption, priority, teaser, bodytext FROM newspaper \
            WHERE issue=\"%s\" AND name=\"%s\"", fields[0], fields[10]);
			if (mysql_query(db, buf))
				aucError(mysql_error(db));
			else {
				res = mysql_store_result(db);
				if ((row = mysql_fetch_row(res))) {
					edit = 1;
					for (x = 0; x < 10; x++)
						fields[x] = (char *) strdup(row[x]);
				}
				mysql_free_result(res);
			}
		}
	} else {
		edit = 1;
		aucHtmlReadForm
			("edition title name author section image caption priority teaser bodytext oldname",
			 fields, 0);
		if (!strlen(fields[0]) || !strlen(fields[1]) || !strlen(fields[2]))
			nogood = 1;
		if (!nogood && fields[10] && strlen(fields[10])) {
			snprintf(buf, sizeof(buf), "DELETE FROM newspaper WHERE issue=\"%s\" AND name=\"%s\"",
					 fields[0], fields[10]);
			if (mysql_query(db, buf))
				aucError(mysql_error(db));
		}
		if (!nogood) {
			aucHtmlReadForm
				("edition title name author section image caption priority teaser bodytext oldname",
				 fields, STR_ESCAPE);
			snprintf(buf, sizeof(buf), "INSERT INTO newspaper \
            (issue, title, name, author, section, image, caption, priority, teaser, bodytext) \
            VALUES (\"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", %s, \"%s\", \"%s\")", fields[0], fields[1], fields[2], fields[3], fields[4], fields[5], fields[6], fields[7], fields[8], fields[9]);
			if (mysql_query(db, buf))
				aucError(mysql_error(db));
			snprintf(buf, sizeof(buf), "editedition/%s", fields[0]);
			cgiHeaderLocation(buf);
			return 0;
		}
	}

	cgiHeaderContentType("text/html");

	if (edit)
		aucHtmlSlimHead("%s", _("Edit Article"));
	else
		aucHtmlSlimHead("%s", _("Add New Article"));

	if (nogood)
		fprintf(output, "<p><b>%s:</b> %s</p>\n\n", _("Error"), _("You must specify a headline and name."));

	aucDrawTableForm(output, "POST", "editarticle");

	aucDrawHidden(output, "edition", fields[0]);
	aucDrawTextbox(output, "title", 55, 255, fields[1], _("Headline"));
	fprintf(output, "<tr valign=top><td>&nbsp;</td><td>%s</td></tr>\n",
			_("The 'Anchor' should be a single word to uniquely identify this article from any other."));
	aucDrawTextbox(output, "name", 8, 16, fields[2], _("Anchor Name"));
	if (edit)
		aucDrawHidden(output, "oldname", fields[10]);
	aucDrawTextbox(output, "author", 30, 64, fields[3], _("Author(s)"));
	aucDrawDropdown(output, "section", sections, sections, fields[4], _("Section"));
	fprintf(output, "<tr valign=top><td>&nbsp;</td><td>%s</td></tr>\n",
			_("If an image file is specified, you should upload that file using the \"Upload/View Files\" option on the Edition Page."));
	aucDrawTextbox(output, "image", 30, 64, fields[5], _("Image filename (if any)"));
	aucDrawTextarea(output, "caption", 2, 55, fields[6], _("Image caption"));
	aucDrawDropdown(output, "priority", priority, prioritydesc, fields[7], _("Story Priority"));
	aucDrawTextarea(output, "teaser", 3, 55, fields[8], _("Teaser Text"));
	aucDrawTextarea(output, "bodytext", 20, 55, fields[9], _("Article Text"));

	if (edit)
		aucDrawSubmit(output, _("Save"));
	else
		aucDrawSubmit(output, _("Add"));

	aucDrawTableFormEnd(output);
	aucHtmlSlimFoot();
	return 0;
}

int
NewsCreateEdition(FILE * output, char *extrapath)
{
	char buf[1024];
	char *fields[32];
	int x;

	if (!staffmember())
		return -2;

	for (x = 0; x < 32; x++)
		fields[x] = NULL;

	if (!strcmp(cgiRequestMethod, "POST")) {
		aucHtmlReadForm("pubdate", fields, STR_ESCAPE);
		snprintf(buf, sizeof(buf), "INSERT INTO newseditions (issue, status) \
         VALUES (\"%s\", \"Unpublished\")", fields[0]);
		if (mysql_query(db, buf))
			aucError(mysql_error(db));

		cgiHeaderLocation("admin");
		return 0;
	}

	cgiHeaderContentType("text/html");
	aucHtmlSlimHead(_("Create New Edition"));
	aucDrawTableForm(output, "POST", "newedition");

	aucDrawDateselect(output, "pubdate", NULL, _("Publication Date"));

	aucDrawSubmit(output, _("Create"));
	aucDrawTableFormEnd(output);
	aucHtmlSlimFoot();
	return 0;
}

int
NewsEditEdition(FILE * output, char *extrapath)
{
	MYSQL_RES *res;
	MYSQL_ROW row;
	char buf[1024];
	int articles = 0, valid = 0, published = 0;

	if (!staffmember())
		return -2;

	cgiHeaderContentType("text/html");

	snprintf(buf, sizeof(buf), "SELECT DATE_FORMAT(issue, \"%s\"), status FROM \
      newseditions WHERE issue=\"%s\"", "%d %b %Y", extrapath);
	if (mysql_query(db, buf))
		aucError(mysql_error(db));
	else {
		res = mysql_store_result(db);
		if ((row = mysql_fetch_row(res))) {
			valid = 1;
			if (!strcasecmp(row[1], "published"))
				published = 1;
			snprintf(buf, sizeof(buf), "%s: %s", aucNewsName, row[0]);
			aucHtmlSlimHead(buf);
			fprintf(output, "<table width=100%%><tr><td>\n");
			fprintf(output, "<p>%s: <b>%s</b>\n", _("Edition Status"), row[1]);
			if (published) {
				fprintf(output, "[%s: ", _("View"));
				linkedition(output, extrapath);
				fprintf(output, "]\n");
			}
			fprintf(output, "</p></td>\n\n");
			fprintf(output, "<td align=right><p align=right>\n");
			fprintf(output, "<a href=\"../admin\">%s</a>\n", _("Back to Edition Index"));
			fprintf(output, "</td></tr></table>\n\n");
		}
		mysql_free_result(res);
	}

	if (!valid) {
		aucHtmlSlimHead("%s", _("Edition Not Found"));
		fprintf(output, "<p><b>%s:</b> %s %s %s</p>\n",
				_("Error"), _("The newspaper edition"), extrapath, _("was not found in the database."));
		aucHtmlSlimFoot();
		return -1;
	}

	fprintf(output, "<p>\n");
	if (!published)
		fprintf(output, "<a href=\"../publish?edition=%s\">%s</a>,\n",
				extrapath, _("Publish this edition"));
	else
		fprintf(output, "<a href=\"../publish?edition=%s\">%s</a>,\n",
				extrapath, _("Republish this edition"));
	fprintf(output, "<a href=\"../files/%s\">%s</a>,\n", extrapath, _("Upload/View Files"));
	fprintf(output, "<a href=\"../deledition?edition=%s\">%s</a></p>\n\n",
			extrapath, _("Delete this edition"));

	fprintf(output, "<p>\n");
	if (published) {
		fprintf(output, "%s\n\n",
			_("If any changes are made to the articles in this issue, click \
the \"republish\" link, above, to update the newspaper."));
	}
	fprintf(output, "%s</p>\n\n",
		_("Upload images or other necessary files for this issue using the \
\"Upload/View\" link above."));

	fprintf(output, "<table width=550 cellpadding=4 cellspacing=2 border=0>\n");
	fprintf(output, "<tr valign=top><td width=550><b><u>%s</u></b>\n", _("Articles"));
	fprintf(output, "<a href=\"../editarticle?edition=%s\">%s</a>\n", extrapath, _("Add New Article"));
	fprintf(output, "</td></tr>\n");

	snprintf(buf, sizeof(buf), "SELECT title, author, name, teaser, section FROM \
      newspaper WHERE issue=\"%s\" ORDER BY priority", extrapath);
	if (mysql_query(db, buf))
		aucError(mysql_error(db));
	else {
		res = mysql_store_result(db);
		while ((row = mysql_fetch_row(res))) {
			articles++;
			fprintf(output, "<tr valign=top bgcolor=#EEEEEE><td>\n");
			fprintf(output,
					"<dl><dt><font size=+1><a href=\"../editarticle?edition=%s&name=%s\">%s</a>\n",
					extrapath, row[2], row[0]);
			fprintf(output, "[%s]</font>\n", row[4]);
			fprintf(output,
					"<font size=\"-1\"><a href=\"../delarticle?edition=%s&name=%s\">Delete article</a></font><br>\n",
					extrapath, row[2]);
			if (row[1] && strlen(row[1]))
				fprintf(output, "by <b>%s</b>", row[1]); 
			fprintf(output, "</dt>\n<dd>%s</dd></td></tr>\n\n", row[3]);
		}
		mysql_free_result(res);
	}

	if (!articles) {
		fprintf(output, "<tr bgcolor=#EEEEEE><td><p><i>%s</i></p></td></tr>\n\n",
			_("No articles were found.  Please create one with the \"Add New Article\" link above."));
	}
	fprintf(output, "</table>\n\n");

	aucHtmlSlimFoot();
	return 0;
}

int
NewsPublishEdition(FILE * output, char *extrapath)
{
	char buf[1024], commpath[256], *issue;
	FILE *commandout;
	int out = 0;

	if (!staffmember())
		return -2;

	cgiHeaderContentType("text/html");
	aucHtmlSlimHead("%s", _("Publishing Edition"));

	aucHtmlReadForm("edition", &issue, 0);
	snprintf(buf, sizeof(buf), "UPDATE newseditions SET status=\"Published\" \
      WHERE issue=\"%s\"", issue);
	if (mysql_query(db, buf)) {
		aucError(mysql_error(db));
		fprintf(output, "<p>%s</p>\n\n", _("Database query failed"));
	} else {
		mode_t oldmask;

		oldmask = umask(0022);
		aucGetFullPath("publishnews", aucDataDir, commpath, sizeof(commpath));
		snprintf(buf, sizeof(buf), "%s %s", commpath, issue);
		if ((commandout = popen(buf, "r"))) {
			fprintf(output, "<p><b>%s %s...</b><br><br>\n\n", _("Publishing issue"), issue);
			while (fgets(buf, sizeof(buf), commandout)) {
				out = 1;
				fprintf(output, "%s<br>", buf);
			}
			pclose(commandout);
			if (out)
				fprintf(output, "<br><b>%s</b></p>\n\n", _("Done!"));
			else
				fprintf(output, "<br><b>%s</b></p>\n\n", _("Error: Publishing script failed!"));
		}
		if (oldmask)
			umask(oldmask);
	}

	fprintf(output, "<p><b><a href=\"editedition/%s\">%s</a></b></p>\n\n",
			issue, _("Return to Article Index"));
	aucHtmlSlimFoot();
	return 0;
}

int
NewsDeleteArticle(FILE * output, char *extrapath)
{
	char issue[16], name[16], buf[1024], force[8];
	int nogood = 0;

	if (!staffmember())
		return -2;

	if (cgiFormSuccess != cgiFormString("edition", issue, sizeof(issue)))
		nogood = 1;
	if (cgiFormSuccess != cgiFormString("name", name, sizeof(name)))
		nogood = 1;

	if (nogood) {
		cgiHeaderLocation("admin");
		return -1;
	}

	if (cgiFormSuccess != cgiFormString("force", force, sizeof(force))) {
		cgiHeaderContentType("text/html");
		aucHtmlSlimHead("%s", _("Confirmation"));
		fprintf(output, "<p>%s <b>%s</b> %s <b>%s</b> %s</p>\n",
			_("Are you sure you want to delete article"), name, _("from the"), issue, _("edition?"));
		fprintf(output, "<ul><p><b><a href=\"delarticle?force=yes&edition=%s&name=%s\">%s</a></b></p>\n",
				issue, name, _("Yes"));
		fprintf(output, "<p><b><a href=\"editedition/%s\">%s</a></b></p></ul>\n", issue, _("No"));
		aucHtmlSlimFoot();
		return 0;
	}

	snprintf(buf, sizeof(buf), "DELETE FROM newspaper WHERE name=\"%s\" AND issue=\"%s\"", name,
			 issue);
	if (mysql_query(db, buf))
		aucError(mysql_error(db));

	snprintf(buf, sizeof(buf), "editedition/%s", issue);
	cgiHeaderLocation(buf);
	return 0;
}

int
NewsDeleteEdition(FILE * output, char *extrapath)
{
	char issue[16], buf[1024], force[8];
	FILE * commandout;
	int nogood = 0, out = 0;
	mode_t oldmask;

	if (!staffmember())
		return -2;

	if (cgiFormSuccess != cgiFormString("edition", issue, sizeof(issue)))
		nogood = 1;

	if (nogood) {
		cgiHeaderLocation("admin");
		return -1;
	}

	if (cgiFormSuccess != cgiFormString("force", force, sizeof(force))) {
		cgiHeaderContentType("text/html");
		aucHtmlSlimHead("%s", _("Confirmation"));
		fprintf(output, "<p>%s <b>%s</b> %s</p>\n",
			_("Are you sure you want to delete the"), issue, _("edition and all its articles?"));
		fprintf(output, "<ul><p><b><a href=\"deledition?force=yes&edition=%s\">%s</a></b></p>\n",
				issue, _("Yes"));
		fprintf(output, "<p><b><a href=\"editedition/%s\">%s</a></b></p></ul>\n", issue, _("No"));
		aucHtmlSlimFoot();
		return 0;
	}

	cgiHeaderContentType("text/html");
	aucHtmlSlimHead("%s", _("Deleting Edition"));

	snprintf(buf, sizeof(buf), "DELETE FROM newspaper WHERE issue=\"%s\"", issue);
	if (mysql_query(db, buf))
		aucError(mysql_error(db));
	snprintf(buf, sizeof(buf), "DELETE FROM newseditions WHERE issue=\"%s\"", issue);
	if (mysql_query(db, buf))
		aucError(mysql_error(db));

	snprintf(buf, sizeof(buf), "%s/%s", aucNewsDir, issue);
	aucSuRoot();
	aucFileRemoveDir(buf, 0);
	aucSuUser();

	oldmask = umask(0022);

	aucGetFullPath("publishnews", aucDataDir, buf, sizeof(buf));
	if ((commandout = popen(buf, "r"))) {
		fprintf(output, "<p><b>%s...</b><br><br>\n\n", _("Updating newspaper files"));
		while (fgets(buf, sizeof(buf), commandout)) {
			out = 1;
			fprintf(output, "%s<br>", buf);
		}
		pclose(commandout);
		if (out)
			fprintf(output, "<br><b>%s</b></p>\n\n", _("Done!"));
		else
			fprintf(output, "<br><b>%s</b></p>\n\n", _("Error: Publishing script failed!"));
	}
	umask(oldmask);

	fprintf(output, "<p><b><a href=\"admin\">%s</a></b></p>\n\n", _("Return to Edition Index"));
	aucHtmlSlimFoot();

	return 0;
}

int
NewsFiles(FILE * output, char *extrapath)
{
	char issue[64], buf[1024], *pathinfo = "";
	int x, foundpath = 0;

	int newshead(FILE * output, char *path, char *name)
	{
		char buf[1024];

		  snprintf(buf, sizeof(buf), _("File View: %s/%s"), issue, pathinfo);
		  aucHtmlSlimHead(buf);
		  fprintf(output, "<table width=100%%><tr><td align=right>\n");
		  fprintf(output, "<p align=right><a href=\"%s/news/editedition/%s\">%s</a></p>\n",
				  aucPagesRoot, issue, _("Back to Article Index"));
		  fprintf(output, "</td></tr></table>\n\n");
		  return 0;
	}
	int newsfoot(FILE * output, char *path, char *name)
	{
		aucHtmlSlimFoot();
		return 0;
	}

	if (!staffmember())
		  return -2;

	snprintf(issue, sizeof(issue), "%s", extrapath);
	for (x = 0; extrapath[x]; x++)
		if (issue[x] == '/') {
			foundpath = 1;
			issue[x] = '\0';
			pathinfo = &extrapath[x + 1];
			break;
		}

	if (!foundpath) {
		snprintf(buf, sizeof(buf), "%s/", issue);
		cgiHeaderLocation(buf);
		return 0;
	}

	snprintf(buf, sizeof(buf), "news/%s", issue);
	aucViewPath(output, newshead, newsfoot, issue, buf, pathinfo, TREE_FORCEWRITE, -1, 0);
	return 0;
}

int
createnewspaperdirectories(void)
{
	MYSQL_RES *res;
	MYSQL_ROW row;
	char buf[1024];

	if (mysql_query(db, "SELECT issue FROM newseditions"))
		aucError(mysql_error(db));
	else {
		res = mysql_store_result(db);
		while ((row = mysql_fetch_row(res))) {
			snprintf(buf, sizeof(buf), "%s/%s", aucNewsDir, row[0]);
			aucCheckDir(buf, 0755, 0, 0);
		}
		mysql_free_result(res);
	}
	return 0;
}

/*
void aucNewsDBIntegrityCheck(FILE * output)
{
   aucCheckTable(output, "newspaper");
   aucCheckTable(output, "newseditions");
}
*/

int
linktopstory(FILE * output, char *edition, int verbose)
{
	MYSQL_RES *res;
	MYSQL_ROW row;
	char buf[1024];
	int found = 0;

	if (edition)
		snprintf(buf, sizeof(buf), "SELECT title, author, name, teaser, issue FROM \
         newspaper LEFT OUTER JOIN newseditions ON newseditions.issue = newspaper.issue \
         WHERE issue=\"%s\" AND section <> \"%s\" ORDER BY priority LIMIT 2", edition, _("Sports"));
	else
		snprintf(buf, sizeof(buf), "SELECT title, author, name, teaser, newspaper.issue FROM \
         newspaper LEFT OUTER JOIN newseditions ON newseditions.issue = newspaper.issue \
         WHERE section <> \"%s\" AND newseditions.status <> \"Unpublished\" \
			ORDER BY issue DESC, priority LIMIT 2", _("Sports"));

//   fprintf(output, "<table width=380 border=0 cellpadding=2 cellspacing=0>\n");
//   fprintf(output, "<tr valign=top><td>\n");

	if (mysql_query(db, buf))
		aucError(mysql_error(db));
	else {
		res = mysql_store_result(db);
		while ((row = mysql_fetch_row(res))) {
			if (found)
				fprintf(output, "<font size=-2><br></font>\n");
			found = 1;
			fprintf(output, "<font size=+1>\n");
			fprintf(output, "<a target=\"_blank\" href=\"%s/%s/%s.html\">%s</a>\n",
					aucNewsUrl, row[4], row[2], row[0]);
			fprintf(output, "</font><font size=-1>");			
			if (row[1] && strlen(row[1]))
			         fprintf(output, "by <b>%s</b>", row[1]);

			fprintf(output, "<br>\n");
			if (row[3] && strlen(row[3]))
				fprintf(output, "&nbsp;&nbsp;%s<br>", row[3]);

			fprintf(output, "</font>\n\n");
		}
		mysql_free_result(res);
	}

	if (!found)
		fprintf(output, "<p><font size=+1><i>%s</i></font></p>\n\n", _("No Articles Found"));

/*   fprintf(output, "</td><td width=160 bgcolor=#EEEEEE>\n\n");
   
   fprintf(output, _("<center><b><font size=+1>S</font>PORTS</b></center>\n\n"));
   
   if (edition)
      snprintf(buf, sizeof(buf), _("SELECT title, author, name, teaser, issue FROM \
         newspaper LEFT OUTER JOIN newseditions ON newseditions.issue = newspaper.issue \
         WHERE issue=\"%s\" AND section=\"Sports\" ORDER BY priority LIMIT 1"), edition);
   else
      snprintf(buf, sizeof(buf), _("SELECT title, author, name, teaser, newspaper.issue FROM \
         newspaper LEFT OUTER JOIN newseditions ON newseditions.issue = newspaper.issue \
         WHERE section=\"Sports\" ORDER BY issue DESC, priority LIMIT 1"));
   
   if (mysql_query(db, buf))
      aucError(mysql_error(db));
   else {
      res = mysql_store_result(db);
      while (row = mysql_fetch_row(res)) {
         found = 1;
         fprintf(output, "<font size=+1>\n");
         fprintf(output, "<a href=\"%s/%s/%s.html\">%s</a>\n",
            aucNewsUrl, row[4], row[2], row[0]);
         fprintf(output, _("</font><br><font size=-1>by <b>%s</b><br>\n"), row[1]);
         fprintf(output, "&nbsp;&nbsp;%s</font><br>\n\n", row[3]);
      }
      mysql_free_result(res);
   }
   
   
   fprintf(output, "</td></tr></table>\n\n");  
   
   */
	return 1;
}

int
linkedition(FILE * output, char *edition)
{
	MYSQL_RES *res;
	MYSQL_ROW row;
	char buf[1024];
	int found = 0;

	if (edition)
		snprintf(buf, sizeof(buf), "SELECT DATE_FORMAT(issue, \"%s\"), issue FROM \
         newseditions WHERE issue=\"%s\"", "%d %b %Y", edition);
	else
		snprintf(buf, sizeof(buf), "SELECT DATE_FORMAT(issue, \"%s\"), issue FROM \
         newseditions WHERE status=\"Published\" ORDER BY issue DESC \
         LIMIT 1", "%d %b %Y");

	if (mysql_query(db, buf))
		aucError(mysql_error(db));
	else {
		res = mysql_store_result(db);
		if ((row = mysql_fetch_row(res))) {
			found = 1;
			fprintf(output, "<a target=\"_blank\" href=\"%s/%s/index.html\">%s</a>", aucNewsUrl, row[1],
					row[0]);
		}
		mysql_free_result(res);
	}

	if (!found)
		fprintf(output, "<i>%s</i>", _("none available"));
	return 1;
}


int
staffmember(void)
{
	if (aucAuthPermsof("sysop") || aucAuthInGroup("newspaper"))
		return 1;
	else
		return 0;
}
