/*
 * Authenticated User Community
 * Copyright (c) 2000 David C. Moore
 *
 * You may distribute and/or use for any purpose modified or unmodified
 * copies of this software if you preserve the copyright notice above.
 *
 * aucwho.c
 *
 * A small console utility to display a list of users currently
 * logged on to AUC
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <errno.h>
#include <time.h>
#include <string.h>
#include <dirent.h>
#include "aucwho.h"

int
main(int argc, char **argv)
{
	DIR *cachedir;
	FILE *infile;
	time_t now;
	struct tm *timebuf;
	aucAuthCache user;
	struct dirent *cachefile;
	char buf[1024];
	int count = 0;

//   aucCopyConfig();

	snprintf(buf, sizeof(buf), "%s/cache", aucVarRoot);
	if (!(cachedir = opendir(buf))) {
		fprintf(stderr, "Could not open cache dir: %s\n", (char *) strerror(errno));
		exit(1);
	}

	now = time(NULL);

	while ((cachefile = readdir(cachedir))) {
		if (cachefile->d_name[0] == '.')
			continue;
		sprintf(buf, "%s/cache/%s", aucVarRoot, cachefile->d_name);
		if ((infile = fopen(buf, "r"))) {
			fread(&user, sizeof(aucAuthCache), 1, infile);
			fclose(infile);
			timebuf = localtime(&user.login);
			strftime(buf, sizeof(buf), "%l:%M%p", timebuf);
			if ((now - user.last_request) <= 5000) {
				count++;
				if (count == 1)
					printf("%-16.16s  %-20.20s %12s %7s %6s\n", "User", "Full Name", "grp/yr",
						   "login@", "idle");
				printf("%-16.16s  %-20.20s ", user.uname, user.fullname);
				if (user.class)
					printf("%12d %7s ", user.class, buf);
				else
					printf("%12s %7s ", user.role, buf);
				if ((now - user.last_request) >= 60)
					printf("%6ld\n", (now - user.last_request) / 60);
				else
					printf("\n");
			}
		}
	}
	closedir(cachedir);

	if (!count)
		printf("No one logged on\n");
	return 0;
}
